<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\Session\Session;
use Teknisa\MAN\Utilities\Util;

use Teknisa\MAN\Services\NovoCodigo;


class PrevisaoEstoque extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $dataSourceName = 'vnd06200_previsaodeconsumo';
    protected $environment;
    protected $crudService;
    protected $novoCodigo;
    protected $PrevisaoEstoqueService;
    
    protected $parameterBagPrevisaoGrid = array('NRORG', 'DTPREVINI', 'DTPREVFIN', 'CDLOJA', 'CDFILIAL', 'TCDLOJA');
    
    public function __construct($crudService, $environment, $PrevisaoEstoqueService, $relCustoComposicaoProdutoService, Util $util, NovoCodigo $novoCodigo) {
        $this->crudService =$crudService;
        $this->environment = $environment;
        $this->relCustoComposicaoProdutoService = $relCustoComposicaoProdutoService;
        $this->PrevisaoEstoqueService = $PrevisaoEstoqueService;
        $this->util                     = $util;
        $this->novoCodigo               = $novoCodigo;
        $this->historicoComposicaoL = array();
        $this->historicoComposicaoP = array();
    }
    
    public function findPrev(DTO\Request $request, DTO\Response $response){
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '512M');
		set_time_limit(1800);
        $this->dataSourceName = 'vnd06200_previsaodeconsumo';
        $conditions = $this->util->getParamsAsArray($request);
        $conditions['DTPREVINI'] = $conditions['DTENTRVENDA'][0];
        $conditions['DTPREVFIN'] = $conditions['DTENTRVENDA'][1];
        	if(array_key_exists('CDLOJA', $conditions)) {
                     $conditions['TCDLOJA'] = $conditions['CDLOJA'] ? '': 'T';
				} else {
				    $conditions['TCDLOJA'] = 'T';
				}
        $likeableConditions = $this->util->getLikeConditions($request);
        foreach ($likeableConditions as $condition) {
				if(array_key_exists($condition['columnName'], $conditions)) {
					 unset($conditions[$condition['columnName']]);
				}
			}
        $newRequest = $this->util->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        $this->parameterBagColumns = array('NRORG', 'DTPREVINI', 'DTPREVFIN', 'CDLOJA', 'CDFILIAL', 'TCDLOJA');
        $newRequest = self::returnNewRequest($newRequest);
        parent::find($newRequest,$response);
        $rows = $response->getDataSets()[0]->getRows();
        foreach ($rows as &$row) {
            $dataIni = self::returnDataYMD($row['DTINIPCONPDV']);
            $dataFin = self::returnDataYMD($row['DTFINPCONPDV']);
            $intervalo = $dataIni->diff( $dataFin );
            $row['QTDEDIAS'] = $intervalo->days + 1;
        }
        $response->addDataSet( new DataSet('vnd06200_previsaodeconsumo', $rows ));
    }
    
    public function findProd(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'vnd06200_previsaodeconsumo_produtos';
        parent::find($request,$response);
    }
    
    public function grupprod(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'grupprod';
        parent::find($request,$response);
    }
    
    public function produto_nvsubgrprod(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'produto_nvsubgrprod';
        parent::find($request,$response);
    }
    
    public function subgrupprod(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'subgrupprod';
        parent::find($request,$response);
    }
    
    public function findMateriaPrima(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'vnd06200_previsaodeconsumo_materiaPrimaConsolidada';
        $conditions = $this->util->getParamsAsArray($request);
        $newRequest = self::returnNewRequest($request);
        $arrayNewResp = [];
        $newResponse = new DTO\Response();
        parent::find($newRequest,$response);
        foreach($response->getDataSets()[0]->getRows() as $prod){
            $prod['CDLOJA'] = $conditions['CDLOJA'];
            $prod['NMTIPOCUSTO'] = $this->PrevisaoEstoqueService->getTipoCusto($conditions);
            $prod['CUSTO'] = $this->relCustoComposicaoProdutoService->procedureCustoCompProd($prod, $prod);
            $prod['CUSTOTOT'] = $prod['QTPRODINFCOM'] * $prod['CUSTO'];
            $arrayNewResp[] = $prod;
        }
        $responde = $newResponse;
        $response->addDataSet(new DataSet($this->dataSourceName, $arrayNewResp));
    }
    
    private function returnNewRequest($request){
        $newConditions = [];
        $likeableConditions = $this->util->getLikeConditions($request);
        $conditions = $this->util->getParamsAsArray($request);
        unset($conditions['DTENTRVENDA']);
        unset($conditions['CDOPERADOR']);
        
        if(array_key_exists('CDLOJA', $conditions)) {
               $conditions['CDLOJA'] = str_replace('%', '', $conditions['CDLOJA']);
			} else {
				 $conditions['CDLOJA'] = '';
			}
        $conditions['TCDLOJA'] =  $conditions['CDLOJA'] ? NULL : 'T';
        
        foreach ($likeableConditions as $condition) {
			if(array_key_exists($condition['columnName'], $conditions) && $condition['columnName'] != 'CDLOJA') {
				 unset($conditions[$condition['columnName']]);
			}
			if(array_key_exists(('T'.$condition['columnName']), $conditions)){
			    $conditions['T'.$condition['columnName']] =  NULL;
			}
		}
        $newRequest = $this->util->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
		foreach ($likeableConditions as $condition) {
		    if($condition['columnName'] == 'CDARVPROD|NMPRODUTO|SGUNIDADE|QTPRODINFCOM|QTPRODCONPDV|QTPRODCONINF'){
                                $newRequest->getFilterCriteria()->addCondition('CDARVPROD|NMPRODUTO', $condition['operator'], $condition['value']);
		    }
        }
        return $newRequest; 
    }
    
    public function calculaPrevisaoReturn ($request){
        return null;
    }
    
    public function findPrevToTabbed(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'vnd06200_previsaodeconsumo_tab';
        parent::find($request,$response);
    }
    
    public function findMateriaPrimaProd(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'vnd06200_previsaodeconsumo_tabMatPrima';
        parent::find($request,$response);
    }
    
    public function saveQtdeCompra(DTO\Request $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        return $this->PrevisaoEstoqueService->saveQtdeCompra($rows);
    }
    
    private function setRow($parameters){
            $histMeta = $this->PrevisaoEstoqueService->getHistMeta($parameters);
            if($histMeta){
                $rows['GERAPERIODO']['start']   = $histMeta['DTGERAINI'];
                $rows['GERAPERIODO']['end']     = $histMeta['DTGERAFIN'];
                $rows['VENDASDIARIAS']['start'] = $histMeta['DTINICIO'];
                $rows['VENDASDIARIAS']['end']   = $histMeta['DTFINAL'];
                $rows['VERPERCENTUAL']          = $histMeta['VRPERCACRES'];
                $rows['CDOPERADOR']             = $histMeta['CDOPERADOR'];
                $rows['CDMETAPREV']             = $histMeta['CDMETAPREV'];
                $rows['CDFILIAL']               = $parameters['CDFILIAL'];
                $rows['NRORG']                  = $parameters['NRORG'];
                $rows['CDLOJA']                 = $parameters['CDLOJA'];
                return $rows;
            } 
    }
    
    public function calculaPrevisao(DTO\Request $request, DTO\Response $response){
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '512M');
		set_time_limit(1800);
        $rows = $request->getDataSet()->getRows()[0];
        $IDPREVCONSPDV = $this->PrevisaoEstoqueService->getIDPREVCONSPDV($rows)["IDPREVCONSPDV"];
        
        if($IDPREVCONSPDV == '4'){ //set row with parameters used in Meta
            $rows = self::setRow($rows);
            if(!$rows) {
                $response->addMessage(new Message("Não existe Meta pendente para esta Unidade", Message::TYPE_ERROR));
                return $response;
            }
            
        }
        
        $intervaloGP = self::returnIntervalDate($rows['GERAPERIODO']['start'], $rows['GERAPERIODO']['end']);
        $intervaloVD = self::returnIntervalDate($rows['VENDASDIARIAS']['start'], $rows['VENDASDIARIAS']['end']);
        $GERASTART = self::returnDataYMD($rows['GERAPERIODO']['start']);
        $GERAEND = self::returnDataYMD($rows['GERAPERIODO']['end']);
        $VENDASTART = self::returnDataYMD($rows['VENDASDIARIAS']['start']);
        $VENDAEND = self::returnDataYMD($rows['VENDASDIARIAS']['end']);
        $arrayDiaVD = self::calculaNumeroDiasDaSemanaWithWeek($rows, $intervaloVD, $rows['VENDASDIARIAS']['start']);
        if($IDPREVCONSPDV == '3'){
            $diasComAbertura = $this->PrevisaoEstoqueService->getDiasComAbertura($rows);
            $arrayDiasVendidos = self::getDiasVendidos($rows['VENDASDIARIAS']['start'], $rows['VENDASDIARIAS']['end']);
            if(count($diasComAbertura) != count($arrayDiasVendidos)){ //verifica se existe dias sem abertura de caixa
                $diff = array_diff($arrayDiasVendidos, $diasComAbertura);
                if(abs(count($arrayDiasVendidos) - count($diff)) < 21 ){
                    $response->addMessage(new Message("O período possui poucos dias com vendas.", Message::TYPE_ERROR));
                    return $response;
                } else {
                    foreach ($diff as $data) {
                        self::subArrayDiaVD($arrayDiaVD, $data);
                    }
                }
            }
        }
        
        if($GERASTART->format('Y-m-d') <= $VENDASTART->format('Y-m-d') || $GERAEND->format('Y-m-d') <= $VENDAEND->format('Y-m-d')){
            $response->addMessage(new Message("Vendas Diárias deve ser anterior ao de Geração!", Message::TYPE_ERROR));
                return $response;
        }
        if($IDPREVCONSPDV == '2'){
            if($intervaloGP != $intervaloVD){
                $response->addMessage(new Message("Vendas Diárias difere da quantidade de dias de Geração.", Message::TYPE_ERROR));
                return $response;
            }
        }
        if ($intervaloVD+2 <= 21 && $IDPREVCONSPDV != 1 && $IDPREVCONSPDV != 2){
            $response->addMessage(new Message("Selecione um período de vendas maior ou igual a 21 dias!", Message::TYPE_ERROR));
            return $response;
        } else {
            self::gerarPrevisao($rows, $IDPREVCONSPDV, ($intervaloVD+1), ($intervaloGP+1), $response ,$arrayDiaVD);
            if($IDPREVCONSPDV == '4'){
                $this->PrevisaoEstoqueService->updateStatusMetaPrevHist($rows);
            }
        }
            
    }
    
    public function getProdutos(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'ger51000_previsaodeconsumo_produtos';
        parent::find($request,$response);
    }
    
    private function returnIntervalDate($start, $end){
        $dataIni = self::returnData($start);
        $dataFin = self::returnData($end);
        return round((strtotime($dataFin)-strtotime($dataIni))/(60*60*24));
    }

    private function returnData ($date){
        return implode("-", array_reverse(explode("/", $date)));
    }
    
    private function returnDataYMD ($date){
         return new \DateTime(date("Y-m-d", strtotime(str_replace("/", "-", $date))));
    }
    
    public function gerarPrevisao($rows, $IDPREVCONSPDV, $intervaloVD, $intervaloGP, $response, $arrayDiaVD){
        $rows = self::checkMatPrima($rows);
        $data = self::getPesquisaVend($rows, $IDPREVCONSPDV, $intervaloVD, $intervaloGP, $arrayDiaVD);
        if(empty($data)){
            $response->addMessage(new Message("Não possui vendas para o período informado.", Message::TYPE_ERROR));
            return $response;
        } else {
            $NEWCODE = $this->novoCodigo->getCodeByTableName('NRSEQPREVPDV'.$rows['CDFILIAL'], $rows['NRORG']);
            
            $response->addDataSet(new DataSet('getTipoPrevisao', array('data'=>$data, 'rows'=>$rows, 'IDPREVCONSPDV'=>$IDPREVCONSPDV, 'NEWCODE'=>$NEWCODE)));
            return $response;
        }
    }
    
    public function checkMatPrima($rows){
        if(array_key_exists('CDPRODUTO', $rows) && gettype($rows['CDPRODUTO']) == 'array') {
            $prodFinal = $this->PrevisaoEstoqueService->findProdFinal($rows['CDPRODUTO'], $rows['NRORG'], $rows['CDFILIAL'], $rows['GERAPERIODO']['start']);
            ($prodFinal == null) ? 0 : $rows['CDPRODUTO'] = $prodFinal;
        } 
        return $rows;
    }
    
    public function insertPrevisao($request, $response){
        try{
            $this->PrevisaoEstoqueService->controlTransaction('begin');
            ini_set('max_execution_time', 1800);
        	ini_set('memory_limit', '512M');
        	set_time_limit(1800);
            $params = $request->getRow();
            $IDPREVCONSPDV  = $params['IDPREVCONSPDV'];
            $NEWCODE        = $params['NEWCODE'];
            $data           = $params['data'];
            $rows           = $params['rows'];
            $this->inserePrevisao($rows, $NEWCODE, $IDPREVCONSPDV, $data);
        } catch(\Exception $e) {
            $this->PrevisaoEstoqueService->controlTransaction('rollback');
            throw $e;
        } finally {
            $this->PrevisaoEstoqueService->controlTransaction('commit');
        }
    }
    
    public function insertProdutos($request, $response){
        try {
            ini_set('max_execution_time', 1800);
		    ini_set('memory_limit', '512M');
		    set_time_limit(1800);
            $this->PrevisaoEstoqueService->controlTransaction('begin');
            $params = $request->getRow();
            $IDPREVCONSPDV  = $params['IDPREVCONSPDV'];
            $NEWCODE        = $params['NEWCODE'];
            $data           = $params['data'];
            $rows           = $params['rows'];
            $prodArray = [];
            
            foreach($data as $produto) {
                if(array_key_exists($produto['CDPRODUTO'],$prodArray)){
                    $prodArray[$produto['CDPRODUTO']] = $produto['QTDCONSUMIDA'] + $prodArray[$produto['CDPRODUTO']];
                } else {
                    $prodArray[$produto['CDPRODUTO']] = $produto['QTDCONSUMIDA'];
                }
            } 
        
            $adqPrevisaoArre = $this->PrevisaoEstoqueService->adqPrevisaoArre($rows, $NEWCODE);
          
            foreach ($adqPrevisaoArre as $previsao) {
                if(array_key_exists($previsao["CDPRODUTO"], $prodArray)){
                    $this->PrevisaoEstoqueService->updateQtde($rows, $NEWCODE, $previsao, $prodArray[$previsao["CDPRODUTO"]]);
                }
            }
        
            if($IDPREVCONSPDV == '4'){
                $this->PrevisaoEstoqueService->insPrevProdHr($rows, $NEWCODE);
            }
            $response->addNotification(new Notification("Operação realizada com sucesso!"));
        } catch(\Exception $e) {
            $this->PrevisaoEstoqueService->controlTransaction('rollback');
            throw $e;
        } finally {
            $this->PrevisaoEstoqueService->controlTransaction('commit');
        }
        
    }
    
    private function getPesquisaVend($rows, $IDPREVCONSPDV, $intervaloVD, $intervaloGP, $arrayDiaVD){
        //DATAINI ate DATAFIN = VENDASDIARIAS
        $QtdDias = 0;
        $cdsMediaVendas = [];
        $DIAESCP = $this->PrevisaoEstoqueService->getDiaEspecial($rows);
        $percentualGeral = !!$rows['VERPERCENTUAL'] ? 1 + (floatval(str_replace('.',',',($rows['VERPERCENTUAL']))) / 100) : 1;
        
        if($IDPREVCONSPDV == '0') { //MEDIA ARITMETICA COM DESVIO PADRAO DAS VENDAS DIÁRIAS
            $produtos = $this->PrevisaoEstoqueService->adqProdComVendaMediaDesvio($rows);
            $DTANT = self::returnDataYMD($rows['GERAPERIODO']['start']);
            $arrayDia = self::calculaNumeroDiasDaSemana($rows, $intervaloVD, $rows['VENDASDIARIAS']['start']);
            $geraEnd = self::returnDataYMD($rows['GERAPERIODO']['end']);
            foreach ($produtos as $prod) {
                $desvio = $this->PrevisaoEstoqueService->adqDesvioPadrao($rows, $prod['CDPRODUTO'], $prod['NRDIASEMANA']+1);
                $diasDesvio = 0;
                if($desvio['QTPRODDIASEMANA'] >= 3){
                    self::subDesvio($prod['QTCOMPRARSUG'], $desvio);
                    self::subDesvio($prod['QTCOMPRAR'], $desvio);
                    self::subDesvio($prod['QTCONSUMO'], $desvio);
                    $diasDesvio = 2;
                }
                for($QtdDias; $DTANT->format('Y-m-d') <= $geraEnd->format('Y-m-d'); $QtdDias++){
                    $newProd = $prod;
                    $data = self::returnDataYMD($rows['GERAPERIODO']['start']);
                    $newProd['DTENTRVENDA'] = date('d/m/Y', strtotime($data->format('d-m-Y')."+".$QtdDias." days"));
                    $diaSemanaDTENTRVENDA = date('w', strtotime(str_replace("/", "-", $newProd['DTENTRVENDA'])));
                    $newProd['NRDIASEMANA'] = $diaSemanaDTENTRVENDA;
                    $percentualDiaEsp = array_key_exists($newProd['DTENTRVENDA'],$DIAESCP) ? (($DIAESCP[$newProd['DTENTRVENDA']]/100)) : 0;
                    $DiaEsp_Meta = $percentualGeral + $percentualDiaEsp;
                    if($prod['NRDIASEMANA'] == $diaSemanaDTENTRVENDA){
                        $newProd['QTDCONSUMIDA']   =  $prod['QTCONSUMO'] / ($arrayDia[$diaSemanaDTENTRVENDA] - $diasDesvio);
                        $newProd['QTCONSUMO']      = ($prod['QTCONSUMO'] / ($arrayDia[$diaSemanaDTENTRVENDA] - $diasDesvio));
                        $newProd['QTCOMPRARSUG']   = ($prod['QTCOMPRARSUG'] / ($arrayDia[$diaSemanaDTENTRVENDA] - $diasDesvio)) * $DiaEsp_Meta;
                        $newProd['QTCOMPRAR']      = ($prod['QTCOMPRAR'] / ($arrayDia[$diaSemanaDTENTRVENDA] - $diasDesvio)) * $DiaEsp_Meta;
                        $cdsMediaVendas[] = $newProd;
                    } 
                    $DTANT = self::returnDataYMD(date('d/m/Y', strtotime($DTANT->format('d-m-Y')."+1 days"))); 
                }
                $QtdDias = 0;
                $DTANT = self::returnDataYMD($rows['GERAPERIODO']['start']);
            }
            
            return $cdsMediaVendas;
        }
        else if($IDPREVCONSPDV == '1') { //MEDIA ARITMETICA DAS VENDAS DIÁRIAS 
            $produtos = $this->PrevisaoEstoqueService->adqProdComVendaMedia($rows);
            $data = self::returnDataYMD($rows['GERAPERIODO']['start']);
            if(count($produtos) > 0){
                $DTANT = null;
                foreach ($produtos as $prod) {
                    for($i=0; $i<$intervaloGP; $i++){
                        $newProd = $prod;
                        $newProd['DTENTRVENDA'] = date('d/m/Y', strtotime($data->format('d-m-Y')."+".$i." days"));
                        $percentualDiaEsp = array_key_exists($newProd['DTENTRVENDA'],$DIAESCP) ? (($DIAESCP[$newProd['DTENTRVENDA']]/100)) : 0;
                        $DiaEsp_Meta = $percentualGeral + $percentualDiaEsp;
                        $newProd['QTDCONSUMIDA']   =  $prod['QTCONSUMO'] / $intervaloVD;
                        $newProd['QTCONSUMO']      =  $prod['QTCONSUMO'] / $intervaloVD;
                        $newProd['QTCOMPRARSUG']   =  $prod['QTCOMPRARSUG'] / $intervaloVD * $DiaEsp_Meta;
                        $newProd['QTCOMPRAR']      =  $prod['QTCOMPRAR'] / $intervaloVD * $DiaEsp_Meta;
                        $cdsMediaVendas[] = $newProd;
                    }
                }
            }
            return $cdsMediaVendas;
        }
        else if($IDPREVCONSPDV == '2'){ //PROPORÇÃO DAS VENDAS DIÁRIAS
            $produtos = $this->PrevisaoEstoqueService->adqProdComVenda($rows);
            if(count($produtos) > 0){
                $QtdDias = self::returnIntervalDate($produtos[0]['DTENTRVENDA'] , $rows['VENDASDIARIAS']['start']);
                $DTANT = null;
                foreach ($produtos as $prod) {
                    $prod['QTDCONSUMIDA']   =  $prod['QTCONSUMO'];
                    if(!isset($DTANT) || $DTANT != $prod['DTENTRVENDA']){
                        $QtdDias = self::returnIntervalDate($rows['VENDASDIARIAS']['start'], $prod['DTENTRVENDA']);
                    }
                    $data = self::returnDataYMD($rows['GERAPERIODO']['start']);
                    $prod['DTENTRVENDA'] = date('d/m/Y', strtotime($data->format('d-m-Y')."+".$QtdDias." days"));
                    $cdsMediaVendas[] = $prod;
                    $DTANT = $prod['DTENTRVENDA'];
                }
            }
            return $cdsMediaVendas;
        } 
        else if ($IDPREVCONSPDV == '3') { //SAZONALIDADE DAS VENDAS DIÁRIAS
            $produtos = $this->PrevisaoEstoqueService->adqProdComVendaSazonalidade($rows);
            $DTANT = self::returnDataYMD($rows['GERAPERIODO']['start']);
            $DTANTformat = $DTANT->format('Y-m-d');
            $geraEnd = self::returnDataYMD($rows['GERAPERIODO']['end'])->format('Y-m-d');
            $data = self::returnDataYMD($rows['GERAPERIODO']['start']);
            for($QtdDias; $DTANTformat <= $geraEnd; $QtdDias++){
                $diaAtual = date('d/m/Y', strtotime($data->format('d-m-Y')."+".$QtdDias." days"));
                $produ = self::returnProdToCalc($produtos, $diaAtual, $data);
                foreach ($produ as $prod) {
                        $newProd = $prod;
                        $newProd['DTENTRVENDA'] = $diaAtual;
                        $diaSemanaDTENTRVENDA = date('w', strtotime(str_replace("/", "-", $newProd['DTENTRVENDA'])));
                        $percentualDiaEsp = array_key_exists($newProd['DTENTRVENDA'],$DIAESCP) ? (($DIAESCP[$newProd['DTENTRVENDA']]/100)) : 0;
                        $DiaEsp_Meta = $percentualGeral + $percentualDiaEsp;
                            $newProd['QTDCONSUMIDA']   =  $prod['QTCONSUMO'];
                            if($arrayDiaVD[$prod['NRSEMANA']-1][$diaSemanaDTENTRVENDA] == 0){
                                $newProd['DIAS'] = 0;
                                $newProd['QTCONSUMO']      = 0;
                                $newProd['QTCOMPRARSUG']   = 0;
                                $newProd['QTCOMPRAR']      = 0;
                            } else {
                                $newProd['DIAS']  = $arrayDiaVD[$prod['NRSEMANA']-1][$diaSemanaDTENTRVENDA];
                                $newProd['QTCONSUMO']      = ($prod['QTCONSUMO'] / ($arrayDiaVD[$prod['NRSEMANA']-1][$diaSemanaDTENTRVENDA]));
                                $newProd['QTCOMPRARSUG']   = ($prod['QTCOMPRARSUG'] / ($arrayDiaVD[$prod['NRSEMANA']-1][$diaSemanaDTENTRVENDA])) * $DiaEsp_Meta;
                                $newProd['QTCOMPRAR']      = ($prod['QTCOMPRAR'] / ($arrayDiaVD[$prod['NRSEMANA']-1][$diaSemanaDTENTRVENDA])) * $DiaEsp_Meta;
                            }
                            $cdsMediaVendas[] = $newProd;
                }
                $DTANT = self::returnDataYMD(($DTANT->format('d-m-Y')."+1 days")); 
                $DTANTformat = $DTANT->format('Y-m-d');
                
            }
        } else { // SAZONALIDE COM HORARIO DAS VENDAS DIÁRIAS
            $produtos = $this->PrevisaoEstoqueService->adqProdComVendaSazonalidade($rows);
            $DTANT = self::returnDataYMD($rows['GERAPERIODO']['start']);
            $DTANTformat = $DTANT->format('Y-m-d');
            $geraEnd = self::returnDataYMD($rows['GERAPERIODO']['end'])->format('Y-m-d');
            $data = self::returnDataYMD($rows['GERAPERIODO']['start']);
            for($QtdDias; $DTANTformat <= $geraEnd; $QtdDias++){
                $diaAtual = date('d/m/Y', strtotime($data->format('d-m-Y')."+".$QtdDias." days"));
                $produ = self::returnProdToCalc($produtos, $diaAtual, $data);
                foreach ($produ as $prod) {
                    $newProd = $prod;
                    $newProd['DTENTRVENDA'] = $diaAtual;
                    $diaSemanaDTENTRVENDA = date('w', strtotime(str_replace("/", "-", $newProd['DTENTRVENDA'])));
                    if(array_key_exists($newProd['DTENTRVENDA'],$DIAESCP)){
                        $percentualDiaEsp = (($DIAESCP[$newProd['DTENTRVENDA']]/100));
                    }
                    $percentualDiaEsp = array_key_exists($newProd['DTENTRVENDA'],$DIAESCP) ? (($DIAESCP[$newProd['DTENTRVENDA']]/100)) : 0;
                    $DiaEsp_Meta = $percentualGeral + $percentualDiaEsp;
                        $newProd['QTDCONSUMIDA']   =  $prod['QTCONSUMO'];
                        if($arrayDiaVD[$prod['NRSEMANA']-1][$diaSemanaDTENTRVENDA] == 0){
                            $newProd['DIAS'] = 0;
                            $newProd['QTCONSUMO']      = 0;
                            $newProd['QTCOMPRARSUG']   = 0;
                            $newProd['QTCOMPRAR']      = 0;
                        } else {
                            $newProd['DIAS']  = $arrayDiaVD[$prod['NRSEMANA']-1][$diaSemanaDTENTRVENDA];
                            $newProd['QTCONSUMO']      = ($prod['QTCONSUMO'] / ($arrayDiaVD[$prod['NRSEMANA']-1][$diaSemanaDTENTRVENDA]));
                            $newProd['QTCOMPRARSUG']   = ($prod['QTCOMPRARSUG'] / ($arrayDiaVD[$prod['NRSEMANA']-1][$diaSemanaDTENTRVENDA])) * $DiaEsp_Meta;
                            $newProd['QTCOMPRAR']      = ($prod['QTCOMPRAR'] / ($arrayDiaVD[$prod['NRSEMANA']-1][$diaSemanaDTENTRVENDA])) * $DiaEsp_Meta;
                        }
                        $cdsMediaVendas[] = $newProd;
                }
                $DTANT = self::returnDataYMD(($DTANT->format('d-m-Y')."+1 days")); 
                $DTANTformat = $DTANT->format('Y-m-d');
                
            }
        }
        return $cdsMediaVendas;
    }
    
    private function returnProdToCalc($produtos, $diaAtual, $data){
        
        $diaSemanaDATADAVENDA = date('w', strtotime(str_replace("/", "-", $diaAtual)))+1 ;
        $semanaDTENTRVENDA = self::returnNrDiaSemana($diaAtual);
        $prod = array_filter($produtos, function($p) use ($diaSemanaDATADAVENDA, $semanaDTENTRVENDA, $diaAtual){
                if($p['NRSEMANA'] == $semanaDTENTRVENDA && $p['NRDIASEMANA'] == $diaSemanaDATADAVENDA){
                    return $p;
                }
            });
        return $prod;
    }
    

    private function returnProdPorcentHr($produtos, $diaAtual, $data, $prod, $prodTotal){
        
        $diaSemanaDATADAVENDA = date('w', strtotime(str_replace("/", "-", $diaAtual)))+1 ;
        $semanaDTENTRVENDA = self::returnNrDiaSemana($diaAtual);
        
        $prodTotal = array_filter($prodTotal, function($p) use ($diaSemanaDATADAVENDA, $semanaDTENTRVENDA, $diaAtual, $prod){
                if($p['NRSEMANA'] == $semanaDTENTRVENDA && $p['NRDIASEMANA'] == $diaSemanaDATADAVENDA && $prod['CDARVPROD'] == $p['CDARVPROD']){
                    return $p;
                }
            });
        
        $proporcao = 100 / array_values($prodTotal)[0]['QTCONSUMO'];
        
        $prod = array_filter($produtos, function($p) use ($diaSemanaDATADAVENDA, $semanaDTENTRVENDA, $diaAtual, $proporcao, $prod){
                if($p['NRSEMANA'] == $semanaDTENTRVENDA && $p['NRDIASEMANA'] == $diaSemanaDATADAVENDA && $prod['CDARVPROD'] == $p['CDARVPROD']){
                    return $p;
                }
            });
        foreach($prod as $key => $produto ){
            $prod[$key]['PERCENTUALHR'] =  $prod[$key]['QTCONSUMO'] * $proporcao;
        }
        return $prod;
    }
    
    private function getDiasVendidos($dtInicial, $dtFinal){
        $dateStart 		= self::returnDataYMD($dtInicial);
        $dateEnd 		= self::returnDataYMD($dtFinal);
        $dateRange = array();
        for($qtDia = 0; $dateStart <= $dateEnd; $qtDia++){
            $dateRange[] = $dateStart->format('d/m/Y');
            $dateStart = $dateStart->modify('+1day');
        }
        return $dateRange;
    }
    
    private function returnNrDiaSemana($data){
        $dia = intval(date('d', strtotime(str_replace("/", "-", $data))));
            switch ($dia) {
                case ($dia >= 1 && $dia <= 7);
                    return 1;
                    break;
                case ($dia > 7 && $dia <= 14);
                    return 2;
                    break;
                case ($dia > 14 && $dia <= 21);
                    return 3;
                    break;
                case ($dia > 21 && $dia <= 28);
                    return 4;
                    break;
                case ($dia > 28 && $dia <= 31);
                    return 5;
                    break;
                default:
                    return false;
                    break;
                }
    }
    
    private function subDesvio(&$quantidade, $desvio){
        $quantidade -= $desvio['MAXQTCONSUMO'];
        $quantidade -= $desvio['MINQTCONSUMO'];
    }
    
    private function inserePrevisao($rows, $NEWCODE, $IDPREVCONSPDV, $data){
        try{
            $this->PrevisaoEstoqueService->acmInsPrevisaoPdv($rows, $NEWCODE);
            foreach ($data as $produto) {
                if($produto['QTCOMPRAR'] > 0){
                    self::InsereProdPrev($produto['CDPRODUTO'], $produto['CDPRODUTO'], $IDPREVCONSPDV, $rows, $NEWCODE, true, $produto['QTCONSUMO'],$produto['QTCOMPRARSUG'],$produto['QTCOMPRAR'], true, $produto['DTENTRVENDA']);
                } else {
                    self::InsereProdPrev($produto['CDPRODUTO'], $produto['CDPRODUTO'], $IDPREVCONSPDV, $rows, $NEWCODE, true, $produto['QTCONSUMO'],$produto['QTCOMPRARSUG'],$produto['QTCOMPRAR'], true, $rows['GERAPERIODO']['start']);    
                }
            }
            
        } catch (\Exception $e) {
            $this->PrevisaoEstoqueService->controlTransaction('rollback');
            throw $e;
        }
    }
    
    private function InsereProdPrev($produto, $prodPai, $IDPREVCONSPDV, $rows, $NEWCODE, $boGravaPai, $doQtdeProd, $doQtdeSug, $doQtdeComp, $boConsDia, $dtConsumo){
        $produtoPai = null;
        $boProdRep  = null;
        $stSeqItPrevDia = null;
        $qtdeVend = 0;
        try {
            $stSeqItPrevCons = $this->PrevisaoEstoqueService->adqMaxNrSeqPrevCons($rows, $NEWCODE);
            // ----
                $adqVerifCompLocal = $this->PrevisaoEstoqueService->adqAuxProdLocal($rows, $produto);
                
            if(!empty($adqVerifCompLocal)){
                $produtoPai = $produto.$prodPai;
            } else if($produto == $prodPai) {
                $produtoPai = $produto;
            } else {
                $produtoPai = $produto.$prodPai;
            }
            
            
            // ----
            $adqPrevConDia = $this->PrevisaoEstoqueService->adqPrevConDia($rows, $produto, $produtoPai, $NEWCODE);
            
            if(empty($adqPrevConDia) || $adqPrevConDia['NRPRODCONPDV'] == ''){
                $stSeqItPrevDia = $stSeqItPrevCons;
                $boProdRep = false;
            } else {
                $stSeqItPrevDia = $adqPrevConDia['NRPRODCONPDV'];
                $boProdRep = true;
                $qtdeVend = $adqPrevConDia['QTPRODCONINF'];
            }
            
            // ----
            if(!empty($adqVerifCompLocal)){
                
                 self::PrevConsBD($boGravaPai,$boProdRep,$boConsDia, $rows, $produto, $produto, $NEWCODE, $stSeqItPrevCons, $stSeqItPrevDia,  $doQtdeProd, $doQtdeSug, $doQtdeComp, $adqVerifCompLocal, $IDPREVCONSPDV, $dtConsumo, $qtdeVend);
                
            } else {
                
                $adqVerifCompPadrao = $this->PrevisaoEstoqueService->adqAuxProdPadrao($rows, $produto);
                
                if(!empty($adqVerifCompPadrao)){
                    self::PrevConsBD($boGravaPai,$boProdRep,$boConsDia, $rows, $produto, $prodPai, $NEWCODE, $stSeqItPrevCons, $stSeqItPrevDia,  $doQtdeProd, $doQtdeSug, $doQtdeComp, $adqVerifCompPadrao, $IDPREVCONSPDV, $dtConsumo, $qtdeVend);
                } else {
                    if($produto != $prodPai){
                        if(!$boProdRep){
                            $this->PrevisaoEstoqueService->insertPrevConsPdv($rows, $produto, $prodPai, $NEWCODE, $stSeqItPrevCons,  $doQtdeProd, $doQtdeSug, $doQtdeComp);
                        } else {
                            $this->PrevisaoEstoqueService->updatePrevConsPai($rows, $produto, $prodPai, $NEWCODE, $doQtdeProd, $qtdeVend);
                        }
                    } else {
                        if($boGravaPai){
                            if(!$boProdRep){
                                $this->PrevisaoEstoqueService->insertPrevConsPdv($rows, $produto, null, $NEWCODE, $stSeqItPrevCons,  $doQtdeProd, $doQtdeSug, $doQtdeComp);
                            }
                           
                            if($boConsDia){
                                self::ConsDia($rows, $produto, null, $NEWCODE, $stSeqItPrevDia, $dtConsumo,  $doQtdeProd, $doQtdeSug, $doQtdeComp);
                            }
                        }
                    }
            
                }
            }
            
        } catch (\Exception $e) {
            $this->PrevisaoEstoqueService->controlTransaction('rollback');
            throw $e;
        }
    }
    
    private function PrevConsBD($boGravaPai,$boProdRep,$boConsDia, $rows, $produto, $prodPai, $NEWCODE, $stSeqItPrevCons, $stSeqItPrevDia,  $doQtdeProd, $doQtdeSug, $doQtdeComp, $adqVerifComp, $IDPREVCONSPDV, $dtConsumo, $qtdeVend){
        
        if($boGravaPai){
            if(!$boProdRep){ 
                $this->PrevisaoEstoqueService->insertPrevConsPdv($rows, $produto, $produto, $NEWCODE, $stSeqItPrevCons,  $doQtdeProd, $doQtdeSug, $doQtdeComp);
            } else {
                $this->PrevisaoEstoqueService->updatePrevConsPdv($rows, $produto, $NEWCODE, $doQtdeProd, $qtdeVend);
            }
            if($boConsDia){
                self::ConsDia($rows, $produto, $prodPai, $NEWCODE, $stSeqItPrevDia, $dtConsumo,  $doQtdeProd, $doQtdeSug, $doQtdeComp);
            }
        }
        
        foreach ($adqVerifComp as $composicao) {
            $doQtdeP = $doQtdeProd * $composicao['QTSUBPRODROT'];
            $doQtdeS = $doQtdeSug * $composicao['QTSUBPRODROT'];
            $doQtdeC = $doQtdeComp * $composicao['QTSUBPRODROT'];
            self::InsereProdPrev($composicao['CDSUBPRODUTO'], $produto, $IDPREVCONSPDV, $rows, $NEWCODE, $boGravaPai, $doQtdeP, $doQtdeS, $doQtdeC, $boConsDia, $dtConsumo );
        }
    }
    
    private function ConsDia ($rows, $produto, $prodPai, $NEWCODE, $stSeqItPrevDia, $dtConsumo,  $doQtdeProd, $doQtdeSug, $doQtdeComp){
        $adqDadosDiaRep = $this->PrevisaoEstoqueService->adqDadosDiaRep($rows, $produto, $NEWCODE, $stSeqItPrevDia, $dtConsumo);
            
        if(empty($adqDadosDiaRep)){
            $this->PrevisaoEstoqueService->insertPrevConsDia($rows, $produto, $prodPai, $NEWCODE, $stSeqItPrevDia, $doQtdeProd, $doQtdeSug, $doQtdeComp, $dtConsumo);
        } else {
            $this->PrevisaoEstoqueService->updatePrevConsDia($rows, $produto, $NEWCODE, $stSeqItPrevDia, $doQtdeProd, $adqDadosDiaRep['QTPRODCONPDVDIA'], $dtConsumo);
            
        }
    }
    
    private function calculaNumeroDiasDaSemana($rows, $intervalo, $DTINICIAL){
        $arrayDia = [0,0,0,0,0,0,0];
        $diasemana_numero = date('w', strtotime(str_replace("/", "-", $DTINICIAL)));
        for($numeroDias = 0; $numeroDias < $intervalo; $numeroDias++) {
            $arrayDia[$diasemana_numero]++;
            $diasemana_numero++;
            $diasemana_numero > 6 ? $diasemana_numero = 0 : 0;
        }
       return $arrayDia;
    }
    
    private function calculaNumeroDiasDaSemanaWithWeek($rows, $intervalo, $DTINICIAL){
        $arrayDia = [[0,0,0,0,0,0,0],[0,0,0,0,0,0,0],[0,0,0,0,0,0,0],[0,0,0,0,0,0,0],[0,0,0,0,0,0,0]];
        $diasemana_numero = date('w', strtotime(str_replace("/", "-", $DTINICIAL)));
        $numeroDeDiasMes = date('t', strtotime(str_replace("/", "-", $DTINICIAL)));
        $firstDayMonth = $diasemana_numero;
        $diaDoMes = intval(date('d', strtotime(str_replace("/", "-", $DTINICIAL))));
        for($numeroDias = 0; $numeroDias < $intervalo; $numeroDias++) {
            switch ($diaDoMes) {
                case ($diaDoMes >= 1 && $diaDoMes <= 7);
                    $arrayDia[0][$diasemana_numero]++;
                    break;
                case ($diaDoMes > 7 && $diaDoMes <= 14);
                    $arrayDia[1][$diasemana_numero]++;
                    break;
                case ($diaDoMes > 14 && $diaDoMes <= 21);
                    $arrayDia[2][$diasemana_numero]++;
                    break;
                case ($diaDoMes > 21 && $diaDoMes <= 28);
                    $arrayDia[3][$diasemana_numero]++;
                    break;
                case ($diaDoMes > 28 && $diaDoMes <= 31);
                    $arrayDia[4][$diasemana_numero]++;
                    break;
                default:
                    return false;
                    break;
                }
            $diasemana_numero++;
            $diasemana_numero > 6 ? $diasemana_numero = 0 : 0;
            if($diaDoMes < $numeroDeDiasMes){
                $diaDoMes++;
            } else {
                $diaDoMes = 1;
                $QtdDias = $numeroDeDiasMes;
                $DTINI = self::returnDataYMD($DTINICIAL);
                $DTINICIAL = date('d/m/Y', strtotime($DTINI->format('d-m-Y')."+".$QtdDias." days"));
                $numeroDeDiasMes = date('t', strtotime(str_replace("/", "-", $DTINICIAL)));
            }
        }
       return $arrayDia;
    }
    
    private function subArrayDiaVD(&$arrayDiaVD, $data){
        $diasemana_numero = date('w', strtotime(str_replace("/", "-", $data)));
        $diaDoMes = intval(date('d', strtotime(str_replace("/", "-", $data))));
        switch ($diaDoMes) {
                case ($diaDoMes >= 1 && $diaDoMes <= 7);
                    $arrayDiaVD[0][$diasemana_numero]--;
                    break;
                case ($diaDoMes > 7 && $diaDoMes <= 14);
                    $arrayDiaVD[1][$diasemana_numero]--;
                    break;
                case ($diaDoMes > 14 && $diaDoMes <= 21);
                    $arrayDiaVD[2][$diasemana_numero]--;
                    break;
                case ($diaDoMes > 21 && $diaDoMes <= 28);
                    $arrayDiaVD[3][$diasemana_numero]--;
                    break;
                case ($diaDoMes > 28 && $diaDoMes <= 31);
                    $arrayDiaVD[4][$diasemana_numero]--;
                    break;
                default:
                    return false;
                    break;
                }
    }
    
    public function deletePrevisao (DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $arrayProdAprov = $this->PrevisaoEstoqueService->checkProdAprov($row['CDFILIAL'], $row['NRSEQPREVPDV'], $row['NRORG']);
        if(empty($arrayProdAprov)){
            
            $this->PrevisaoEstoqueService->delPrevisao($row['CDFILIAL'], $row['NRSEQPREVPDV'], $row['NRORG']);
            $response->addNotification(new Notification("Previsão excluída com sucesso."));
             
        } else {
            $response->addMessage(new Message("A previsão de consumo não pode ser excluída pois existem produtos já aprovados.", Message::TYPE_WARNING));
        }
        return $response;
    }
    
    public function getTipoPrevisao ($request, $response){
        $row = $request->getRow();
        $data =  $this->PrevisaoEstoqueService->getIDPREVCONSPDV($row)["NMPREVCONSPDV"];
        $response->addDataSet(new DataSet('getTipoPrevisao', array($data)));
            
    }
}