<?php
namespace Teknisa\MAN\Services;


use \Util\Exception;

use Teknisa\MAN\Utilities\Resposta; 
use NFePHP\NFe\Common\Standardize;
use Teknisa\Models\NFe;
use NFePHP\Common\Certificate;
use NFePHP\Common\Soap\SoapCurl;
use NFePHP\NFe\Make;
use \Datetime;
use Teknisa\MAN\Factories\TransmissaoNFCE;
use Teknisa\MAN\Controllers\TransmissaoNfceNewController;

/*use Teknisa\MAN\Utilities\ToolsNFe;
use Teknisa\MAN\Utilities\Utilities;
use Teknisa\MAN\Utilities\Crypt; */

use NFePHP\NFe\Tools;

use Teknisa\Libs\Util\Utilities as Util;

class TransmissaoNfceNewService {
    
    const AMB_EMISSAO_ONLINE = '1';
    const AMB_EMISSAO_OFFLINE = '9';
    const MODELO_DOC = '65'; // 55 - NFe; 65 - NFC-e
    const VERSAO_LAYOUT = '4.00';
	const ORACLE = 'oci8';
	
    const APROVADA = 'O';
    const ENTRADA = 'E';
    const SAIDA = 'S';
    const PRODUTO_NACIONAL = 'N';
    const VERSAO_APLICACAO = 'Odhen Retail';
    const DEFAULT_XML = '
        <NFe xmlns="http://www.portalfiscal.inf.br/nfe">
            
        </NFe>';
        
    const MSG_CERTIFICADO = "Verificar se o certificado foi parametrizado em Parâmetros da Unidade => Frente de Caixa => Definições NFC-e/SAT e Empresa => Inscrição Estadual";
    
    public $entityManager;
    private $util, $notaFiscal, $nfeTools, $DTEMISSAO;

   	public function __construct(\Doctrine\ORM\EntityManager $entityManager, TransmissaoNFCE $notaFiscal)  {
		$this->entityManager = $entityManager;
		$this->notaFiscal = $notaFiscal;
	}
	
	public function executarQuery($query, $params) {
	    $retorno = new Resposta();
	    $retorno->setParameters($params);
	    try {
	        $resultado = $this->entityManager->getConnection()->fetchAll($query, $params);
	        $retorno->setResult($resultado);
	        $retorno->setMessage("Query executada com sucesso. Linhas retornadas: ".$retorno->getLinhasResult());
	        $retorno->setStatus(Resposta::SUCESSO);
	    } catch (\Exception $e){
            $retorno->setMessage($this->exceptionMessage($e->getMessage()));
            $retorno->setError(true);
            $retorno->setStatus(Resposta::ERRO);
        }
        return $retorno;
	}
	
	public function runQuery($commit = true, $query, $params, $type = array()) {
		if($commit){
			$this->entityManager->beginTransaction();
		}
		try{
            $this->entityManager->getConnection()->executeQuery($query, $params, $type);
            if($commit){
				$this->entityManager->commit();
			}
            return true;
        }catch (\Exception $e){
        	if($commit){
				$this->entityManager->rollback();
			}
            throw $e;
        }
	}
	
	private function getRejeicaoConsulta(){
	    
		$arrayRejeicoes = [
			'203' => 'Emissor não habilitado para emissão de NF-e',
			'217' => 'NF-e não consta na base de dados da SEFAZ',
			'219' => 'Circulação da NF-e verificada',
			'220' => 'Prazo de Cancelamento superior ao previsto na Legislação',
			'221' => 'Confirmado o recebimento da NF-e pelo destinatário',
			'222' => 'Protocolo de Autorização de Uso difere do cadastrado',
			'240' => 'Cancelamento/Inutilização - Irregularidade Fiscal do Emitente',
			'241' => 'Rejeicao: Um numero da faixa ja foi utilizado',
			'304' => 'Pedido de Cancelamento para NF-e com evento da Suframa',
			'501' => 'Pedido de Cancelamento intempestivo (NF-e autorizada a mais de 7 dias)',
			'503' => 'Série utilizada fora da faixa permitida no SCAN (900-999)',
			'526' => 'Rejeição: Ano-Mês da Chave de Acesso com atraso superior a 6 meses em relação ao Ano-Mês atual',
			'539' => 'Duplicidade de NF-e com diferença na Chave de Acesso',
			'580' => 'O evento exige uma NF-e autorizada',
			'594' => 'O número de sequencia do evento informado é maior que o permitido',
			'613' => 'Codigo Numerico informado na Chave de Acesso difere do Codigo Numerico da NF-e',
			'656' => 'Consumo Indevido',
			'690' => 'Pedido de Cancelamento para NF-e com CT-e',
			'770' => 'NFC-e autorizada há mais de 24 horas',
			'910' => 'Chave de Acesso NF-e Substituta inválida', 
			'911' => 'Chave de Acesso NF-e Substituta incorreta', 
			'912' => 'NF-e Substituta inexistente',  
			'913' => 'NF-e Substituta Denegada ou Cancelada',  
			'914' => 'Data de emissão da NF-e Substituta maior que 2 horas da data de emissão da NF- e a ser cancelada', 
			'915' => 'Valor total da NF-e Substituta difere do valor da NF-e a ser cancelada', 
			'916' => 'Valor total do ICMS da NF-e Substituta difere do valor da NF-e a ser cancelada', 
			'917' => 'Identificação do destinatário da NF-e Substituta difere da identificação do destinatário da NF-e a ser cancelada', 
			'918' => 'Quantidade de itens da NF-e Substituta difere da quantidade de itens da NF-e a ser cancelada', 
			'919' => 'Item da NF-e Substituta difere do mesmo item da NF-e a ser cancelada', 
			'920' => 'Tipo de Emissão inválido no Cancelamento por Substituição'
			];
		return $arrayRejeicoes;
	}
	
	private function getSucessoConsulta(){
	    
		$arraySucessos = [
			'100' => 'Autorizado o uso da NF-e',
			'101' => 'Cancelamento homologado',
			'102' => 'Inutilizacao de numero homologado',
			'110' => 'Uso Denegado',
			'150' => 'Autorizado o uso da NF-e, autorização concedida fora do prazo',
			'155' => 'Cancelamento homologado fora de prazo',
			'135' => 'Evento registrado e vinculado a NF-e'
		];
		return $arraySucessos;
	}
	
	private function updateVendaStatusNfceCan($nota, $resultXML){
		if(!empty($resultXML)){
			$stNfeTools = new Standardize($resultXML);
			$resultXMLArray = $stNfeTools->toArray();
			if(!isset($resultXMLArray['retEvento'])){
				throw new \Exception("Não foi possível recuperar retorno do evento ao tentar realizar o cancelamento da nota!");
			}
			if(!isset($resultXMLArray['retEvento']['infEvento'])){
				throw new \Exception("Não foi possível recuperar informações do evento ao tentar realizar o cancelamento da nota!");
			}
		}else{
			throw new \Exception("Não foi possível recuperar informações ao tentar realizar o cancelamento da nota!");
		}
		
    	$result = array(
    		'error' => true,
    		'message' => ''
    	);
    	
    	$arrayRejeicoes = $this->getRejeicaoConsulta();
    	$arraySucessos = $this->getSucessoConsulta();
		
    	try {
    		$DSOBSSTATUSNFCE = null;
    		$DSOBSSTATUSNFCERET = null;
    		$NRPROTOCOLOCANC = null;
			$DTHRPROTOCOCANC = null;
			$STATUSCANCELA = 'A';
			
			$infEvento = $resultXMLArray['retEvento']['infEvento'];
			
    		if (!empty($infEvento)){
				//verifica se nota foi realmente cancelada através do status retornado
				if (!array_key_exists($infEvento['cStat'], $arrayRejeicoes) && array_key_exists($infEvento['cStat'], $arraySucessos)) { 
					$NRPROTOCOLOCANC = empty($infEvento['nProt']) ? ' ' : $infEvento['nProt'];
					$DTHRPROTOCOCANC = \DateTime::createFromFormat('Y-m-d\TH:i:sP', $infEvento['dhRegEvento']);
					$STATUSCANCELA = 'C';
					$DSOBSSTATUSNFCE = $infEvento['cStat'] . ' - ' . $infEvento['xMotivo'] . ' (NFC-e Cancelada na SEFAZ)';
					$DSOBSSTATUSNFCERET = $DSOBSSTATUSNFCE;
				}else{
					$statusRetorno = !empty($arrayRejeicoes[$infEvento['cStat']]) ? $arrayRejeicoes[$infEvento['cStat']] : 'Não definido na aplicação!';
					$DSOBSSTATUSNFCERET = 'Evento de Cancelamento: '.$infEvento['cStat'] . ' - ' . $statusRetorno;
					$DSOBSSTATUSNFCE = $nota['DSOBSSTATUSNFCE'];
				}
				
				$params = array(
	    			'CDFILIAL' => $nota['CDFILIAL'],
					'CDCAIXA' => $nota['CDCAIXA'],
					'NRSEQVENDA' => $nota['NRSEQVENDA'],
					'DSOBSSTATUSNFCE' => $DSOBSSTATUSNFCE,
					'NRPROTOCOLOCANC' => $NRPROTOCOLOCANC,
					'DTHRPROTOCOCANC' => $DTHRPROTOCOCANC,
					'DSRAZAOCANCNFCE' => $nota['JUSTIFICATIVA'],
					'DSARQXMLCANCNFCE' => $resultXML,
					'CDOPERADORCANC' => $nota['CDOPERADOR'],
					'IDSTATUSNFCE' => $STATUSCANCELA
	    		);
	    		
	    		$type = array(
	    			'DTHRPROTOCOCANC' => \Doctrine\DBAL\TypeS\Type::DATETIME
	    		);	
			
	    		if (!$this->databaseIsOracle()){
					$type['DSARQXMLCANCNFCE'] = \Doctrine\DBAL\Types\Type::BINARY;
				}
				
				TransmissaoNfceNewController::escreverLog("Atualizando informações da venda.");
				TransmissaoNfceNewController::escreverLog("Status Retorno: ". $DSOBSSTATUSNFCERET);
				
				//verifica se ha dados em ARQXMLNFCESAT
				$resultArqXML = $this->executarQuery(TransmissaoNFCE::SELECT_VENDA_ARQXMLNFCESAT, $params);
				if($resultArqXML->error()) {
		            throw new \Exception("Erro! ".$result->getMessage(), 1);
		        } else {
		        	//se nao tiver dados na table ira ARQXMLNFCESAT inserir, caso tenha apenas ira updatar
		            if($resultArqXML->getResult()[0]['VEND'] == 0){
		            	$this->runQuery(true, TransmissaoNFCE::INSERT_ARQXMLNFCESAT_STATUS_NFCE_CAN_SB, $params);
		            }else{
		        		$this->runQuery(true, TransmissaoNFCE::UPDATE_VENDA_STATUS_NFCE_CAN_SB, $params, $type);    	
		        		$this->runQuery(true, TransmissaoNFCE::UPDATE_ARQXMLNFCESAT_STATUS_NFCE_CAN_SB, $params, $type);    	
		            }
		            if($STATUSCANCELA == 'C'){
		            	$this->cancelaDadosVendaBanco($nota);
		            	//altera os dados gravados no xml com informacoes de protocolo no xml de cancelamento
						//quando nota cancelada o XML tem que receber as informacoes de protocolo para salvar certo
						$this->createProtocoloXMLCANC($params);
		            }
		        }
		        
	        	$result['error'] = false;
	        	$result['DSOBSSTATUSNFCE'] = $DSOBSSTATUSNFCE;
				
    		}else{
    			$result['error'] = true;
        		$result['DSOBSSTATUSNFCE'] = "Erro ao obter informações do evento!";
    		}
    		
    	} catch (\Exception $e) {
    		$result['message'] = $e->getMessage();
    	}

    	return $result;
    }
    
    public function getNfeTools($nota, $certif){
    	
    	if ($nota['IDTPAMBNFCE'] == '1'){
			$CSC =  isset($nota['CDCODSCONSPROD']) ? $nota['CDCODSCONSPROD'] : null;
			$CSCid = isset($nota['CDIDTOKENPROD']) ? $nota['CDIDTOKENPROD'] : null;
		} else {
			$CSC = isset($nota['CDCODSCONSHOMO']) ? $nota['CDCODSCONSHOMO'] : null;
			$CSCid = isset($nota['CDIDTOKENHOMO']) ? $nota['CDIDTOKENHOMO'] : null;
		}
    	
		$configContent = array(
			'atualizacao' => '2018-08-29 00:00:00',
      		'tpAmb' => intval($nota['IDTPAMBNFCE']),
      		'razaosocial' => $nota['NMRAZSOCFILI'],
      		'cnpj' => preg_replace('/[^0-9]/', '',  $nota['NRINSJURIEST']),
      		'ie' => $nota['CDINSCESTA'],
      		'siglaUF' => $nota['SGESTADO'],
      		'schemes' => 'PL_008i2',
      		'versao' => self::VERSAO_LAYOUT,
      		'tokenIBPT' => null,
      		'CSC' => $CSC,
      		'CSCid' => $CSCid,
      		// proxy setado aqui não funciona
      		'aProxyConf' => [
      		    'proxyIp' => null,
      		    'proxyPort' => null,
      		    'proxyUser' => null,
      		    'proxyPass' => null
      		]
		);
		
		try {
		    
		    $soap = new SoapCurl();
		    $certificado = Certificate::readPfx(file_get_contents($certif[0]["DSDIRCERTIFICADO"]), $certif[0]['DSSENHACERTNFCE']);
            // seta protocolo
            $soap->protocol($soap::SSL_TLSV1_2);
            
            $soap->loadCertificate($certificado);
           
            $nfeTools = new Tools(json_encode($configContent), $certificado);
			// seta modelo
			$nfeTools->model('65');
			
			return $nfeTools;
			
		} catch (\Exception $e) {
			throw new \Exception("Erro ao ler certificado, verifique data de validade e senha!".$this->exceptionMessage($e->getMessage()));
		}
	
	}
	
	private function getCertificado($nota){
		TransmissaoNfceNewController::escreverLog("");
		TransmissaoNfceNewController::escreverLog("Recuperando certificado...");
		$params[':CDINSCESTA'] = $nota['CDINSCESTA'];
		$params[':CDFILIAL'] = $nota['CDFILIAL'];
		
		$result = $this->executarQuery(TransmissaoNFCE::GET_LINK_CERTIFICADO_UNID, $params);
		
		if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            $result = $result->getResult();
            if (sizeof($result) > 0) {
            	if (empty($result[0]['CERTIFICADO'])) {
					$result = $this->executarQuery(TransmissaoNFCE::GET_LINK_CERTIFICADO, $params);
					if($result->error()) {
					    throw new \Exception("Erro! ".$result->getMessage(), 1);
					} else {
					    $result = $result->getResult();
					}
				}
            }
        }
        
		if (sizeof($result) > 0) {
			if (empty($result[0]['CERTIFICADO'])) {
				throw new \Exception('Erro, certificado não parametrizado. '.self::MSG_CERTIFICADO);
			}
			$link = $result[0]['DSDIRCERTIFICADO'];
			
			TransmissaoNfceNewController::escreverLog("Link do certificado: ".$link);
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $link);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_TIMEOUT, 5);

			TransmissaoNfceNewController::escreverLog("Estabelecendo conexão com o servidor...");
			$resultLink = curl_exec($ch);
            $err    = curl_errno($ch);
            $errmsg = curl_error($ch);
			curl_close($ch);
			
			if ($err){
                throw new \Exception("Erro ao obter conteúdo do Certificado Digital: " . $errmsg);
            } 
            
            if (strpos($resultLink, 'Not Found')) {
            	throw new \Exception("Erro 404. Não foi possível recuperar o certificado no servidor.". PHP_EOL . 
            	"Verificar se o Link do Servidor de Imagens encontra-se parametrizado em Paramêtros Gerais => Gerais => Geral. " . PHP_EOL . 
            	 self::MSG_CERTIFICADO. PHP_EOL . $errmsg);
            }
            
            TransmissaoNfceNewController::escreverLog("Certificado recuperado no servidor com sucesso.");
            TransmissaoNfceNewController::escreverLog("");
		} else {
			throw new \Exception('Não foi possível recuperar os dados de Inscrição Estadual parametrizada. ' . PHP_EOL . self::MSG_CERTIFICADO);
		}
		return $result;
	}
	
	private function getLogProcessNota($nota){
		TransmissaoNfceNewController::escreverLog("");
		TransmissaoNfceNewController::escreverLog("############ Processando Nota #############");
		if(isset($nota['NMFILIAL'])){
			$mensagemFilial = "Unidade/Filial: ".$nota['CDFILIAL']. ' - '. $nota['NMFILIAL'];
		}else{
			$mensagemFilial = "Unidade/Filial: ".$nota['CDFILIAL'];
		}
		TransmissaoNfceNewController::escreverLog($mensagemFilial);
		
		if(isset($nota['NMCAIXA'])){
			$mensagemCaixa = "Caixa: ".$nota['CDCAIXA']. ' - '. $nota['NMCAIXA'];
		}else{
			$mensagemCaixa = "Caixa: ".$nota['CDCAIXA'];
		}
		TransmissaoNfceNewController::escreverLog($mensagemCaixa);
		
		if(isset($nota['NRSEQVENDA'])){
			TransmissaoNfceNewController::escreverLog("Nº Venda: ".$nota['NRSEQVENDA']);
		}
		
		if(isset($nota['NRNOTAFISCALCE'])){
			TransmissaoNfceNewController::escreverLog("Nº Nota: ".$nota['NRNOTAFISCALCE']);
		}
	}
	
	public function getParamsVend($nota) {
        $parametros = array(
            ':CDFILIAL' => $nota['CDFILIAL'],
            ':NRORG'    => $nota['NRORG']
        );
        $result = $this->executarQuery(TransmissaoNFCE::ADQPARAVEND, $parametros);
        if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            $result = $result->getResult();
        }
        
        return $result;
    }
    
    public function cancelarNota($type, $nota){
    	
    	//seta informacoes iniciais no log
		$this->getLogProcessNota($nota);
		//le certificado
		$certificado = $this->getCertificado($nota);
		//config certificado
		$this->nfeTools = $this->getNfeTools($nota, $certificado);
		
		return $this->callCancela($type, $nota);
    }
    
    private function callCancela($tipo, $nota){
    	try {
    		
    		if($tipo == 'SB'){
    			$cancelamento = $this->nfeTools->sefazCancelaPorSubstituicao($nota['NRACESSONFCE'], $nota['JUSTIFICATIVA'], $nota['NRPROTOCOLONFCE'], $nota['NRACESSONFCE2'], self::VERSAO_APLICACAO);
    		}else if($tipo == 'N'){
    			$cancelamento = $this->nfeTools->sefazCancela($nota['NRACESSONFCE'], $nota['JUSTIFICATIVA'], $nota['NRPROTOCOLONFCE']);	
    		}
		    
		    if(!empty($cancelamento)){
				return $this->updateVendaStatusNfceCan($nota, $cancelamento);
		    }else{
		    	throw new \Exception("Não houve retorno ao tentar realizar o cancelamento de nota!");
		    }
			
		} catch (\Exception $e) {
			throw new \Exception("Erro ao tentar realizar o cancelamento de nota!");
		}
    	
    }
    
    private function callTransmitir($nota){
    	
		$return = array(
    		'error' => false,
    		'message' => ''
    	);
    	
    	try {
	        
	        $parametros = array(
	            ':CDFILIAL' => $nota['CDFILIAL'],
	            ':CDCAIXA' => $nota['CDCAIXA'],
	            ':CDLOJA' => $nota['CDLOJA'],
	            ':NRSEQVENDA'    => $nota['NRSEQVENDA']
	        );
	        
	        
	        /* Faz validacao produtos que vieram de pedidos da integracao e que estao sem imposto cadastrado na ALIQIMPFIS*/
	        $produtosItemSemImposto = $this->executarQuery(TransmissaoNFCE::ITENS_SEM_IMPOSTOS, $parametros);
	        if($produtosItemSemImposto->error()) {
	            throw new \Exception("Erro! ".$produtosItemSemImposto->getMessage(), 1);
	        } else {
	            if(!empty($produtosItemSemImposto->getResult())){
	        		$produtos = $produtosItemSemImposto->getResult();   
	        		if(!empty($produtos)){
	        			$textObsPrd = 'Produtos: ';
	        			foreach($produtos as $key => $value){
	        				$textObsPrd	.= $value['CDPRODUTO'].'-'.preg_replace('/[^A-Za-z0-9 \-]/', '', trim($value['NMPRODUTO'])).'; ';
	        			}
	        			$parametros['DSOBSSTATUSNFCE'] = $textObsPrd.' sem Imposto cadastrado.';
	        			$this->runQuery(true, TransmissaoNFCE::UPDATE_SEM_IMPOSTOS, $parametros);
	        			throw new \Exception("Erro! Há produto sem cadastro de imposto!");
	        		}
	            }
	        }
	        
	        $result = $this->executarQuery(TransmissaoNFCE::ADQTRANSMISSAONFCE, $parametros);
	        if($result->error()) {
	            throw new \Exception("Erro! ".$result->getMessage(), 1);
	        } else {
	            if(!empty($result->getResult())){
	               $resposta = $result->getResult();
	               $resposta[0]['CDOPERADOR'] = $nota['CDOPERADORLOCAL'];
		    	   $resposta[0]['TOTALVENDA'] = $nota['VRTOTALVENDA'];
		    	   $resposta[0]['DTABERTUR'] = $nota['DTABERTUR'];
	               $retornoXML = $this->montaXMLTransmitir($resposta);
	               if(!$retornoXML['error']){
	               	  $this->iniciaTransmissao($retornoXML, $nota);
	               }else{
	               		$return['error'] = true;
	               		$return['message'] = 'Erro ao montar XML';
	               }
	            }else{
	            	$return['error'] = true;
	               	$return['message'] = 'Erro ao buscar dados da Nota!';
	            }
	        }
	        return $return;
	        
		} catch (\Exception $e) {
			TransmissaoNfceNewController::escreverLog("Erro ao tentar realizar a transmissão da nota! ". $e->getMessage());
			throw new \Exception("Erro ao tentar realizar a transmissão da nota!");
		}
    	return $return;
    }
    
    private function cancelaDadosVendaBanco($nota){
    	
    	$this->entityManager->beginTransaction();
		try{
            
            $paramsVenda = array(
				':CDFILIAL'   => $nota['CDFILIAL'],
				':CDCAIXA'    => $nota['CDCAIXA'],
				':NRSEQVENDA' => $nota['NRSEQVENDA']
			);
			
			$paramsToACMCANCELAVENDA = $paramsVenda;
			$paramsToACMCANCELAVENDA[':MENSAGEM'] = $nota['JUSTIFICATIVA'];
	        
	        $this->runQuery(false, TransmissaoNFCE::ACMCANCELAVENDA, $paramsToACMCANCELAVENDA);    	
	        $this->runQuery(false, TransmissaoNFCE::ACMCANCELAMOVCAIXA, $paramsVenda);    
	        $this->runQuery(false, TransmissaoNFCE::ACMUPDULTMOVICLIE, $paramsVenda);    
	    	
	        $dadosMovCli =  $this->executarQuery(TransmissaoNFCE::ADQMOVICLIE, $paramsVenda);
	        if($dadosMovCli->error()) {
	            throw new \Exception("Erro! ".$dadosMovCli->getMessage(), 1);
	        } else {
	        	if(!empty($dadosMovCli->getResult())){
	        		$result = $dadosMovCli->getResult()[0];
	        		$paramsToACMUPDSALDOMOVICLIE = array(
						':NRSEQVENDA' => $nota['NRSEQVENDA'],
						':CDCONSUMIDOR' => $result['CDCONSUMIDOR'],
						':NRSEQMOVCLI' => $result['NRSEQMOVCLI'],
						':CDCLIENTE' => $result['CDCLIENTE'],
						':VRSALDOCONS' => $result['VRSALDOCONS']
					);
					$this->runQuery(false, TransmissaoNFCE::ACMUPDSALDOMOVICLIE, $paramsToACMUPDSALDOMOVICLIE);    
	        	}
	        }
            
			$this->entityManager->commit();
            return true;
        }catch (\Exception $e){
			$this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function inutilizarNota($nota){
    	
    	//seta informacoes iniciais no log
		$this->getLogProcessNota($nota);
		//le certificado
		$certificado = $this->getCertificado($nota);
		//config certificado
		$this->nfeTools = $this->getNfeTools($nota, $certificado);
		
		try {
    		
    		$inutiliza = $this->nfeTools->sefazInutiliza((int)$nota['CDSERIENFCE'], (int)$nota['NRNOTAFISCALCE'], (int)$nota['NRNOTAFISCALCE'], $nota['DSOBSMOVBENECONS'], $nota['IDTPAMBNFCE']);
		    if(!empty($inutiliza)){
				return $this->updateInutilizacaoNFCe($nota, $inutiliza);
		    }else{
		    	throw new \Exception("Não houve retorno ao tentar realizar inutilização da nota!");
		    }
			
		} catch (\Exception $e) {
			throw new \Exception("Erro ao tentar realizar a inutilização de nota!");
		}
    	
    }
    
    public function updateInutilizacaoNFCe($nota, $inutiliza){
    	if(!empty($inutiliza)){
			$stNfeTools = new Standardize($inutiliza);
			$resultXMLArray = $stNfeTools->toArray();
			if(!isset($resultXMLArray['infInut'])){
				throw new \Exception("Não foi possível recuperar retorno do evento ao tentar realizar a inutilização da nota!");
			}
		}else{
			throw new \Exception("Não foi possível recuperar informações ao tentar realizar o inutilização da nota!");
		}
		
    	$result = array(
    		'error' => true,
    		'message' => ''
    	);
    	
    	$arrayRejeicoes = $this->getRejeicaoConsulta();
    	$arraySucessos = $this->getSucessoConsulta();
    	try {
			
			$infEvento = $resultXMLArray['infInut'];
			
    		if (!empty($infEvento)){
				//verifica se nota foi realmente inutilizada através do status retornado
				if (!array_key_exists($infEvento['cStat'], $arrayRejeicoes) && array_key_exists($infEvento['cStat'], $arraySucessos)) { 
					
		            $dtHrProtocoNFCe = substr($infEvento['dhRecbto'], 0, 10).' '.substr($infEvento['dhRecbto'], 11, 8);
		            $dtHrProtocoNFCe = date_create_from_format('Y-m-d H:s:i', $dtHrProtocoNFCe);
		            $dtHrProtocoNFCe = $dtHrProtocoNFCe->format('d/m/Y H:s:i');
					
					$parametros['IDSTATUSNFCE'   ] = 'A';
		            $parametros['DTHRINUTNFCE'   ] = $dtHrProtocoNFCe;
		            $parametros['NRPROTOINUTNFCE'] = $infEvento['nProt'];
		            $parametros['IDTPEMISINUNFCE'] = $nota['IDAMBTRABNFCE'];
		            $parametros['DSRAZAOINUTNFCE'] = $nota['DSOBSMOVBENECONS'];
		            $parametros['DSOBSINUTNFCE'  ] = $infEvento['xMotivo'];
		            $parametros['CDFILIAL'       ] = $nota['CDFILIAL'];
		            $parametros['CDCAIXA'        ] = $nota['CDCAIXA'];
		            $parametros['NRNOTAFISCALCE' ] = $nota['NRNOTAFISCALCE'];
		            $parametros['NRINSJURFILI' ] = $nota['NRINSJURIEST'];
		            
		            $this->runQuery(true, TransmissaoNFCE::ACMUPDINUTILIZANFCE, $parametros);
		            $statusRetorno = !empty($arraySucessos[$infEvento['cStat']]) ? $arraySucessos[$infEvento['cStat']] : 'Não definido na aplicação!';
					
				}else{
					
					$parametros['IDSTATUSNFCE'   ] = 'R';
		            $parametros['IDTPEMISINUNFCE'] = $nota['IDAMBTRABNFCE'];
		            $parametros['DSRAZAOINUTNFCE'] = $nota['DSOBSMOVBENECONS'];
		            $parametros['DSOBSINUTNFCE'  ] = $infEvento['xMotivo'];
		            $parametros['CDFILIAL'       ] = $nota['CDFILIAL'];
		            $parametros['CDCAIXA'        ] = $nota['CDCAIXA'];
		            $parametros['NRNOTAFISCALCE' ] = $nota['NRNOTAFISCALCE'];
		            $parametros['NRINSJURFILI' ] = $nota['NRINSJURIEST'];
		            
		            $this->runQuery(true, TransmissaoNFCE::ACMUPDINUTILIZA, $parametros);
					$statusRetorno = !empty($arrayRejeicoes[$infEvento['cStat']]) ? $arrayRejeicoes[$infEvento['cStat']] : 'Não definido na aplicação!';
					
				}
				
				$DSOBSSTATUSNFCERET = 'Evento de Inutilização: '.$infEvento['cStat'] . ' - ' . $statusRetorno;
				
				
				TransmissaoNfceNewController::escreverLog("Atualizando informações da venda.");
				TransmissaoNfceNewController::escreverLog("Status Retorno: ". $DSOBSSTATUSNFCERET);
	        	$result['error'] = false;
	        	$result['DSOBSSTATUSNFCE'] = $DSOBSSTATUSNFCERET;
				
    		}else{
    			$result['error'] = true;
        		$result['DSOBSSTATUSNFCE'] = "Erro ao obter informações de inutilização!";
    		}
    		
    	} catch (\Exception $e) {
    		$result['message'] = $e->getMessage();
    	}
    	return $result;
    }
    
    public function transmiteNota($nota){
    	
    	//seta informacoes iniciais no log
		$this->getLogProcessNota($nota);
		
		//verifica status NFC-e na base de dados
		$this->getStatusNota($nota);
	
		//le certificado
		$certificado = $this->getCertificado($nota);
		
		//config certificado
		$this->nfeTools = $this->getNfeTools($nota, $certificado);
		
		try {
    	   return $this->callTransmitir($nota);
		} catch (\Exception $e) {
			TransmissaoNfceNewController::escreverLog("Erro ao tentar realizar a transmissão nota! ". $e->getMessage());
			throw new \Exception("Erro ao tentar realizar a transmissão nota!");
		}
    }
    
    private function getStatusNota(&$nota){
    	$mensagemGenerica = "Não foi selecionada nota válida. Notas na seguinte situação: ";
    	 $parametros = array(
            ':CDFILIAL' => $nota['CDFILIAL'],
            ':CDCAIXA'    => $nota['CDCAIXA'],
            ':NRSEQVENDA'    => $nota['NRSEQVENDA']
        );
        $result = $this->executarQuery(TransmissaoNFCE::SELECT_STATUS_VENDA, $parametros);
        if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            $result = $result->getResult();
            if(!empty($result)){
            	$result = $result[0];
                if ($result['IDSTATUSNFCE'] == 'A') {
                	throw new \Exception($mensagemGenerica."Transmitida", 3);
                }
                if ($result['IDSTATUSNFCE'] == 'D') {
                	throw new \Exception($mensagemGenerica."Denegada", 3);
                }
                if ($result['NRENVIOSEFAZ'] > 9 ) {
                	throw new \Exception($mensagemGenerica."Com 9 ou mais envios a SEFAZ. Reinicie tentativa de transmissão pela aplicação!", 3);
                }
                $nota['DSARQXMLNFCE'] = $result['DSARQXMLNFCE'];
                return;
            }else{
            	throw new \Exception("Não foi selecionada nenhuma Nota Fiscal.");
            }
        }
        return;
    }
    
    private function getStatusNotaSefaz($nota){
    	//caso nao tenha arquivo XML tem que transmistir para gerar no Retaguarda
    	if(empty($nota['DSARQXMLNFCE'])){
           return false; 
        }else{
            return true;
        }
    }
    
    private function callConsultaNFCe($chaveAcesso, $ambiente){
    	try {
		    $consulta = $this->nfeTools->sefazConsultaChave($chaveAcesso, $ambiente);
		    if(!empty($consulta)){
				if(!empty($consulta)){
					$stNfeTools = new Standardize($consulta);
					$resultXMLArray = $stNfeTools->toArray();
					return $resultXMLArray;
				}else{
					TransmissaoNfceNewController::escreverLog("Não foi possível recuperar informações ao tentar consultar status da nota!");
					//throw new \Exception("Não foi possível recuperar informações ao tentar consultar status da nota!");
				}
		    }else{
		    	TransmissaoNfceNewController::escreverLog("Não houve retorno ao tentar realizar a consulta da nota!");
		    	//throw new \Exception("Não houve retorno ao tentar realizar a consulta da nota!");
		    }
			
		} catch (\Exception $e) {
			//throw new \Exception("Erro ao tentar realizar a consulta da nota!");
			TransmissaoNfceNewController::escreverLog("Erro ao tentar realizar a consulta da nota! Message: ".$this->exceptionMessage($e->getMessage()));
		}
		return;
    }
    
    private function validaRetornoConsulta($nota, $retornoXML, $newChave = null, $regerouChave = false){
    	$this->entityManager->beginTransaction();
    	
    	$chaveAcessoConsulta = $nota['NRACESSONFCE'];
    	
    	$result = array(
    		'error' => false,
    		'message' => ''
    	);
    	
    	try{
    		
    		//funcao para verificar se chave de acesso encontra-se aprovada na base de dados do oracle
    		if($regerouChave){
    			$chaveAcessoConsulta = $newChave;
    		}
    		
    		if(!$this->validaChaveOracle($chaveAcessoConsulta, $nota)){
	        	 TransmissaoNfceNewController::escreverLog("Chave de acesso vinculada na Sefaz com as mesmas informações!");
	        	 $parametros['DSOBSSTATUSNFCE'] = "Chave de acesso vinculada na Sefaz com as mesmas informações!";
		         $parametros['CDFILIAL'       ] = $nota['CDFILIAL'];
		         $parametros['CDCAIXA'        ] = $nota['CDCAIXA'];
		         $parametros['NRSEQVENDA'     ] = $nota['NRSEQVENDA'];
		         TransmissaoNfceNewController::escreverLog("");
		         TransmissaoNfceNewController::escreverLog("Atualizando banco de dados!");
		         $this->runQuery(false, TransmissaoNFCE::ACMUPDPROTOCNFCECVAC, $parametros);
	        	 
	        }else{
		        
		        //verifica se o que esta aprovado o que encontra-se cadastrado
		        if($regerouChave){
		            //regera XML pq difere do que esta no banco
		            TransmissaoNfceNewController::escreverLog("Chave de acesso recalculada, alterado de ".$nota['NRACESSONFCE']." para ".$newChave." !");
		            $nota['NRACESSONFCE'] = $newChave;
		            $xmlRegerado =  $this->regerarXMLValido($newChave, null, $nota, null);
		            if (!empty($xmlRegerado)) {
		            	$xmlAssinado = $this->signNFCe($xmlRegerado);
		            	if($xmlAssinado){
		            		$getQrCode = $this->getQrCode($xmlAssinado);
		            	}
		            	TransmissaoNfceNewController::escreverLog("Atualizando dados da venda com nova chave de acesso.");
		            	$this->atualizarQRCODEBanco($xmlAssinado, $nota, $getQrCode);
		            }else{
				        throw new \Exception("XML gravado no banco inválido!");
				        TransmissaoNfceNewController::escreverLog("XML gravado no banco inválido!");
				    }
		        }
		        
		        if ($retornoXML['cStat'] == 100 || $retornoXML['cStat'] == 150) { 
		            $stStatusNFe = 'A';
		        } else {
		            if ($retornoXML['cStat'] == 101){
		            	$stStatusNFe = 'C';
		            }
		            if ($retornoXML['cStat'] == 110){
		            	$stStatusNFe = 'D';
		            }
		        }
		        
		        if(!isset($retornoXML['protNFe'])){
		        	throw new \Exception("Não foi possível recuperar retorno do protocolo através da chave de acesso da nota!");
		        }
		        
		        if(!isset($retornoXML['protNFe']['infProt'])){
		        	throw new \Exception("Não foi possível recuperar informações retorno do protocolo através da chave de acesso da nota!");
		        }
		        
		        $dadosProtocolo = $retornoXML['protNFe']['infProt'];
		        
		        $dhRecbto = $dadosProtocolo['dhRecbto'];
		        $dtHrProtocoNFCe = substr($dhRecbto, 0, 10).' '.substr($dhRecbto, 11, 8);
		        $dtHrProtocoNFCe = date_create_from_format('Y-m-d H:s:i', $dtHrProtocoNFCe);
		        $dtHrProtocoNFCe = $dtHrProtocoNFCe->format('d/m/Y H:s:i');
		        
		        $parametros['IDSTATUSNFCE'   ] = $stStatusNFe;
		        $parametros['DSOBSSTATUSNFCE'] = $dadosProtocolo['cStat'] . ' - ' . $dadosProtocolo['xMotivo'];
		        $parametros['NRPROTOCOLONFCE'] = $dadosProtocolo['nProt'];
		        $parametros['DTHRPROTOCONFCE'] = $dtHrProtocoNFCe;
		        $parametros['CDFILIAL'       ] = $nota['CDFILIAL'];
		        $parametros['CDCAIXA'        ] = $nota['CDCAIXA'];
		        $parametros['NRSEQVENDA'     ] = $nota['NRSEQVENDA'];
		        $parametros['NRACESSONFCE'   ] = $dadosProtocolo['chNFe'];
		        TransmissaoNfceNewController::escreverLog("Atualizando situação atual - Nota Fiscal ".$nota['NRNOTAFISCALCE']." - através da chave de acesso.");
		        TransmissaoNfceNewController::escreverLog("");
		        TransmissaoNfceNewController::escreverLog("Atualizando banco de dados!");
		        $this->runQuery(false, TransmissaoNFCE::ACMUPDPROTOCNFCE, $parametros);
		        
		        //se venda estiver cancelada e sua estiver aprovada, deverá tentar executar o cancelamento da nota na hora
		        if (($nota['IDSITUVENDA'] == 'C') and ($stStatusNFe == 'A')) {
		        	TransmissaoNfceNewController::escreverLog("Venda encontra-se cancelada, por esse motivo executando o cancelamento da nota...");
		            $nota['NRPROTOCOLONFCE'] = $dadosProtocolo['nProt'];
		            $this->callCancela('N', $nota);
		        }	
	        }
	        
	        $this->entityManager->commit();
    	}catch(\Exception $e){
    		$result['error'] = true;
    		$result['message'] = $e->getMessage();
    		$this->entityManager->rollback();
    	}
    	
        return $result;
    }
    
    private function recalculaChave($chaveAcesso, $permFEMISSAO = false){
		$NRACESSONFCE1 = substr($chaveAcesso, 0, 34);
		$FEMISSAO = substr($chaveAcesso, 34, 1);
        $NRACESSONFCE2 = substr($chaveAcesso, 35, 8);
        
        if(!$permFEMISSAO){
        	if($FEMISSAO == 9){
        		$FEMISSAO = 1;
	        }else{
	        	$FEMISSAO = 9;
	        }
        }
        
        $CHAVE = $NRACESSONFCE1.$FEMISSAO.$NRACESSONFCE2;
        $DIGITO = self::calculaDigitoVerificadorNFe($CHAVE); //$this->modulo11($NRACESSONFCE1.'9'.$NRACESSONFCE2);
        $modulo = $CHAVE.$DIGITO;
        
        return $modulo;
	}
	
	public static function calculaDigitoVerificadorNFe($chave){

        $nAlgarismo = 42;
        $multiplicadores = array(2,3,4,5,6,7,8,9);
        $nMultiplicador = 0;
        $soma = 0;
        $DV = 0;

        while($nAlgarismo >= 0){
            $algarismo = substr($chave, $nAlgarismo , 1);
            $soma += $algarismo*$multiplicadores[$nMultiplicador];
            if($nMultiplicador == 7){
                $nMultiplicador = 0;
            }else{
                $nMultiplicador++;
            }
            $nAlgarismo--;
        }
        $restoDivisao = $soma % 11;
        if($restoDivisao > 1){
            $DV = 11 - $restoDivisao;
        }

        return $DV;

    }
    
    private function regerarXMLValido($chave, $recibo = null, $nota, $dadosRetorno = null){
        //retira tags que tenham antes de <NFe e depois </NFe>
        if(!empty($nota['DSARQXMLNFCE'])){
        	$posIn = strpos($nota['DSARQXMLNFCE'], '<NFe');
	    	$xmlGerado = substr($nota['DSARQXMLNFCE'], $posIn);
	    	$posFi = strpos($xmlGerado, '</NFe>');
	    	$xmlGerado = substr($xmlGerado, 0, ($posFi+6));
        }else{
        	$xmlGerado = '';
        }
        
        $xml = simplexml_load_string($xmlGerado);
        if(empty($xml)){
        	return false;
        }
        
        $arrayData = $this->xmlToArray($xml);
        
        //troca a chave de acesso de referencia da nota fiscal
        $arrayData['NFe']['infNFe']['@Id'] = 'NFe'.$chave;
        
        //verifica chave de acesso - se foi emissao normal ou contigencia 
        $tpEmis = substr($chave, -10, 1);
        $arrayData['NFe']['infNFe']['ide']['tpEmis'] = $tpEmis;
        
        //verifica digitoVerificador
        $cDV = self::calculaDigitoVerificadorNFe($chave); 
        $arrayData['NFe']['infNFe']['ide']['cDV'] = $cDV;
        
        //se o tpEmissao for 9, preencher informacoes, caso nao, remover
        if($tpEmis != 9){
            unset($arrayData['NFe']['infNFe']['ide']['dhCont']);
            unset($arrayData['NFe']['infNFe']['ide']['xJust']);
            unset($arrayData['NFe']['infNFe']['infAdic']['infCpl']);
        }
        
        //informacoes complementares (verificar como essa tag fica no FORSALE)
        //$arrayData['NFe']['infNFe']['infAdic']['infCpl'] = '';
        //$arrayData['NFe']['infNFe']['infAdic']['obsCont']['xCampo'] = 'NrSeqVen-IdOrigemVen';
        //$arrayData['NFe']['infNFe']['infAdic']['obsCont']['xTexto'] = '0000175373-BAL_FNC';
        
        //remover qrcode, pq sera gerado novamente
        unset($arrayData['NFe']['infNFeSupl']);
        unset($arrayData['NFe']['Signature']);
        
        $xmlRetorno = $this->arrayToXml($arrayData['NFe']);
        return $xmlRetorno;      
    }
    
    private function xmlToArray($xml, $options = array()) {
        $defaults = array(
            'namespaceSeparator' => ':', // você pode querer que isso seja algo diferente de um cólon
            'attributePrefix' => '@',    // para distinguir entre os nós e os atributos com o mesmo nome
            'alwaysArray' => array(),    // array de tags que devem sempre ser array
            'autoArray' => true,         // só criar arrays para as tags que aparecem mais de uma vez
            'textContent' => '$',        // chave utilizada para o conteúdo do texto de elementos
            'autoText' => true,          // pular chave "textContent" se o nó não tem atributos ou nós filho
            'keySearch' => false,        // pesquisa opcional e substituir na tag e nomes de atributos
            'keyReplace' => false        // substituir valores por valores acima de busca
        );
        $options = array_merge($defaults, $options);
        $namespaces = $xml->getDocNamespaces();
        $namespaces[''] = null; // adiciona namespace base(vazio) 
    
        // Obtém os atributos de todos os namespaces
        $attributesArray = array();
        foreach ($namespaces as $prefix => $namespace) {
            foreach ($xml->attributes($namespace) as $attributeName => $attribute) {
                // Substituir caracteres no nome do atributo
                if ($options['keySearch']) $attributeName =
                        str_replace($options['keySearch'], $options['keyReplace'], $attributeName);
                $attributeKey = $options['attributePrefix']
                        . ($prefix ? $prefix . $options['namespaceSeparator'] : '')
                        . $attributeName;
                $attributesArray[$attributeKey] = (string)$attribute;
            }
        }
    
        // Obtém nós filhos de todos os namespaces
        $tagsArray = array();
        foreach ($namespaces as $prefix => $namespace) {
            foreach ($xml->children($namespace) as $childXml) {
                // Recursividade em nós filho
                $childArray = $this->xmlToArray($childXml, $options);
                list($childTagName, $childProperties) = $this->myEach($childArray);
    
                // Substituir caracteres no nome da tag
                if ($options['keySearch']) $childTagName =
                        str_replace($options['keySearch'], $options['keyReplace'], $childTagName);
                // Adiciona um prefixo namespace, se houver
                if ($prefix) $childTagName = $prefix . $options['namespaceSeparator'] . $childTagName;
    
                if (!isset($tagsArray[$childTagName])) {
                    // Só entra com esta chave
                    // Testa se as tags deste tipo deve ser sempre matrizes, não importa a contagem de elementos
                    $tagsArray[$childTagName] =
                            in_array($childTagName, $options['alwaysArray']) || !$options['autoArray']
                            ? array($childProperties) : $childProperties;
                } elseif (
                    is_array($tagsArray[$childTagName]) && array_keys($tagsArray[$childTagName]) 
                    === range(0, count($tagsArray[$childTagName]) - 1)
                ) {
                    $tagsArray[$childTagName][] = $childProperties;
                } else {
                    $tagsArray[$childTagName] = array($tagsArray[$childTagName], $childProperties);
                }
            }
        }
    
        // Obtém o texto do nó
        $textContentArray = array();
        $plainText = trim((string)$xml);
        if ($plainText !== '') $textContentArray[$options['textContent']] = $plainText;
    
        $propertiesArray = !$options['autoText'] || $attributesArray || $tagsArray || ($plainText === '')
                ? array_merge($attributesArray, $tagsArray, $textContentArray) : $plainText;
    
        // Retorna o nó como array
        return array(
            $xml->getName() => $propertiesArray
        );
    }
    
    private function validaChaveOracle($chaveAcesso, $nota){
    	
    	$anoMesChaveRetornada = substr($chaveAcesso, 2, 4);
    	$anoChaveRetornada = substr($anoMesChaveRetornada, 0, 2);
    	$mesChaveRetornada = substr($anoMesChaveRetornada, 2, 2);
    	
    	$anoMesChaveBanco = substr($nota['NRACESSONFCE'], 2, 4);
    	
    	$NRNOTAFISCALCE = substr($chaveAcesso, 25, 9);
    	$CDSERIENFCE = substr($chaveAcesso, 22, 3);
    	$NRINSJURIEST = substr($chaveAcesso, 6, 14);
    	
    	if($anoMesChaveRetornada != $anoMesChaveBanco){
    		return false;
    	}else{
    		$parametros = array(
	            ':NRACESSONFCE' => $chaveAcesso,
	            ':NRNOTAFISCALCE' => $NRNOTAFISCALCE,
	            ':CDSERIENFCE' => $CDSERIENFCE,
	            ':NRINSJURIEST' => $NRINSJURIEST,
	            ':ANO' => $anoChaveRetornada,
	            ':MES' => $mesChaveRetornada,
	            ':CDFILIAL' => $nota['CDFILIAL'],
	            ':CDCAIXA' => $nota['CDCAIXA'],
	            ':NRSEQVENDA' => $nota['NRSEQVENDA']
	        );
	        
	        $result = $this->executarQuery(TransmissaoNFCE::GETVALIDACHAVEACESSO, $parametros);
	        
	        if($result->error()) {
	            throw new \Exception("Erro ao buscar chave de acesso: ".$result->getMessage(), 1);
	        }else{
	          $result = $result->getResult();
	          if($result[0]['TOT'] > 0){
	          	 return false;
	          }else{
	          	return true;
	          }
	        }	
    	}
    }
    
    private function myEach(&$arr) {
        $key = key($arr);
        $result = ($key === null) ? false : [$key, current($arr), 'key' => $key, 'value' => current($arr)];
        next($arr);
        return $result;
    }
    
    /**
     * @param array $array the array to be converted
     * @param string? $rootElement if specified will be taken as root element, otherwise defaults to 
     *                <root>
     * @param SimpleXMLElement? if specified content will be appended, used for recursion
     * @return string XML version of $array
     */
    private function arrayToXml($array) {
      $xmlRetorno = simplexml_load_string(self::DEFAULT_XML);
      foreach($array as $key => $value){
          $infNFe = $xmlRetorno->addChild('infNFe');
          $infNFe->addAttribute('Id', $value['@Id']);
          $infNFe->addAttribute('versao', $value['@versao']);
          
          //forma o destinatario
          $ide = $infNFe->addChild('ide');
          foreach($value['ide'] as $k => $v){
              $ide->addChild($k, $v);
          }
          
          //forma o emitente
          $emit = $infNFe->addChild('emit');
          foreach($value['emit'] as $k => $v){
              if(is_array($v)){
                  $enderEmit = $emit->addChild('enderEmit');
                  foreach($v as $ke => $vl){
                      $enderEmit->addChild($ke, $vl);
                  }
              }else{
                  $emit->addChild($k, $v);
              }
          }
          
          if(!isset($value['det']['@nItem'])){
              foreach($value['det'] as $k => $v){
                  $det = $infNFe->addChild('det');
                  $det->addAttribute('nItem', $v['@nItem']);
                  $prod = $det->addChild('prod');
                  foreach($v['prod'] as $pe => $pl){
                      if(is_array($pl)){
                          $prd = $prod->addChild($pe);
                          foreach($pl as $im => $il){
                              if(is_array($il)){
                                  $pr = $prd->addChild($im);
                                  foreach($il as $chil => $viml){
                                      $pr->addChild($chil, $viml);
                                  }
                              }else{
                                  $prd->addChild($im, $il);
                              }
                          }
                      }else{
                          $prod->addChild($pe, $pl);
                      }
                  }
                  
                  //formara os impostos 
                  $imposto = $det->addChild('imposto');
                  foreach($v['imposto'] as $pe => $pl){
                      if(is_array($pl)){
                          $impl = $imposto->addChild($pe);
                          foreach($pl as $im => $il){
                              if(is_array($il)){
                                  $im = $impl->addChild($im);
                                  foreach($il as $chil => $viml){
                                      $im->addChild($chil, $viml);
                                  }
                              }else{
                                  $impl->addChild($im, $il);
                              }
                          }
                      }else{
                          $imposto->addChild($pe, $pl);
                      }
                  }
              }
          }else{
              $det = $infNFe->addChild('det');
              $det->addAttribute('nItem', $value['det']['@nItem']);
              $prod = $det->addChild('prod');
              foreach($value['det']['prod'] as $pe => $pl){
                  if(is_array($pl)){
                      $prd = $prod->addChild($pe);
                      foreach($pl as $im => $il){
                          if(is_array($il)){
                              $pr = $prd->addChild($im);
                              foreach($il as $chil => $viml){
                                  $pr->addChild($chil, $viml);
                              }
                          }else{
                              $prd->addChild($im, $il);
                          }
                      }
                  }else{
                      $prod->addChild($pe, $pl);
                  }
              }
              
              //formara os impostos 
              $imposto = $det->addChild('imposto');
              foreach($value['det']['imposto'] as $pe => $pl){
                  if(is_array($pl)){
                      $impl = $imposto->addChild($pe);
                      foreach($pl as $im => $il){
                          if(is_array($il)){
                              $im = $impl->addChild($im);
                              foreach($il as $chil => $viml){
                                  $im->addChild($chil, $viml);
                              }
                          }else{
                              $impl->addChild($im, $il);
                          }
                      }
                  }else{
                      $imposto->addChild($pe, $pl);
                  }
              }
          }
          
          //formara o total
          $total = $infNFe->addChild('total');
          foreach($value['total'] as $k => $v){
              if(is_array($v)){
                  $ICMSTot = $total->addChild('ICMSTot');
                  foreach($v as $ke => $vl){
                      $ICMSTot->addChild($ke, $vl);
                  }
              }else{
                  $total->addChild($k, $v);
              }
          }
          
          //formara transportadora
          $transp = $infNFe->addChild('transp');
          foreach($value['transp'] as $k => $v){
              $transp->addChild($k, $v);
          }
          //formara pagamento
          $pag = $infNFe->addChild('pag');
          foreach($value['pag'] as $k => $v){
          	if($k == 'vTroco'){
          		$pag->addChild($k, $v);
          	}
          	
          	if($k == 'detPag'){
          		$conditionPg1 = !array_key_exists('tPag', $value['pag']['detPag']);
          		$conditionPg2 = !array_key_exists('vPag', $value['pag']['detPag']);
          		$conditionPg3 = !array_key_exists('card', $value['pag']['detPag']);
          		if($conditionPg1 && $conditionPg2 && $conditionPg3){
          			foreach($value['pag']['detPag'] as $dt => $pg){
	          			if($dt != 'tPag' && $dt != 'vPag' && $dt != 'card'){
	          				$detPag = $pag->addChild('detPag');
	          				foreach($pg as $g => $l){
			          			if(!is_array($l)){
			          				$detPag->addChild($g, $l);
			          			}else{
			          				$card = $detPag->addChild('card');
			                        foreach($l as $crd => $cd){
			                           $card->addChild($crd, $cd);
			                        } 
			          			}
		          			}
	          			}
	          		}
          		}else{
          			$detPag = $pag->addChild('detPag');
          			foreach($value['pag']['detPag'] as $dt => $pg){
	      				if(!is_array($pg)){
	          				$detPag->addChild($dt, $pg);
	          			}else{
	          				$card = $detPag->addChild('card');
	                        foreach($pg as $crd => $cd){
	                           $card->addChild($crd, $cd);
	                        } 
	          			}
          			}
          		}
          	 }
          }
          
          //formara informacoes adicionais
          $infAdic = $infNFe->addChild('infAdic');
          foreach($value['infAdic'] as $k => $v){
              if(is_array($v)){
                  $obsCont = $infAdic->addChild('obsCont');
                  foreach($v as $ke => $vl){
                      if($ke === '@xCampo'){
                        $obsCont->addAttribute('xCampo', $vl);   
                      }else{
                        $obsCont->addChild($ke, $vl);    
                      }
                  }
              }else{
                  $infAdic->addChild($k, $v);
              }
          }
          
          //formara informacoes do responsavel tecnico caso tenha
          if(!empty($array['infNFe']['infRespTec'])){
            $infRespTec = $infNFe->addChild('infRespTec');
            foreach($value['infRespTec'] as $k => $v){
              $infRespTec->addChild($k, $v);
            }   
          }
      }
      return $xmlRetorno->asXML();
    }
    
    private function signNFCe($unsignedXML){
    	try {
    		
    		$XmlSign = $this->nfeTools->signNFe($unsignedXML);
		    if(!empty($XmlSign)){
				return $XmlSign;
		    }else{
		    	throw new \Exception("Não foi possível assinar XML gerado da nota!");
		    }
			
		} catch (\Exception $e) {
			throw new \Exception("Erro ao tentar assinar XML gerado da nota! Erro:".$this->exceptionMessage($e->getMessage()));
		}
    }
    
    private function getQrCode($signedXML){
    	$stNfeTools = new Standardize($signedXML);
		$resultXMLArray = $stNfeTools->toArray();
    	if(isset($resultXMLArray['infNFeSupl'])){
    		if(isset($resultXMLArray['infNFeSupl']['qrCode'])){
    			$qrCode = $resultXMLArray['infNFeSupl']['qrCode'];
    		}else{
    			$qrCode = "QRCode inexistente.";
    		}
    	}else{
    		$qrCode = "QRCode inexistente.";
    	}
		return $qrCode;
	}
	
	private function atualizarQRCODEBanco($xml, $nota, $qrCode = null) {
        
        $xml = str_replace('<?xml version="1.0" encoding="UTF-8"?>', '', $xml);
        $xml = str_replace('<?xml version="1.0"?>', '', $xml);
        
        $parametros = array(
            ':NRSEQVENDA'   => $nota['NRSEQVENDA'],
            ':CDCAIXA'      => $nota['CDCAIXA'],
            ':CDFILIAL'     => $nota['CDFILIAL']
        );
        
        if(strpos($xml, '<qrCode>') !== FALSE && $qrCode !== null){
        	$posIn = strpos($xml, '<qrCode>');
			$posFn = strpos($xml, '</qrCode>');
			
			$stQRCode = '<qrCode><![CDATA['.trim($qrCode).']]></qrCode>';
			
			$firstPart = substr($xml, 0, $posIn);
			$secondPart = substr($xml, ($posFn) + 9);
			$xml = trim($firstPart).$stQRCode.trim($secondPart);
        	
        }
        
        $parametrosXML = $parametros;
        $parametrosXML[':DSARQXMLNFCE'] = $xml;
        
        $parametrosQRCODE = $parametros;
        $parametrosQRCODE[':DSQRCODENFCE'] = $qrCode;
        
        //verifica se ha dados em ARQXMLNFCESAT
		$resultArqXML = $this->executarQuery(TransmissaoNFCE::SELECT_VENDA_ARQXMLNFCESAT, $parametros);
		if($resultArqXML->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
        	//se nao tiver dados na table ira ARQXMLNFCESAT inserir, caso tenha apenas ira updatar
            if($resultArqXML->getResult()[0]['VEND'] == 0){
            	$this->runQuery(false, TransmissaoNFCE::INSERT_ARQXMLNFCESAT_XML, $parametrosXML);
            }else{
        		$this->runQuery(false, TransmissaoNFCE::UPDATE_ARQXMLNFCESAT_XML, $parametrosXML);    	
            }
        }
        $this->runQuery(false, TransmissaoNFCE::UPDATE_VENDA_NFCE, $parametrosQRCODE);
        return;
    }
    
    public function montaXMLTransmitir($dadosNota, $sign = true) {
    	
    	$result = array(
            'error' => false,
            'message' => '',
            'xml' => '',
            'nrAcessoNFCE' => ''
        );
    	
	    $idimpvenda 	= 'I';
	    $this->DTEMISSAO = new \DateTime($dadosNota[0]['DTEMISSAONFCE']);
	    // 1-> Producao, 2 -> Homologação
	    
		$cdtipooper = '66';
        $dadosEmpresaResult = $this->validaDadosEmpresaNew($dadosNota[0]);
        if($dadosEmpresaResult['error'] == false){
            $endeFiliResult = $this->validaEnderecoFilialNew($dadosNota[0]);
            if($endeFiliResult['error'] == false){
                $dadosEmitente = $this->validaDadosEmitenteXMLNew($dadosNota[0]);
                if($dadosEmitente['error'] == false){
				    $filialInfo = $this->montaArrayFilialNew($dadosNota[0]);
					$emitArray = $this->montaArrayEmit($filialInfo);
				    $serieNFCE = $this->validaSerieNFCENew($dadosNota[0]);
  					if($serieNFCE['error'] == false){
				    	$nfceInfo = $this->montaArrayInfoNFCENew($dadosNota[0], $filialInfo);
				    	//VALIDA SE HA TODOS OS PARAMETROS
				    	$validateNfce = $this->validateDataNfce(array_merge($nfceInfo, $filialInfo));
				    	if (!$validateNfce['error']){
							$infoConsumer = $this->getInfoConsumer($filialInfo, $nfceInfo);
					        $ideArray = $this->montaArrayIdeNew($filialInfo, $nfceInfo, $dadosNota[0]);
							$totalArray = $this->montaArrayTotalNew($filialInfo, $dadosNota);
							$transpArray = $this->montaArrayTransp();
							if($dadosNota[0]['IDRESPTECNFCE'] == 'S'){
								$infRespTec = $this->montaArrayRespTec();	
							}else{
								$infRespTec = array();
							}
							$infAdicArray = $this->montaArrayInfAdic($dadosNota[0]);
							$paymentArray = $this->montaArrayPayments($filialInfo, $dadosNota[0]['DTABERTUR'], $dadosNota[0]['NRSEQVENDA'], $dadosNota[0]['TOTALVENDA'], $dadosNota[0]['IDNOTATECNFCE']);
	                        $saleArray = $this->montaArraySale($ideArray, $emitArray, $infRespTec, $totalArray, $transpArray, $infAdicArray, $paymentArray, $dadosNota[0]['IDAMBTRABNFCE'], $infoConsumer);
	                        $xml = $this->gerarXMLValidado($saleArray, $nfceInfo['nrAcessoNFCE'], $this->getPermiteContingencia($filialInfo), $sign);
	                        if(!$xml){
	                        	TransmissaoNfceNewController::escreverLog("Erro ao gerar XML da Nota. ");
				    			$result['error'] = true;	
	                        }else{
	                        	$result['xml'] = $xml;	
	                        	$result['nfceInfo'] = $nfceInfo;	
	                        }
							return $result;
				    	} else {
				    		TransmissaoNfceNewController::escreverLog("Erro ao validar dados da NFCE. ". $validateNfce['message']);
				    		$result['error'] = true;
				    	}
  					} else {
  						TransmissaoNfceNewController::escreverLog("Erro ao validar série NFCE. ".$serieNFCE['message']);
						$result['error'] = true;
					}
  				} else {
  					TransmissaoNfceNewController::escreverLog("Erro ao validar dados do emitente do XML. ".$dadosEmitente['message']);
					$result['error'] = true;
				}
	    	} else {
	    		TransmissaoNfceNewController::escreverLog("Erro ao validar endereço da unidade. ".$endeFiliResult['message']);
				$result['error'] = true;
			}
		} else {
			TransmissaoNfceNewController::escreverLog("Erro ao validar dados da empresa. ".$dadosEmpresaResult['message']);
			$result['error'] = true;
		}
	}
	
	private function validaDadosEmpresaNew($empresa){
		$arrayResult = array();
		//CDINSCESTA, CDEMPRESA, CDPAIS, SGESTADO, CDMUNICIPIO, NRINSJURFILI, NMRAZSOCEMPR
		if(!empty($empresa['CDINSCESTA']) 
			|| 	!empty($empresa['CDEMPRESA'])
			|| !empty($empresa['CDPAIS'])
			|| !empty($empresa['SGESTADO'])
			|| !empty($empresa['CDMUNICIPIO'])
			|| !empty($empresa['NRINSJURFILI'])
			|| !empty($empresa['NMRAZSOCEMPR'])
		){
			$arrayResult['error'] = false;
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há dados para a filial informada";
		}
		return $arrayResult;
	}
	
	private function validaEnderecoFilialNew($endereco){
    	$arrayResult = array();
		if(!empty($endereco)){
			$arrayResult['error'] = false;
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há endereço para a filial logada";
		}
		return $arrayResult;
    }
    
    private function validaDadosEmitenteXMLNew($emitente){
    	$arrayResult = array();
		if(!empty($emitente)){
			$arrayResult['error'] = false;
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há dados de emitente da filial.";
		}
		return $arrayResult;
    }
    
    private function montaArrayFilialNew($dadosEmitente){
    	$filialInfo = array(
    		'NMRAZSOCFILI' => $this->formataCaracteresXML($dadosEmitente['NMRAZSOCFILI']),
		    'NMFILIAL' => $this->formataCaracteresXML($dadosEmitente['NMFILIAL']),
		    'NRINSJURFILI' => $this->noSpecials($dadosEmitente['NRINSJURFILI']),
		    'CDESTADOIBGE' => $dadosEmitente['CDESTADOIBGE'],
		    'CDMUNICIBGE' => strlen($dadosEmitente['CDMUNICIBGE']) != 5 ? $dadosEmitente['CDMUNICIBGE'] : $dadosEmitente['CDESTADOIBGE'] . $dadosEmitente['CDMUNICIBGE'],
		    'CDINSCESTAFILI' => $this->noSpecials($dadosEmitente['CDINSCESTA']),
		    'CDINSCMUNIFILI' => $dadosEmitente['CDINSCMUNI'],
		    'DSENDEFILI' => $this->formataCaracteresXML($dadosEmitente['DSENDEIEST']),
		    'NRENDEFILISEP' => '000',
		    'NMBAIRFILI' => $this->formataCaracteresXML($dadosEmitente['NMBAIRIEST']),
		    'NMMUNICIPIO' => $this->formataCaracteresXML($dadosEmitente['NMMUNICIPIO']),
		    'SGESTADOFILI' => $dadosEmitente['SGESTADO'],
		    'NRCEPFILI' => str_replace('-','',$dadosEmitente['NRCEPIEST']),
		    'CDPAISBACEN' => $dadosEmitente['CDPAISBACEN'],
		    'NMPAIS' => $dadosEmitente['NMPAIS'],
		    'CDCNAE' => $dadosEmitente['CDCNAE'],
		    'CDSITUCRT'  => $dadosEmitente['CDSITUCRT'],
		    'CDEMPRESA' => $dadosEmitente['CDEMPRESA'],
		    'CDINSCESTA' => $dadosEmitente['CDINSCESTA'],
		    'CDPAIS' => $dadosEmitente['CDPAIS'],
		    'SGESTADO' 	=> $dadosEmitente['SGESTADO'],
			'CDMUNICIPIO' => $dadosEmitente['CDMUNICIPIO'],
			'CDFILIAL' => $dadosEmitente['CDFILIAL'],
			'CDCAIXA' => $dadosEmitente['CDCAIXA'],
			'NRORG' => $dadosEmitente['NRORG'],
			'CDLOJA' => $dadosEmitente['CDLOJA'],
			'CDOPERADOR' => $dadosEmitente['CDOPERADOR'],
			'CDURLWSNFC' => $dadosEmitente['CDURLWSNFC']
    	);
    	return $filialInfo;
    }
    
    private function montaArrayEmit($filialInfo){
   		// --- IDENTIFICAÇÃO DO EMITENTE
		// - identificação do endereço do emitente
		$enderEmit = array(
						'xLgr' 	  =>	$this->formataCaracteresXML($filialInfo['DSENDEFILI']), // DSENDEFILI
						'nro'	  =>	$filialInfo['NRENDEFILISEP'], // NRENDEFILISEP
						'xBairro' =>	$this->formataCaracteresXML($filialInfo['NMBAIRFILI']), // NMBAIRFILI
						'cMun'	  =>	$filialInfo['CDMUNICIBGE'], // CDMUNICIBGE
						'xMun'	  =>	$this->formataCaracteresXML($filialInfo['NMMUNICIPIO']), // NMMUNICIPIO
						'UF'	  =>	$filialInfo['SGESTADOFILI'], // SGESTADO
						'CEP'	  =>	str_replace(array('.', '-'), '', $filialInfo['NRCEPFILI']), // CEP
						'cPais'	  =>	$filialInfo['CDPAISBACEN'], // CDPAISBACEN
						'xPais'	  =>	$filialInfo['NMPAIS']  // NMPAIS
					 );
		// - identificação do emitente
		$emit = array(
					'CNPJ'		 =>	$filialInfo['NRINSJURFILI'], // NRINSJURFILI
					'xNome'	     =>	$this->formataCaracteresXML($filialInfo['NMRAZSOCFILI']), // NMRAZSOCFILI
					'xFant'	     =>	$this->formataCaracteresXML($filialInfo['NMFILIAL']), // NMFILIAL
					'enderEmit'  => $enderEmit, // ENDERECO EMITENTE
					'IE'		 => $filialInfo['CDINSCESTAFILI'], // CDINSCESTA
					'IM'		 => $filialInfo['CDINSCMUNIFILI'], // CDINSCMUNI
					'CNAE'		 => $filialInfo['CDCNAE'], // CDCNAE
					'CRT'		 => $filialInfo['CDSITUCRT']  // CDSITUCRT
				);

		return $emit;
   	}
   	
   	private function validaSerieNFCENew($serieNFCE){
		$arrayResult = array();
		if(!empty($serieNFCE)){
			$arrayResult['error'] = false;
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há serie NFCE no caixa logado.";
		}
		return $arrayResult;
    }
    
    private function montaArrayInfoNFCENew($dadosEmitente, $filialInfo){
		
		// define informações do certificado para produção/homologação
		if ($dadosEmitente['IDAMBTRABNFCE'] == '1'){
			$CSC =  $dadosEmitente['CDCODSCONSPROD'];
			$CSCid = $dadosEmitente['CDIDTOKENPROD'];
		} else {
			$CSC = $dadosEmitente['CDCODSCONSHOMO'];
			$CSCid = $dadosEmitente['CDIDTOKENHOMO'];
		}
		
	    $serieCH = str_pad($dadosEmitente['CDSERIENFCE'],3,"0",STR_PAD_LEFT);
        /*
        > 8 pega so os 8 ultimo
        < 8 preenche zero a esquerda
        = 8 os oitos :)
        */
        $nrlanctonfce = $this->formatValueNrlanctonfce($dadosEmitente['NRLANCTONFCE']);
	    $nrNFE = $dadosEmitente['NRNOTAFISCALCE'];
	    $tipEmissao = $this->getPermiteContingencia($filialInfo) ? self::AMB_EMISSAO_OFFLINE : self::AMB_EMISSAO_ONLINE;
		$nrAcessoNFCE = $this->gerarChaveAcessoNFCeNew($filialInfo, $serieCH, $nrNFE, $tipEmissao, $this->DTEMISSAO, $nrlanctonfce);
		
		$nfceInfo = array(
			'CSC' => $CSC,
			'CSCid' => $CSCid,
			'NMARQCERTNFCE' => $dadosEmitente['NMARQCERTNFCE'],
		    'DSSENHACERTNFCE' => $dadosEmitente['DSSENHACERTNFCE'],
			'tpEmis' => $tipEmissao,
			'nrNFE' => $nrNFE,
			'nrNFEInt' => intval($nrNFE),
			'serieCH' => intval($serieCH),
			'cDV' => substr($nrAcessoNFCE, -1),
			'nrAcessoNFCE' => $nrAcessoNFCE,
			'VRTOTTRIBIBPT' => $dadosEmitente['VRTOTTRIBIBPT'],
			'nrseqvenda' => $dadosEmitente['NRSEQVENDA'],
			'totalVenda' => $dadosEmitente['TOTALVENDA'],
			'tpAmb' => $dadosEmitente['IDAMBTRABNFCE'],
			'NRLANCTONFCE' => $nrlanctonfce
		);
		return $nfceInfo;
	}
	
	private function validateDataNfce($nfceData){
		$result = array(
			'error' => false,
			'message' => ''
		);

		$nfceParams = $this->associateCodeToMeaning();
		$missingParameters = array();
		foreach ($nfceParams as $param => $message) {
			if (empty($nfceData[$param])){
				array_push($missingParameters, $message);
			}
		}

		if (!empty($missingParameters)){
			$result['error'] = true;
			$result['message'] = 'Não foi possível utilizar a NFCE. ' . $this->nfceMissingParameterMessage($missingParameters);
		}
		return $result;
	}
	
	public function getInfoConsumer($filialInfo, $nfceInfo){
		$paramsToConsumer = array(
			':CDFILIAL'   => $filialInfo['CDFILIAL'],
			':CDCAIXA'    => $filialInfo['CDCAIXA'],
			':NRSEQVENDA' => $nfceInfo['nrseqvenda']
		);
        $result = $this->executarQuery(TransmissaoNFCE::GET_CONSUMER_SALE, $paramsToConsumer);
        if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            if(!empty($result->getResult())){
               return $result->getResult()[0];
            }
        }
        
        return array();
	}
	
	private function montaArrayIdeNew($filialInfo, $nfceInfo, $dadosNota){
   		// --- IDENTIFICAÇÃO DA NOTA
		$ide =  array(
					 'cUF'    	=>  $filialInfo['CDESTADOIBGE'], // CDESTADOIBGE
			         'cNF'    	=>  $nfceInfo['NRLANCTONFCE'], // NUMERO NRLANCTONFCE 8 DIG
			         'natOp'  	=>  'Descricao da natureza de operacao : Venda', // DESCRIÇÃO DA NATUREZA DE OPERAÇÃO
			         'mod'   	=>  self::MODELO_DOC,
			         'serie'  	=>  $nfceInfo['serieCH'], // CDSERIENFE
			         'nNF'	 	=>  $nfceInfo['nrNFEInt'], // NÚMERO DA NOTA (NOVOCODIGO NFCE_FILIAL_AMBIENTE)
			         'dhEmi'  	=>  $this->DTEMISSAO->format('c'), // FULL DATETIME, MOMENTO DE EMISSÃO
			         'tpNF'   	=>  1, // TIPO DA NOTA FISCAL : 0 - ENTRADA, 1 - SAIDA
			         'idDest' 	=>  1, // ID DESTINO: 1 - INTERNO, 2 - INTERESTADUAL , 3 - EXTERIOR
			         'cMunFG' 	=>  $this->noSpecials($filialInfo['CDMUNICIBGE']), // CDMUNICIBGE
			         'tpImp'  	=>	4, // TIPO DE IMPRESSAO : 4 - DANFE NFCe
			         'tpEmis' 	=>  $nfceInfo['tpEmis'], // TIPO DE EMISSÃO : 1 - NORMAL / 9 - Contigência OFFLINE
			         'cDV'    	=>  $nfceInfo['cDV'], // DÍGITO VERIFICADOR nrAcessoNFCE
			         'tpAmb'  	=>  $nfceInfo['tpAmb'], // TIPO DE AMBIENTE : 1 - PRODUÇÃO, 2 - HOMOLOGAÇÃO
			         'finNFe' 	=>  1, // FINALIDADE DA NOTA : 1 - NORMAL
			         'indFinal' =>	1, // CONSUMIDOR FINAL : 1 - SIM
			         'indPres'  =>	1 // OPERACAO PRESENCIAL : 1 - SIM

	    );
	    
	    //PARAMETRO PARAVEND  
	    /*
	    	110 => COM TAG indIntermed
	    	120 OU NULL => SEM TAG indIntermed
	    */
	    
	    if($dadosNota['IDNOTATECNFCE'] == '110'){
        	$ide['indIntermed']  =	0; // Indicativo do Intermediador  : 0 - Operacao sem Intermediador	, 1 - Operacao com Intermediador
        }
         
        $ide['procEmi']  =	0; // PROCESSO DE EMISSÃO : 0 - POR APP DO CONTRIBUINTE
        $ide['verProc']  =	'Odhen Retail';  // VERSÃO DO PROCESSO EMISSOR
		return $ide;
   	}
   	
   	private function montaArrayTotalNew($filialInfo, $dadosNota) {
	
   		$icmsArray = $this->montaArrayICMS();
		$totalArray = array();
		$totalArray['ICMSTot'] = $icmsArray;
	    $indexItemAprov = 0;
	    
	    $pkCount = (is_array($dadosNota) ? count($dadosNota) : 0);
	    $impProd = $this->calcImpProd($dadosNota[0]['VRTOTTRIBIBPT'], $pkCount);
	    
	    foreach ($dadosNota as $item) {
			$nrsequitvend = $indexItemAprov + 1;
			$nrsequitvend = str_pad($nrsequitvend, 3, '0', STR_PAD_LEFT);
			
			$itemXml = $this->getSaleItemNew(
				$filialInfo['CDFILIAL'],
				$filialInfo['CDLOJA'],
				$filialInfo['CDCAIXA'],
				$filialInfo['NRORG'],
				$item['NRSEQVENDA'],
				$nrsequitvend,
				$filialInfo['CDEMPRESA'],
				$filialInfo['CDPAIS'],
				$filialInfo['SGESTADO'],
				$filialInfo['CDMUNICIPIO'],
				$filialInfo['CDINSCESTA'],
				$item,
				$dadosNota[0]['CDOPERADOR'],
				$impProd['perProd']
			);
			
			if (!empty($itemXml['prod']['cProd'])){
				$indexItemAprov++;
				$nItem = "nItem+".$indexItemAprov;
				if($dadosNota[0]['IDAMBTRABNFCE'] == 2 && $indexItemAprov == 1){
					$itemXml['prod']['xProd'] = 'NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL';
				}
				// $saleArray['det+'.$nItem] = $itemXml;
				$totalArray['toSaleArray']['det+'.$nItem] = $itemXml;
				if($itemXml['prod']['indTot'] == '1'){
					$icms = $itemXml['imposto']['ICMS'];
					$totalArray = $this->updateTotal($itemXml,$totalArray, $icms);
				}
			}
	    }
		$totalArray['ICMSTot']['vNF'] = $this->calcTotalNF($totalArray);
		// evitar diferenças no total do imposto no caso de valor ímpar
		$totalArray['ICMSTot']['vTotTrib'] = $impProd['totTrib'];
	    return $totalArray;
   	}
   	
   	private function montaArrayTransp(){
   		$transp = array(
         	'modFrete'	 =>	9
        );
        return $transp;
   	}
   	
   	private function montaArrayRespTec(){
		$infRespTec = array(
         	'CNPJ'	 =>	'26269316000177',
         	'xContato'	 =>	'Wilson Lima de Paula',
         	'email'	 =>	'qualidade@teknisa.com',
         	'fone'	 =>	'03121222300'
        );
        return $infRespTec;
   	}
   	
   	private function montaArrayInfAdic($dadosNota){
   		
   		$infAdic = array(
         	'obsCont+xCampo+NrSeqVen-IdOrigemVen' => array(
         		'xTexto' => $dadosNota['CDFILIAL'].'-'.$dadosNota['CDCAIXA'].'-'.$dadosNota['NRSEQVENDA'].'-'.$dadosNota['IDORIGEMVENDA']
         		
         	),
    //Campos inseridos para rodar a query GET_DSOBSCUPFIS dentro da função generatesStringXMLByIndexedArray, os mesmos são retirados na mesma função utilizando o unset
         	
         	'filial'=> $dadosNota['CDFILIAL'],
         	'caixa'=> $dadosNota['CDCAIXA'],
         	'loja' => $dadosNota['CDLOJA']
         	
			
        );
        return $infAdic;
   	}
   	
   	private function montaArrayPayments($filialInfo, $dtabercaix, $nrseqvenda, $totalVenda = null, $IDNOTATECNFCE = null){
		$paymentArray = array();
		$vTroco = null;
		$vTotalPgto = null;
		$params = array(
			'CDFILIAL' => $filialInfo['CDFILIAL'],
			'CDCAIXA' => $filialInfo['CDCAIXA'],
			'NRSEQVENDA' => $nrseqvenda
		);
		$paymentModes = $this->entityManager->getConnection()->fetchAll(TransmissaoNFCE::GET_PAYMENTS, $params);
		$idpag = 1;
		if(!empty($paymentModes)){
			foreach ($paymentModes as $paymentMode) {
				if ($paymentMode['IDTIPOMOVIVE'] != 'S'){
					if($paymentMode['VRMOVIVEND'] > 0){
						$paymentArray['detPag+nItem+' . $idpag] = $this->getPayment($paymentMode, $IDNOTATECNFCE);
						$idpag++;
						$vTotalPgto = $vTotalPgto + ($paymentMode['VRMOVIVEND']);
					}
				} else {
					$vTroco = $paymentMode;
				}
			}
		}else{
		  $paymentArray['detPag+nItem+' . $idpag] = $this->getPayment(null, $IDNOTATECNFCE);
		}
		
		$vTotalPgto = $this->numberFormat($vTotalPgto, 2, '.', '');
		$totalVenda = $this->numberFormat($totalVenda, 2, '.', '');
		
		if(!empty($vTroco) && !empty($paymentModes)){
			$paymentArray['vTroco'] = !empty($vTroco) ? $this->numberFormat($vTroco['VRMOVIVEND'], 2, '.', '') : '0.00';	
		}else if (($vTotalPgto != $totalVenda) && $vTotalPgto > 0 && ($vTotalPgto > $totalVenda)){
			$paymentArray['vTroco'] = $this->numberFormat($vTotalPgto - $totalVenda, 2, '.', '');
		}else if(!empty($paymentModes)){
			$paymentArray['vTroco'] = '0.00';
		}
		return $paymentArray;
	}
	
	private function montaArraySale($ideArray, $emitArray, $infRespTec,$totalArray, $transpArray, $infAdicArray, $paymentArray, $IDAMBTRABNFCE, $infoConsumer) {
		$saleArray = array();
		$saleArray['ide'] = $ideArray;
		$saleArray['emit'] = $emitArray;
		if (!empty($infoConsumer['NRINSCRCONS'])){
	        $destArray = $this->montaArrayDest($IDAMBTRABNFCE, $infoConsumer);
	        $saleArray['dest'] = $destArray;
		}
		$indexItemAprov = 0;
		foreach ($totalArray['toSaleArray'] as $item) {
			$indexItemAprov++;
			$nItem = "nItem+".$indexItemAprov;
			$saleArray['det+'.$nItem] = $item;
		}
		
		unset($totalArray['toSaleArray']);
		$saleArray['total'] = $totalArray;
		$saleArray['transp'] = $transpArray;
		$saleArray['pag'] = $paymentArray;
		$saleArray['infAdic'] = $infAdicArray;
		if(!empty($infRespTec)){
			$saleArray['infRespTec'] = $infRespTec;	
		}
		return $saleArray;
	}
	
	private function iniciaTransmissao($dadosTransmissao, $nota){
		$idLote = substr(str_replace(',', '', number_format(microtime(true)*1000000, 0)), 0, 15);
		try {
    		
    		$resultTrans = $this->nfeTools->sefazEnviaLote(array($dadosTransmissao['xml']), $idLote, 1);	
		    if(!empty($resultTrans)){
				return $this->validarTransmissao($resultTrans, $dadosTransmissao, $nota);
		    }else{
		    	throw new \Exception("Não houve retorno ao tentar transmitir nota!");
		    }
		} catch (\Exception $e) {
			throw new \Exception("Erro ao tentar transmitir nota! Message: " .$this->exceptionMessage($e->getMessage()));
		}
	}
	
	private function validarTransmissao($resultTrans, $dadosTransmissao, $nota){
		if(!empty($resultTrans)){
			$stNfeTools = new Standardize($resultTrans);
			$resultXMLArray = $stNfeTools->toArray();
		}
		
		$result = array(
    		'error' => true,
    		'message' => ''
    	);
    	
    	$arrayRejeicoes = $this->getRejeicaoConsulta();
    	$arraySucessos = $this->getSucessoConsulta();
    	
    	$XMLenvio = str_replace('<?xml version="1.0"?>', '', $dadosTransmissao['xml']);
    	$QRCODENFCE = $this->getQrCode($XMLenvio);
    	$nfceInfo = $dadosTransmissao['nfceInfo'];
    	
    	
    	$params = array(
				'CDSERIENFCE' => str_pad($nfceInfo['serieCH'],3,"0",STR_PAD_LEFT),
				'NRACESSONFCE' => $nfceInfo['nrAcessoNFCE'],
				'NRNOTAFISCALCE' => $nfceInfo['nrNFE'],
				'NRLANCTONFCE' => $nfceInfo['NRLANCTONFCE'],
				'IDTPAMBNFCE' => $nfceInfo['tpAmb'],
				'IDSTATUSNFCE' => "P",
				'DTEMISSAONFCE' => $this->DTEMISSAO,
				'NRPROTOCOLONFCE' => '',
				'DSOBSSTATUSNFCE' => "",
				'IDMODOPERACNFCE' => '1',
				'NRRECIBONFCE' => null,
				'CDOPERENVIONFCE' => null,
				'DTHRPROTOCONFCE' => null,
				'IDXMLVALIDNFCE' => 'S',
				'DSQRCODENFCE' => $QRCODENFCE,
				'CDFILIAL' => $nota['CDFILIAL'],
				'CDCAIXA' => $nota['CDCAIXA'],
				'NRSEQVENDA' => $nota['NRSEQVENDA'],
				'CDVERSXMLNFCVND' => self::VERSAO_LAYOUT,
				'NRORG' => $nota['NRORG']
			);
    	try {
			
			if(!isset($resultXMLArray['protNFe'])){
	        	throw new \Exception("Não foi possível recuperar protocolo ao tentar transmitir nota!");
	        }
	        
	        if(!isset($resultXMLArray['protNFe']['infProt'])){
	        	throw new \Exception("Não foi possível recuperar informações do protocolo ao tentar transmitir nota!");
	        }
	        
		    $infEvento = $resultXMLArray['protNFe']['infProt'];
	
			TransmissaoNfceNewController::escreverLog("Validando retorno da SEFAZ!.");
        	$idStatusNFCE = $this->getIDSTATUSNFCE($infEvento['cStat']);
        	$params["IDSTATUSNFCE"] = $idStatusNFCE;
			$params["DSOBSSTATUSNFCE"] = $infEvento['cStat']." - ".$infEvento['xMotivo'];
			$params["DTHRPROTOCONFCE"] = new \DateTime($infEvento['dhRecbto']);
			//verifica se nota foi realmente transmitida através do status retornado
			if (!array_key_exists($infEvento['cStat'], $arrayRejeicoes) && array_key_exists($infEvento['cStat'], $arraySucessos)) { 
				TransmissaoNfceNewController::escreverLog("Nota APROVADA pela SEFAZ! ".$infEvento['cStat']." - ".$infEvento['xMotivo']);
				$params['NRPROTOCOLONFCE'] = !empty($infEvento['nProt']) ? $infEvento['nProt'] : null;
				
				$this->updateVendaNFCE($params, $XMLenvio);
				
				//altera os dados gravados no xml com informacoes de protocolo no xml
				//quando nota aprovada o XML tem que receber as informacoes de protocolo para salvar certo
				$this->createProtocoloXML($params);
				
				//se nota foi aprovada e venda esta cancelada, deve cancelar nota imediatamente
				if($idStatusNFCE == 'A' && $nota['IDSITUVENDA'] == 'C'){
					$nota['JUSTIFICATIVA'] = 'Nota cancelada, pois a venda encontra-se cancelada';
					$nota['NRPROTOCOLONFCE'] = $params['NRPROTOCOLONFCE'];
                	//chamar uma funcao que cancele transmissao
	                TransmissaoNfceNewController::escreverLog("Venda encontra-se cancelada, por esse motivo executando o cancelamento da nota...");
		            $this->callCancela('N', $nota);
				}
			}else{
				TransmissaoNfceNewController::escreverLog("Nota não consta com retorno de aprovação");
				$this->verificaDuplicidadeNota($infEvento, $nfceInfo, $nota, $XMLenvio, $params);
			}
        	$result['error'] = false;
    	} catch (\Exception $e) {
    		TransmissaoNfceNewController::escreverLog('Exception: '.$e->getMessage());
    		$this->verificaDuplicidadeNota($resultXMLArray, $nfceInfo, $nota, $XMLenvio, $params);
    		$result['message'] = $e->getMessage();
    	}
    	return $result;
	}
	
	private function verificaDuplicidadeNota($resultXMLArray, $nfceInfo, $nota, $XMLenvio, $params){
		
		TransmissaoNfceNewController::escreverLog("Validando se nota está em duplicidade!");
		if($resultXMLArray['cStat'] == '539' || $resultXMLArray['cStat'] == '204' || $resultXMLArray['cStat'] == '52' || $resultXMLArray['cStat'] == '51' || $resultXMLArray['cStat'] == '613'){
			
			$item['CSTAT'] = $resultXMLArray['cStat'];
			$item['NRACESSONFCE'] = $nfceInfo['nrAcessoNFCE'];
			$item['NRRECIBONFCE'] = null;
			$item['DSOBSSTATUSNFCE'] = $resultXMLArray['cStat']." - ".$resultXMLArray['xMotivo'];
			
			TransmissaoNfceNewController::escreverLog($item['DSOBSSTATUSNFCE']);
			
			$regerouChave = false;
			
			$parametrosDup['NRRECIBONFCE'] = null;
			$parametrosDup['NRACESSONFCE'] = $nfceInfo['nrAcessoNFCE'];
			$parametrosDup['CDFILIAL']     = $nota['CDFILIAL'];
            $parametrosDup['CDCAIXA']      = $nota['CDCAIXA'];
            $parametrosDup['NRSEQVENDA']   = $nota['NRSEQVENDA'];
			
			if(!$this->corrigeDuplicidade($item, $regerouChave, $parametrosDup, $nota, $XMLenvio)){
				
				TransmissaoNfceNewController::escreverLog("Nota Rejeitada! Ao consultar chave não consta como aprovada na SEFAZ");
	    		$params["DSOBSSTATUSNFCE"] = $resultXMLArray['cStat']." - ".$resultXMLArray['xMotivo'];
	    		$idStatusNFCE = "R";
		        $params["IDSTATUSNFCE"] = $idStatusNFCE;
	    		
	    		//tratando mensagens em relacao aos itens retornados
		        //get function que trata mensagens
		        $params = $this->trataRetornoMensagemSefaz($params, $XMLenvio);
				TransmissaoNfceNewController::escreverLog("Nota rejeitada pela SEFAZ! ".$params["DSOBSSTATUSNFCE"]);
	    		
	    		$this->updateVendaNFCE($params, $XMLenvio);
			}else{
				//altera os dados gravados no xml com informacoes de protocolo no xml
				//quando nota aprovada o XML tem que receber as informacoes de protocolo para salvar certo
				$this->createProtocoloXML($params);
			}
		}else{
			TransmissaoNfceNewController::escreverLog("Nota com Rejeicão!");
    		$params["DSOBSSTATUSNFCE"] = $resultXMLArray['cStat']." - ".$resultXMLArray['xMotivo'];
    		$idStatusNFCE = "R";
	        $params["IDSTATUSNFCE"] = $idStatusNFCE;
    		
    		//tratando mensagens em relacao aos itens retornados
	        //get function que trata mensagens
	        $params = $this->trataRetornoMensagemSefaz($params, $XMLenvio);
			TransmissaoNfceNewController::escreverLog("Nota rejeitada pela SEFAZ! ".$params["DSOBSSTATUSNFCE"]);
    		
    		$this->updateVendaNFCE($params, $XMLenvio);
		}
		
		return;
	}
	
	
	private function updateVendaNFCE($params, $xml){
		$types = array(
			'DTEMISSAONFCE' => \Doctrine\DBAL\Types\Type::DATETIME,
			'DTHRPROTOCONFCE' => \Doctrine\DBAL\Types\Type::DATETIME
		);
		if (!$this->databaseIsOracle()){
			$types['DSARQXMLNFCE'] = \Doctrine\DBAL\Types\Type::BINARY;
		}
		
		$parametros = array(
            ':NRSEQVENDA'   => $params['NRSEQVENDA'],
            ':CDCAIXA'      => $params['CDCAIXA'],
            ':CDFILIAL'     => $params['CDFILIAL']
        );
        
        $parametrosXML = $parametros;
        
        //posicao do qrcode para acrescentar '<!CDATA[ ... ]]>'
        if(strpos($xml, '<qrCode>') !== FALSE){
        	$posIn = strpos($xml, '<qrCode>');
			$posFn = strpos($xml, '</qrCode>');
			
			$qrCode = substr($xml, ($posIn + 8), ($posFn - ($posIn + 8)));
			$stQRCode = '<qrCode><![CDATA['.trim($qrCode).']]></qrCode>';
			
			$firstPart = substr($xml, 0, $posIn);
			$secondPart = substr($xml, ($posFn) + 9);
			$xml = trim($firstPart).$stQRCode.trim($secondPart);
        }
        
        $parametrosXML[':DSARQXMLNFCE'] = $xml;
		
		//verifica se ha dados em ARQXMLNFCESAT
		$resultArqXML = $this->executarQuery(TransmissaoNFCE::SELECT_VENDA_ARQXMLNFCESAT, $parametros);
		if($resultArqXML->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
        	TransmissaoNfceNewController::escreverLog("Atualizando XML!");
        	//se nao tiver dados na table ira ARQXMLNFCESAT inserir, caso tenha apenas ira updatar
            if($resultArqXML->getResult()[0]['VEND'] == 0){
            	$this->runQuery(true, TransmissaoNFCE::INSERT_ARQXMLNFCESAT_XML, $parametrosXML);
            }else{
        		$this->runQuery(true, TransmissaoNFCE::UPDATE_ARQXMLNFCESAT_XML, $parametrosXML);    	
            }
        }
        
        TransmissaoNfceNewController::escreverLog("Atualizando dados da venda!");
        
        //tratando mensagens em relacao aos itens retornados
        //get function que trata mensagens
        $params = $this->trataRetornoMensagemSefaz($params, $xml);
        
		$this->runQuery(true, TransmissaoNFCE::UPDATE_VENDA_NFCE_XML, $params, $types);
		return;
		
	}
	
	private function getIDSTATUSNFCE($cStat) {
		$idStatus = '';

		switch($cStat){
			case '100':
			case '150':
				$idStatus = 'A';
				break;
			case '105':
				$idStatus = 'T';
				break;
			case '215':
				$idStatus = 'R';
			case '110':
			case '301':
			case '302':
			case '303':
			case '999':
				$idStatus = 'D';
				break;
			default:
				$idStatus = 'R';
				break;
		}
		return $idStatus;
	}
	
	private function noSpecials($texto){
		$valor = preg_replace('/\D/', '', $texto);
		return $valor;
	}
	
	private function formatValueNrlanctonfce($nrlanctonfce){
    	$value = $nrlanctonfce;
    	$sizeNrlanctonfce = strlen($nrlanctonfce);
    	$CD_LENGTH = 8;
    	if($sizeNrlanctonfce > 8){
    		$value = substr($nrlanctonfce, 2);
    	}elseif($sizeNrlanctonfce < 8){
            $value = $this->leftZeroFill($value, $CD_LENGTH);
    	}
    	return $value;
    }
    
    private function leftZeroFill($string, $length){
        $string = str_pad(strval($string), $length, '0', STR_PAD_LEFT); 
        return $string;
    }
    
    private function getPermiteContingencia($filialInfo) {
    	if($filialInfo['SGESTADOFILI'] != 'SP' && $filialInfo['SGESTADOFILI'] != 'CE'){
    		return true;
    	}else{
    		return false;
    	}
	}
	
	private function gerarChaveAcessoNFCeNew($filialInfo, $serieCH, $nrNFE, $AMBEMISSAO, $DTEMISSAOVENDA, $nrlanctonfce){
		$nrAcessoNFCE = $filialInfo['CDESTADOIBGE'] . $DTEMISSAOVENDA->format('ym') . $filialInfo['NRINSJURFILI'] .
			self::MODELO_DOC . $serieCH . $nrNFE . $AMBEMISSAO . $nrlanctonfce;
		return $nrAcessoNFCE . self::calculaDigitoVerificadorNFe($nrAcessoNFCE);
	}
	
	private function associateCodeToMeaning(){
		return array(
			'NMRAZSOCFILI' => 'razão social',
			'SGESTADO' => 'estado',
			'NRINSJURFILI' => 'CNPJ',
			'CDINSCESTA' => 'Inscrição Estadual',
			'CSC' => 'token do CSC',
			'CSCid' => 'identificador do CSC',
			'NMARQCERTNFCE' => 'nome do arquivo do certificado digital',
			'DSSENHACERTNFCE' => 'senha do arquivo do certificado digital'
		);
	}
	
	private function nfceMissingParameterMessage($missingParameters){
		
		$pkCount = (is_array($missingParameters) ? count($missingParameters) : 0);
		$oneMissingParameter = $pkCount == 1;
		$message = $oneMissingParameter ? 'O parâmetro ' : 'Os parâmetros ';

		if($oneMissingParameter) {
			$message .= $missingParameters[0];
		} else {
			$pkCountMissing = (is_array($missingParameters) ? count($missingParameters) : 0);
			$lastIndex = $pkCountMissing - 1;
			foreach ($missingParameters as $key => $missing) {
				$message .= $missing;

				$divisor = '';
				if($key == ($lastIndex - 1)){
					$divisor = ' e ';
				} else if ($key != $lastIndex){
					$divisor = ', ';
				}
				$message .= $divisor;
			}
		}

		return $message . ' da filial estão nulos e necessitam ser parametrizados. Deve ser realizada na tela Parâmetros da Unidade -> Frente de Caixa -> Definições NFC-e / SAT';
	}
	
	private function montaArrayICMS(){
   		$icmsArray = array(
         	'vBC'		 =>	0,
         	'vICMS' 	 =>	0,
         	'vICMSDeson' =>	0,
         	'vFCP'		 => 0,
         	'vBCST' 	 =>	0,
         	'vST' 		 =>	0,
         	'vFCPST' 	 => 0,
         	'vFCPSTRet'  => 0,
    		'vProd'      =>	0,
    		'vFrete'	 =>	0,
    		'vSeg'		 =>	0,
    		'vDesc'		 =>	0,
    		'vII'		 =>	0,
    		'vIPI'		 =>	0,
    		'vIPIDevol'	 => 0,
    		'vPIS' 		 =>	0,
    		'vCOFINS'	 =>	0,
    		'vOutro'	 =>	0,
    		'vNF'		 =>	0
	    );
	    return $icmsArray;
   	}
   	
   	private function calcImpProd($VRTOTTRIBIBPT, $nrProd){
   		$perProd = floatval(bcdiv($VRTOTTRIBIBPT / $nrProd, 1, 2));

   		return array(
   			'totTrib' => $this->numberFormat($perProd * $nrProd, 2, '.', ''),
   			'perProd' => $this->numberFormat($perProd, 2, '.', '')
   		);
   	}
   	
   	private function numberFormat($number, $decimals = 2, $dec_point = '.', $thousands_sep = '') {
		
		$result = 0;
		
		try {
			$result = number_format($this->trunc($number), $decimals, $dec_point, $thousands_sep);
		} catch (\Exception $e) {
			TransmissaoNfceNewController::escreverLog($e->getTraceAsString());
			throw new \Exception($e->getMessage().' <br> Parâmetros: number '.$number);
		}
		
		return $result;
	}

	private function trunc($val, $f = "2") {
	    if (($p = strpos($val, '.')) !== false) {
	        $val = floatval(substr($val, 0, $p + 1 + $f));
	    }
	    return $val;
	}
	
	private function getSaleItemNew($cdfilial, $cdloja, $cdcaixa, $nrorg, $nrseqvenda, $nrsequitvend, $cdempresa, $cdpais, $sgestado, $cdmunicipio, $cdinscesta, $item, $cdtipooper, $VRTOTTRIBIBPTporItem) {
		
		$item_array =  $this->montaArray_item();
		$cdproduto = $item['CDPRODUTO'];
		$qtprodvend = str_replace(',', '.', $item['QTPRODVEND']);
		$vrunitvend = str_replace(',', '.', $item['VRUNITVEND']);
		$nmproduto = trim($item['NMPRODUTO']);
		
		
   		$vracritvend = isset($item['VRACRITVEND']) ? $item['VRACRITVEND'] : 0;
		$vracritvend = floatval(str_replace(',', '.', $vracritvend));
   		
		$vrdesitvend = isset($item['VRDESITVEND']) ? $item['VRDESITVEND'] : 0;
		$vrdesitvend = floatval(str_replace(',', '.', $vrdesitvend));
		
  		$item['VRALIQPIS'] = $this->trataMascaraImposto($item['VRALIQPIS']);

	  	$sgunidade = $item['SGUNIDADE'];
	  	$cdclasfisc = $item['CDCLASFISC'];
		$cdarvprod = $item['CDARVPROD'];
		
		// --- Dados de impostos
		$cdimposto = $item['CDCSTICMS'];//CDIMPOSTO
		$cstImp = $item['CDCSTPRODI'];
		$cstPisCof = $item['CDCSTPISCOF'];//CDCSTPRODPC
		$vraliquotaImp = $item['VRPEALPRODIT'];
		$vraliquotaPis = $item['VRALIQPIS'];
		$vraliquotaCof = $item['VRALIQCOFINS'];
		$vrbasecalcicms = $item['VRBCPISCOFINS'];//VRBASECALCICMS
		$cdCest = str_replace('.','',$item['CDCEST']);
		
		$pRed = ($item['VRPERCREDUCAO'] == null) ? 0 : $item['VRPERCREDUCAO'];
		$VRALIQICMSDES = ($item['VRALIQICMSDES'] == null) ? 0 : $item['VRALIQICMSDES'];
		
		$VRALIQFCP = $item['VRALIQFCP'];
		$cfopImp = $item['CDCFOPPFIS'];
		$cdcbenef = $item['CDCBENEF']; // !!!
		
	  	$vUnCom = $this->numberFormat($vrunitvend, 10, '.', '');
	  	$qCom = $this->roundABNT($qtprodvend, 4);
	  	//$qCom = $this->numberFormat(floor($qtprodvend),5, '.', '');
	  	//$qCom = $qtprodvend;
	  	
	  	$item_array['prod']['cProd'] = $cdarvprod;
	  	$item_array['prod']['cEAN'] = 'SEM GTIN';
	  	$item_array['prod']['xProd'] = preg_replace('/[^A-Za-z0-9 \-]/', '', $nmproduto);
	  	$item_array['prod']['NCM'] = $cdclasfisc;
	  	
	  	if(!empty($cdcbenef)){
	  		$item_array['prod']['cBenef'] = $cdcbenef;
	  	}else{
	  		unset($item_array['prod']['cBenef']);
	  	}
	  	
	  	$item_array['prod']['uCom'] = $sgunidade;
	  	$item_array['prod']['qCom'] = $qtprodvend;
	  	$item_array['prod']['vUnCom'] = $vUnCom;
	  	$item_array['prod']['qCom'] = $qCom;

	  	$item_array['prod']['vProd'] = $this->numberFormat(($vUnCom * $qtprodvend),2 , '.', '');
	  	$item_array['prod']['cEANTrib'] = 'SEM GTIN';
	  	$item_array['prod']['qCom'] = $qCom;
		if($vrdesitvend > 0){
	  		$item_array['prod']['vDesc'] = $this->numberFormat($vrdesitvend, 2, '.', '');
	  	}else{
	  		unset($item_array['prod']['vDesc']);
	  	}
	  	if($vracritvend > 0){
	  		$item_array['prod']['vOutro'] = $this->numberFormat($vracritvend, 2, '.', '');
	  	}else{
	  		unset($item_array['prod']['vOutro']);
	  	}
	  	$item_array['prod']['uTrib'] = $sgunidade;
     	$item_array['prod']['qTrib'] = $qCom;
     	$item_array['prod']['vUnTrib'] = $vUnCom;
     	$item_array['prod']['indTot'] = 1;//-- Compõe o total da nota

     	$baseCalcItVend = str_replace('.', ',',(floatval(str_replace(',', '.', $qtprodvend))*floatval(str_replace(',', '.', $vrunitvend))) - $vrdesitvend + $vracritvend);
		
		$orig = '0';
	   	if($cdimposto != null){
		   	$nrseqitimpos      = '001';
		   	$vrimpoprodit      = $this->valorMoeda($vraliquotaImp) * $this->valorMoeda($baseCalcItVend);
		   	$modBC             = '3';

		   	$icms_array        = $this->getICMSbyCST($cdimposto, $orig, $modBC, $vraliquotaImp, $vrbasecalcicms, $pRed, $VRALIQICMSDES, $VRALIQFCP, $baseCalcItVend);
		 	$pis_array 		   = $this->getPISbyCST($cstPisCof, $orig, $modBC, $vraliquotaPis, $vrbasecalcicms);
		 	$cofins_array 	   = $this->getCOFINSbyCST($cstPisCof, $orig, $modBC, $vraliquotaCof, $vrbasecalcicms);

		 	$tagICMS = $icms_array['TAG'];
		 	unset($icms_array['TAG']);

		 	$tagPIS = $pis_array['TAG'];
		 	unset($pis_array['TAG']);

		 	$tagCOFINS = $cofins_array['TAG'];
		 	unset($cofins_array['TAG']);

		 	$item_array['imposto']['vTotTrib'] = $VRTOTTRIBIBPTporItem;
		 	$item_array['imposto']['ICMS'][$tagICMS] = $icms_array;
		 	$item_array['imposto']['PIS'][$tagPIS] = $pis_array;
		 	$item_array['imposto']['COFINS'][$tagCOFINS] = $cofins_array;

			if(($cdimposto == '10' || $cdimposto == '30' || $cdimposto == '60' || $cdimposto == '70' || $cdimposto == '90'
				|| $cdimposto == '201' || $cdimposto == '202' || $cdimposto == '203' || $cdimposto == '500' || $cdimposto == '900')
				&& !empty($cdCest)){
		 		$item_array['prod']['CEST'] = $cdCest;
		 	}else{
		 		unset($item_array['prod']['CEST']);
		 	}

		 	$item_array['prod']['CFOP'] = $cfopImp;

		 	if($this->isComb($cfopImp)){
		 		$comb_array = $this->montaArray_comb();
		 		if(isset($cProdANP)){
		 			$comb_array['cProdANP'] = $cProdANP;
		 			$comb_array['UFCons'] = $sgestado;
		 			$comb_array['encerrante']['nBico']   =  intval($cdbico);
   					$comb_array['encerrante']['nBomba']  =  intval($cdbomba);
   					$comb_array['encerrante']['nTanque'] =  intval($cdalmoxarifado);
   					$comb_array['encerrante']['vEncIni'] =  $this->numberFormat($vrencerranteinicial,3,'.','');
   					$comb_array['encerrante']['vEncFin'] =  $this->numberFormat($vrencerrantefinal,3,'.','');
   					$item_array['prod']['comb'] = $comb_array;
   				}
		 	}
	  	}
		return $item_array;
	}
	
	private function montaArray_item(){
   		// --- IDENTIFICAÇÃO DE UM ITEM DA VENDA
		// - identificação do produto que compõe o item
		$prod = array(
			'cProd'		 => null, // CDARVPROD
			'cEAN'		 => NULL, // ?????????
			'xProd'		 => null, // NMPRODUTO
			'NCM'		 => null, // CDCLASFISC
			'cBenef'	 => null, // cBenef
			'CEST'       => null, // CDCEST
			'CFOP'		 => null, // Consulta regra fiscal
			'uCom'		 => null, // SGUNIDADE
			'qCom'		 => null, // QTPRODVEND - 4 casas decimais
			'vUnCom'	 => null, // VRUNITPROD - 10 casas decimais
			'vProd'		 => null, // Valor Bruto
			'cEANTrib'	 => null, // ????????
			'uTrib'		 => null, // SGUNIDADE
			'qTrib'		 => null, // qCom
			'vUnTrib'	 => null, // vUnCom
			'vDesc'	     => null, // VRDESITVEND
			'vOutro'	 => null,
			'indTot'	 => null  // COMPÕE O TOTAL DA NOTA: 0 - NÃO; 1 - SIM
		);

		$imposto = array(
			'vTotTrib' => null,
			'ICMS'     => array(),
			'PIS'      => array(),
			'COFINS'   => array()
		);

		// - identificação do imposto de um item
		$item = array(
			'prod' 	  => $prod,
			'imposto' => $imposto
		);

		return $item;
   	}
   	
   	private function trataMascaraImposto($valor) {
		$valor = str_replace(' ', '', $valor);
		$valor = str_replace(',', '.', $valor);
		return floatval($valor);
	}
	
	private function valorMoeda($valorAplicado){
		
		$posVirgula = strpos( $valorAplicado, "," );
		$posPont = strpos( $valorAplicado, "." );
		
		if (!$posVirgula === false && !$posPont === false) {
			$valor = str_replace("." , "" , $valorAplicado ); // Primeiro tira os pontos
			$valorAplicado2 = str_replace(',','.',$valor);
		}else if(!$posVirgula === false && $posPont === false){
			$valorAplicado2 = str_replace(',','.',$valorAplicado);
		}else {
			$valorAplicado2 = $valorAplicado;
		}
		
		return $valorAplicado2;
	}
	
	private function getICMSbyCST($cstImp, $orig, $modBC, $vraliquotaImp, $vrbasecalcicms, $pRed, $VRALIQICMSDES = null, $VRALIQFCP = null, $baseCalcItVend = null){
		$icms_array = array();
		$tagXML = '';
		$vraliquotaImp = $vraliquotaImp/100;
		if($cstImp == '00'){
			$tagXML = 'ICMS00';
			$icms_array = $this->montaArray_ICMS00();
			$icms_array['orig']  = 	$orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);

			$vICMS = (double) $vraliquotaImp * (double) $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2);
			$vICMS = $this->numberFormat($this->roundABNT($vICMS, 2),2, '.', '');
			$icms_array['vICMS'] = $vICMS;
			
			if(!empty($VRALIQFCP) && $VRALIQFCP > 0){
				
				//doVrBaseCalc := Trunc2(doVrBaseCalc - (doVrBaseCalc * cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat / 100));
				$VrBaseCalc = $this->valorMoeda($baseCalcItVend);
				$VraliqfcpCalc = $this->valorMoeda($VRALIQFCP);
				
				$VrIcmsFCP = $VrBaseCalc * ($VraliqfcpCalc/100) ;
				//doVrIcmsFCP := RoundABNT(doVrBaseCalc * (cdsVendaNFCe.FieldByName('VRALIQFCP').AsFloat/100),2);
				
				$VrIcmsFCP = $this->numberFormat($this->roundABNT($VrIcmsFCP, 2),2, '.', ''); 

				$icms_array['pFCP']   =  $VraliqfcpCalc;
				$icms_array['vFCP']   =  $VrIcmsFCP;
				
				//var_dump($icms_array);
			}
			
		}else if($cstImp == '10'){
			$tagXML = 'ICMS10';
			$icms_array = $this->montaArray_ICMS10();
			$icms_array['orig']  = 	$orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $this->numberFormat($this->roundABNT($vICMS, 2),2, '.', '');
		}else if($cstImp == '20'){
			$tagXML = 'ICMS20';
			$icms_array = $this->montaArray_ICMS20();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['pRedBC'] = $this->numberFormat(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $this->numberFormat($this->roundABNT($vICMS, 2),2, '.', '');
			
			if(!empty($VRALIQICMSDES) && $VRALIQICMSDES > 0){
				
				$VrBaseCalcDes = $this->valorMoeda($baseCalcItVend);
				$VrAliqIcmsDes = $this->valorMoeda($VRALIQICMSDES) / 100;
				$VrIcmsDes = $VrBaseCalcDes * (1 - ($VrAliqIcmsDes * (1 - ($pRed/100)) ) ) / (1 - $VrAliqIcmsDes) - $VrBaseCalcDes ;
				$VrIcmsDes = $this->numberFormat($this->roundABNT($VrIcmsDes, 2),2, '.', ''); 
				if($VrIcmsDes < 0.01){
					$VrIcmsDes = 0.01;
				}
				$icms_array['vICMSDeson'] = $VrIcmsDes;
				$icms_array['motDesICMS']   =  '9';
			}
			
			if(!empty($VRALIQFCP) && $VRALIQFCP > 0){
				//doVrBaseCalc := Trunc2(doVrBaseCalc - (doVrBaseCalc * cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat / 100));
				$VrBaseCalc = $this->valorMoeda($baseCalcItVend);
				$VraliqfcpCalc = ($this->valorMoeda($VRALIQFCP) * $pRed) / 100;
				
				$VrIcmsFCP = $VrBaseCalc * ($VraliqfcpCalc/100) ;
				//doVrIcmsFCP := RoundABNT(doVrBaseCalc * (cdsVendaNFCe.FieldByName('VRALIQFCP').AsFloat/100),2);
				
				$VrIcmsFCP = $this->numberFormat($this->roundABNT($VrIcmsFCP, 2),2, '.', ''); 

				$icms_array['vBCFCP'] =  $VrBaseCalc;
				$icms_array['pFCP']   =  $VraliqfcpCalc;
				$icms_array['vFCP']   =  $VrIcmsFCP;
			}
			
			
		}else if($cstImp == '30'){
			$tagXML = 'ICMS30';
			$icms_array = $this->montaArray_ICMS30();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '40'){
			$tagXML = 'ICMS40';
			$icms_array = $this->montaArray_ICMS40();
			
			if(!empty($VRALIQICMSDES) && $VRALIQICMSDES > 0){
				
				$VrBaseCalcDes = $this->valorMoeda($baseCalcItVend);
				$VrAliqIcmsDes = $this->valorMoeda($VRALIQICMSDES) / 100;
				//$VrIcmsDes = $VrBaseCalcDes * (1 - ($VrAliqIcmsDes * (1 - ($pRed/100)) ) ) / (1 - $VrAliqIcmsDes) - $VrBaseCalcDes ;
				$VrIcmsDes =   ($VrBaseCalcDes / (1 - $VrAliqIcmsDes)) * $VrAliqIcmsDes;
				$VrIcmsDes = $this->numberFormat($this->roundABNT($VrIcmsDes, 2),2, '.', ''); 
				if($VrIcmsDes < 0.01){
					$VrIcmsDes = 0.01;
				}
				$icms_array['vICMSDeson'] = $VrIcmsDes;
				$icms_array['motDesICMS']   =  '9';
			}
			
			$icms_array['orig']  = $orig;
		}else if($cstImp == '41'){
			$tagXML = 'ICMS40';
			$icms_array = $this->montaArray_ICMS41();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '50'){
			$tagXML = 'ICMS50';
			$icms_array = $this->montaArray_ICMS50();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '51'){
			$tagXML = 'ICMS51';
			$icms_array = $this->montaArray_ICMS51();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['pRedBC'] = $this->numberFormat(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = $this->numberFormat($this->roundABNT(substr($vICMSStr,0,$dotPos+3), 2));
		}else if($cstImp == '60'){
			$tagXML = 'ICMS60';
			$icms_array = $this->montaArray_ICMS60();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '70'){
			$tagXML = 'ICMS70';
			$icms_array = $this->montaArray_ICMS70();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['pRedBC'] = $this->numberFormat(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $this->numberFormat($this->roundABNT($vICMS, 2),2, '.', '');
		}else if($cstImp == '90'){
			$tagXML = 'ICMS90';
			$icms_array = $this->montaArray_ICMS90();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pRedBC'] = $this->numberFormat(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $this->numberFormat($this->roundABNT($vICMS, 2),2, '.', '');
			
			if(!empty($VRALIQICMSDES) && $VRALIQICMSDES > 0){
				
				$VrBaseCalcDes = $this->valorMoeda($baseCalcItVend);
				$VrAliqIcmsDes = $this->valorMoeda($VRALIQICMSDES) / 100;
				$VrIcmsDes = $VrBaseCalcDes * (1 - ($VrAliqIcmsDes * (1 - ($pRed/100)) ) ) / (1 - $VrAliqIcmsDes) - $VrBaseCalcDes ;
				$VrIcmsDes = $this->numberFormat($this->roundABNT($VrIcmsDes, 2),2, '.', ''); 
				if($VrIcmsDes < 0.01){
					$VrIcmsDes = 0.01;
				}
				$icms_array['vICMSDeson'] = $VrIcmsDes;
				$icms_array['motDesICMS']   =  '9';
			}
			
			if(!empty($VRALIQFCP) && $VRALIQFCP > 0){
				
				//doVrBaseCalc := Trunc2(doVrBaseCalc - (doVrBaseCalc * cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat / 100));
				
				$VrBaseCalc = $this->valorMoeda($baseCalcItVend);
				$VraliqfcpCalc = ($this->valorMoeda($VRALIQFCP) * $pRed) / 100;
				
				$VrIcmsFCP = $VrBaseCalc * ($VraliqfcpCalc/100) ;
				//doVrIcmsFCP := RoundABNT(doVrBaseCalc * (cdsVendaNFCe.FieldByName('VRALIQFCP').AsFloat/100),2);
				
				$VrIcmsFCP = $this->numberFormat($this->roundABNT($VrIcmsFCP, 2),2, '.', ''); 

				$icms_array['vBCFCP'] =  $VrBaseCalc;
				$icms_array['pFCP']   =  $VraliqfcpCalc;
				$icms_array['vFCP']   =  $VrIcmsFCP;
			}
			
			
		}else if($cstImp == '102' || $cstImp == '103' || $cstImp == '300' || $cstImp == '400'){
            $tagXML = 'ICMSSN102';
            $icms_array = array();
            $icms_array['orig'] = '0';
            $icms_array['CSOSN'] = '102';
        }else if($cstImp == '500'){
            $tagXML = 'ICMSSN500';
            $icms_array = array();
            $icms_array['orig'] = '0';
			$icms_array['CSOSN'] = '500';
		}else if($cstImp == '900'){
            $tagXML = 'ICMSSN900';
            $icms_array = array();
            $icms_array['orig'] = '0';
			$icms_array['CSOSN'] = '900';
			$icms_array['modBC'] = '3';
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pRedBC'] = $this->numberFormat(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $this->numberFormat($this->roundABNT($vICMS, 2),2, '.', '');
        }
		$icms_array['TAG'] = $tagXML;
		return $icms_array;
	}
	
	private function montaArray_ICMS00(){
   		return array(
   			'orig' => null,
   			'CST' => '00',
   			'modBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   	}

   	private function montaArray_ICMS10(){
   		return array(
   			'orig' => null,
   			'CST' => '10',
   			'modBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   	}

   	private function montaArray_ICMS20(){
   		return array(
   			'orig' => null,
   			'CST' => '20',
   			'modBC' => null,
   			'pRedBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   	}

   	private function montaArray_ICMS30(){
   		return array(
   			'orig' => null,
   			'CST' => '30'
   		);
   	}

   	private function montaArray_ICMS40(){
		return array(
   			'orig' => null,
   			'CST' => '40'
   		);
   	}

   	private function montaArray_ICMS41(){
		return array(
   			'orig' => null,
   			'CST' => '41'
   		);
   	}

   	private function montaArray_ICMS50(){
		return array(
   			'orig' => null,
   			'CST' => '50'
   		);
   	}

   	private function montaArray_ICMS51(){
		return array(
   			'orig' => null,
   			'CST' => '51',
   			'modBC' => null,
   			'pRedBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   	}

   	private function montaArray_ICMS60(){
		return array(
   			'orig' => null,
   			'CST' => '60'
   		);
   	}

   	private function montaArray_ICMS70(){
		return array(
   			'orig'    => null,
   			'CST'     => '70',
   			'modBC'   => null,
   			'pRedBC'  => null,
   			'vBC'     => null,
   			'pICMS'   => null,
   			'vICMS'   => null,
   			'modBCST' => '3',
            'pMVAST'  => '0.00',
            'pRedBCST'=> '0.00',
            'vBCST'   => '0.00',
            'pICMSST' => '0.00',
            'vICMSST' => '0.00'
   		);
   	}

   	private function montaArray_ICMS90(){
   		return array(
   			'orig'    => null,
   			'CST'     => '90',
   			'modBC'   => null,
   			'vBC'     => null,
   			'pRedBC'  => null,
   			'pICMS'   => null,
   			'vICMS'   => null
   		);
   	}
   	
   	private function getPISbyCST($cstPis, $orig, $modBC, $vraliquotaPis, $vrbasecalcicms){
		$pis_array = array();
		$vraliquotaPis = $vraliquotaPis / 100;
		$vrbasecalcicms = str_replace(',', '.', $vrbasecalcicms);
		$tagXML = '';
		if ($cstPis == '01' || $cstPis == '02') {
			$pis_array = $this->montaArray_PISAliq();
			$tagXML = 'PISAliq';
			$pis_array['pPIS'] = $this->numberFormat($vraliquotaPis * 100, 2);
			$pis_array['vBC']  = $this->numberFormat($vrbasecalcicms, 2, '.', '');
			$pis_array['vPIS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaPis, 2);
		} else if ($cstPis == '03') {
			$tagXML = 'PISQtde';
			$pis_array = $this->montaArray_PISQtde();
			$pis_array['pPIS'] = $this->numberFormat($vraliquotaPis * 100, 2);
			$pis_array['vBC']  = $this->numberFormat($vrbasecalcicms, 2, '.', '');
			$pis_array['vPIS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaPis, 2);
		} else if ($cstPis == '04' || $cstPis == '05' || $cstPis == '06' || $cstPis == '07' || $cstPis == '08' || $cstPis == '09'){
			$tagXML = 'PISNT';
			$pis_array = $this->montaArray_PISNT();
		} else if ($cstPis == '49' || $cstPis == '50' || $cstPis == '51' ||
			       $cstPis == '52' || $cstPis == '53' || $cstPis == '54' || $cstPis == '55' ||
			       $cstPis == '56' || $cstPis == '60' || $cstPis == '61' || $cstPis == '62' ||
			       $cstPis == '63' || $cstPis == '64' || $cstPis == '65' || $cstPis == '66' ||
			       $cstPis == '67' || $cstPis == '70' || $cstPis == '71' || $cstPis == '72' ||
			       $cstPis == '73' || $cstPis == '74' || $cstPis == '75' || $cstPis == '98' ||
			       $cstPis == '99') {
			$tagXML = 'PISOutr';
			$pis_array = $this->montaArray_PISOutr();
			$pis_array['pPIS'] = $this->numberFormat($vraliquotaPis * 100, 2);
			$pis_array['vBC']  = $this->numberFormat($vrbasecalcicms, 2, '.', '');
			$pis_array['vPIS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaPis, 2);
		}
		$pis_array['CST'] = $cstPis;
		$pis_array['TAG'] = $tagXML;
		return $pis_array;
	}

   	private function montaArray_PISAliq(){
		return array(
   			'CST'  => null,
   			'vBC'  => null,
			'pPIS' => null,
			'vPIS' => null
   		);
   	}

   	private function montaArray_PISNT(){
		return array(
   			'CST'  => null
   		);
   	}

   	private function montaArray_PISQtde(){
		return array(
   			'CST'  => null,
   			'vBC'  => null,
			'pPIS' => null,
			'vPIS' => null
   		);
   	}

   	private function montaArray_PISOutr(){
		return array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pPIS' => null,
			'vPIS' => null
   		);
   	}
	
	private function getCOFINSbyCST($cstCof, $orig, $modBC, $vraliquotaCof, $vrbasecalcicms){
		$cof_array = array();
		$tagXML = '';
		$vraliquotaCof  = $this->valorMoeda($vraliquotaCof);
		$vrbasecalcicms = $this->valorMoeda($vrbasecalcicms);
		$vraliquotaCof = $vraliquotaCof/100;
		if($cstCof == '01' || $cstCof == '02'){
			$tagXML = 'COFINSAliq';
			$cof_array = $this->montaArray_COFINSAliq();
			$cof_array['pCOFINS'] = $this->numberFormat($vraliquotaCof*100,2);
			$cof_array['vBC']     = $this->numberFormat($vrbasecalcicms,2, '.', '');
			$cof_array['vCOFINS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaCof,2);
		}else if($cstCof == '03'){
			$tagXML = 'COFINSQtde';
			$cof_array = $this->montaArray_COFINSQtde();
			$cof_array['pCOFINS'] = $this->numberFormat($vraliquotaCof*100,2);
			$cof_array['vBC']     = $this->numberFormat($vrbasecalcicms,2, '.', '');
			$cof_array['vCOFINS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaCof,2);
		}else if($cstCof == '04'  || $cstCof == '05' || $cstCof == '06' || $cstCof == '07' || $cstCof == '08' || $cstCof == '09'){
			$tagXML = 'COFINSNT';
			$cof_array = $this->montaArray_COFINSNT();
		}else if($cstCof == '49' || $cstCof == '50' || $cstCof == '51' ||
			     $cstCof == '52' || $cstCof == '53' || $cstCof == '54' || $cstCof == '55' ||
			     $cstCof == '56' || $cstCof == '60' || $cstCof == '61' || $cstCof == '62' ||
			     $cstCof == '63' || $cstCof == '64' || $cstCof == '65' || $cstCof == '66' ||
			     $cstCof == '67' || $cstCof == '70' || $cstCof == '71' || $cstCof == '72' ||
			     $cstCof == '73' || $cstCof == '74' || $cstCof == '75' || $cstCof == '98' ||
			     $cstCof == '99'){
			$tagXML = 'COFINSOutr';
			$cof_array = $this->montaArray_COFINSOutr();
			$cof_array['pCOFINS'] = $this->numberFormat($vraliquotaCof*100,2);
			$cof_array['vBC']     = $this->numberFormat($vrbasecalcicms,2, '.', '');
			$cof_array['vCOFINS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaCof,2);
		}

		$cof_array['CST'] = $cstCof;
		$cof_array['TAG'] = $tagXML;

		return $cof_array;
	}
	
   	private function montaArray_COFINSAliq() {
   		return array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pCOFINS' => null,
			'vCOFINS' => null
   		);
   	}

   	private function montaArray_COFINSQtde() {
		return array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pCOFINS' => null,
			'vCOFINS' => null
   		);
   	}

   	private function montaArray_COFINSNT() {
   		return array(
   			'CST' => null
   		);
   	}

   	private function montaArray_COFINSOutr() {
   		return array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pCOFINS' => null,
			'vCOFINS' => null
   		);
   	}
   	
   	private function isComb($cfop){
		$isComb = false;
		if($cfop =='1651' || $cfop =='1652' || $cfop =='1653' || $cfop =='1658' || $cfop =='1659' ||
		   $cfop =='1660' || $cfop =='1661' || $cfop =='1662' || $cfop =='1663' || $cfop =='1664' ||
		   $cfop =='2651' || $cfop =='2652' || $cfop =='2653' || $cfop =='2658' || $cfop =='2659' ||
		   $cfop =='2660' || $cfop =='2661' || $cfop =='2662' || $cfop =='2663' || $cfop =='2664' ||
		   $cfop =='3651' || $cfop =='3652' || $cfop =='3653' || $cfop =='5651' || $cfop =='5652' ||
		   $cfop =='5653' || $cfop =='5654' || $cfop =='5655' || $cfop =='5656' || $cfop =='5657' ||
		   $cfop =='5658' || $cfop =='5659' || $cfop =='5660' || $cfop =='5661' || $cfop =='5662' ||
		   $cfop =='5663' || $cfop =='5664' || $cfop =='5665' || $cfop =='5666' || $cfop =='5667' ||
		   $cfop =='6651' || $cfop =='6652' || $cfop =='6653' || $cfop =='6654' || $cfop =='6655' ||
		   $cfop =='6656' || $cfop =='6657' || $cfop =='6658' || $cfop =='6659' || $cfop =='6660' ||
		   $cfop =='6661' || $cfop =='6662' || $cfop =='6663' || $cfop =='6664' || $cfop =='6665' ||
		   $cfop =='6666' || $cfop =='6667' || $cfop =='7651' || $cfop =='7654' || $cfop =='7667') {

				$isComb = true;
		}

		return $isComb;
	}
	
	private function montaArray_comb(){
   		$encerrante  = array(
   			'nBico'   =>  null,
   			'nBomba'  =>  null,
   			'nTanque' =>  null,
   			'vEncIni' =>  null,
   			'vEncFin' =>  null
   		);
   		$comb = array(
   			'cProdANP'	=> null,
   			'UFCons'	=> null,
   			'encerrante' => $encerrante
   		);
   	}
   	
   	private function updateTotal($property_array,&$total_xml, $icms){
		$ICMSTot = $total_xml['ICMSTot'];
		$vBCAcumulado = $ICMSTot['vBC'];
		$ICMSTot['vBC'] = 0;
		foreach($ICMSTot as $tKey=>$totalProperty){
			$this->updateTotalProperty($tKey,$totalProperty, $property_array);
			$ICMSTot[$tKey] = $this->numberFormat($totalProperty,2 ,'.', '');
		}

		$this->updateVBCTot($ICMSTot,$icms);

		$ICMSTot['vBC'] = $this->numberFormat($ICMSTot['vBC'] + $vBCAcumulado,2 ,'.', '');
		$total_xml['ICMSTot'] = $ICMSTot;
		return $total_xml;
	}
	
	private function updateTotalProperty($propertyName, &$propertyValue, $arrayToCompare){
		foreach($arrayToCompare as $pKey=>$item){
			if(is_array($item)){
			   $this->updateTotalProperty($propertyName,$propertyValue,$item);
			}else{
				if($propertyName == $pKey && $propertyName != 'vBC'){
					$propertyValue = $propertyValue + $item;
				}
				// else if($propertyName == $pKey && $propertyName == 'vBC'){
				// 	$propertyValue = $item;
				// }
			}
		}
	}
	
	private function updateVBCTot(&$ICMSTot,$icms){
		$icmsArray  = array_values($icms);
		$icmsNode  = $icmsArray[0];
		if(isset($icmsNode['vBC'])){
			$ICMSTot['vBC'] += (double)$icmsNode['vBC'];
		}
	}
	
	private function calcTotalNF($totalArray){
   		return $this->numberFormat(round($totalArray['ICMSTot']['vProd'] - $totalArray['ICMSTot']['vDesc'] + $totalArray['ICMSTot']['vOutro'], 2), 2, '.', '');
   	}
   	
   	private function getPayment($payment, $IDNOTATECNFCE = null) {
        $payment_array = $this->montaArray_pag();
		$idtiporeceSEFAZ = $this->getSEFAZIdtiporece($payment['IDTIPORECE']);
		$payment_array['tPag'] = $idtiporeceSEFAZ;
		
		//necessario para norma tecnica 2020.006 quando 120 e ser outros
        if($idtiporeceSEFAZ == '99' && $IDNOTATECNFCE == '120'){
        	$payment_array['xPag'] = $payment['NMTIPORECE'];
        }else{
        	unset($payment_array['xPag']);
        }
		
		$payment_array['vPag'] = $this->numberFormat($payment['VRMOVIVEND'], 2, '.', '');
		if($idtiporeceSEFAZ == '03' || $idtiporeceSEFAZ == '04'){
            $payment_array['card'] = $this->montaArray_card($payment['IDDESABTEF'], $payment['CDADMINCART'], $payment['CDBANCARTSEFAZ'], $payment['CDNSUHOSTTEF']);
        }
        
		return $payment_array;
	}
	
	private function montaArray_pag(){
   		// --- IDENTIFICAÇÃO DE UM PAGAMENTO DA VENDA
		return array(
			'tPag' => null, // TIPO DE PAGAMENTO - idtiporeceSEFAZ
			'xPag' => null, // Descricao do Tipo de Pagamento, necessario para norma tecnica 2020.006 quando 120 e ser outros
			'vPag' => null, // VALOR DO PAGAMENTO - VRMOVIVEND
		);
   	}
   	
   	private function getSEFAZIdtiporece($idtiporeceSAAS){
   		$idtiporeceSEFAZ = '99'; //Outros
   		switch($idtiporeceSAAS){ 
   			//DINHEIRO,OUTRASMOEDAS
   			case '4':
   			case '6':
   				$idtiporeceSEFAZ = '01'; //Dinheiro
   				break;
   			//CHEQUE
   			case '3':
   				$idtiporeceSEFAZ = '02'; //Cheque
   				break;
   			// CARTAOCREDITO
   			case '1':
   				$idtiporeceSEFAZ = '03'; //Cartao de Credito
   				break;
   			// CARTAODEBITO
   			case '2':
   				$idtiporeceSEFAZ = '04'; //Cartao de Debito
   				break;
   			//CONVENIO,CRE_PESSOAL,DEB_CONSUM,CartaoChart
   			case '8':
   			case '9':
   			case 'A':
   			case 'E':
   				$idtiporeceSEFAZ = '05'; //Crédito Loja
   				break;
   			// VOUCHERALIMENTACAO
   			case 'F':
   				$idtiporeceSEFAZ = '10'; //Vale Alimentação
   				break;
   			// TICKET,VALES,VOUCHERREFEICAO
   			case '5':
   			case '7':
   			case 'G':
   				$idtiporeceSEFAZ = '11'; //Vale Refeição
   				break;
   			// CARTEIRADIGITALIT
   			case 'H':
   				$idtiporeceSEFAZ = '18'; //Transferência bancária, Carteira Digital
   				break;
   			// PagamentoSite,CARTAME
   			case 'C':
   			case 'I':
   				$idtiporeceSEFAZ = '19'; //Programa de fidelidade, Cashback, Crédito Virtual
   				break;
   			// Devolucao
   			case 'B':
   				$idtiporeceSEFAZ = '90'; //Sem pagamento
   				break;
   			default:
   				break;
	   	}
	   	return $idtiporeceSEFAZ;
	}
	
	private function montaArrayDest($IDAMBTRABNFCE, $infoConsumer){
   		// --- IDENTIFICAÇÃO DO DESTINATARIO
   		$dest_array = array();
   		$cpfCNPJ = preg_replace('/[^0-9]/', '', $infoConsumer['NRINSCRCONS']);
   		if (strlen($cpfCNPJ) == 11) {
   			$dest_array['CPF'] = $cpfCNPJ;
   		} else {
   			$dest_array['CNPJ'] = $cpfCNPJ;
   		}
   		$xNome = '';
   		if ($IDAMBTRABNFCE == 1){
        	if (!empty($infoConsumer['NMCONSVEND'])){
        		$xNome = trim($this->formataCaracteresXML($infoConsumer['NMCONSVEND']));
        	} else if (!empty($infoConsumer['NMCONSUMIDOR'])){
        		$xNome = trim($this->formataCaracteresXML($infoConsumer['NMCONSUMIDOR']));
        	} else {
        		$xNome = 'CONSUMIDOR FINAL';
        	}
   		} else {
   			$xNome = 'NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL';
   		}
   		$dest_array['xNome'] = $xNome;

   		if (!empty($infoConsumer['DSENDECONSVENDA']) && strlen($infoConsumer['DSENDECONSVENDA']) > 3){
   			$dest_array['enderDest'] = array(
   				'xLgr' => trim($this->formataCaracteresXML($infoConsumer['DSENDECONSVENDA'])),
				'nro' => '000',
				'xBairro' => $infoConsumer['NMBAIRIEST'],
				'cMun' => strlen($infoConsumer['CDMUNICIBGE']) == 7 ? $infoConsumer['CDMUNICIBGE'] : $infoConsumer['CDESTADOIBGE'] . $infoConsumer['CDMUNICIBGE'],
				'xMun' => $infoConsumer['NMMUNICIPIO'],
				'UF' => $infoConsumer['SGESTADO'],
				'CEP' => '00000000',
				'cPais' => '1058',
				'xPais' => 'BRASIL'
   			);
   		}
   		$dest_array['indIEDest'] = '9';
		return $dest_array;
   	}
   	
   	private function gerarXMLValidado($saleArray, $nrAcessoNFCE, $contingencia, $sign = true){
        $unsignedXML = $this->gerarXML($saleArray,$nrAcessoNFCE, $contingencia);
        if($sign){
        	$XMLASSINADO = $this->callSignNFCe($unsignedXML);
   			return $XMLASSINADO;
        }else{
        	return $unsignedXML;
        }
	}
	
	private function gerarXML($saleArray, $nrAcessoNFCE, $contingencia){
        $dom = $this->generatesStringXMLByIndexedArray($saleArray, $nrAcessoNFCE, $contingencia);
        // trata xml
	    $XML = $this->clearXmlString($dom->saveXML());
		return $XML;
	}
	
	
	private function generatesStringXMLByIndexedArray($saleArray, $nrAcessoNFCE, $contingencia){
		
		$dom = new \DOMDocument('1.0', 'utf-8');
		//------ Atributos
		$xmlns = $dom->createAttribute('xmlns');
		$xmlns->value = 'http://www.portalfiscal.inf.br/nfe';

        //------ NFe
        $NFe = $dom->createElement('NFe');
        $NFe->appendChild($xmlns);
        $dom->appendChild($NFe);

        //------ infNFe
        $infNFe = $dom->createElement('infNFe');
        $Id = $dom->createAttribute('Id');
        $infNFeVersao = $dom->createAttribute('versao');
        $Id->value = 'NFe'.$nrAcessoNFCE;
        $infNFeVersao->value = self::VERSAO_LAYOUT;

		$infNFe->appendChild($Id);
		$infNFe->appendChild($infNFeVersao);

		$NFe->appendChild($infNFe);
		


		$parametros = array(
	            ':CDFILIAL' => $saleArray['infAdic']['filial'],
	            ':CDCAIXA' => $saleArray['infAdic']['caixa'],
	            ':CDLOJA' => $saleArray['infAdic']['loja']
	            
	        );
	
		$result = $this->executarQuery(TransmissaoNFCE::GET_DSOBSCUPFIS, $parametros);
		$resposta = $result->getResult();


		
		if ( $saleArray['emit']['enderEmit']['UF'] == 'RJ' && $saleArray['ide']['tpEmis'] == 9) {
			$saleArray['ide']['dhCont'] = $saleArray['ide']['dhEmi'];
            $saleArray['ide']['xJust'] = 'Motivo Emissao em Contingencia: Caixa OffLine';
    		$saleArray['infAdic']['infCpl'] ='NFCe EMITIDA EM CONTIGENCIA - PROCOM RJ - FONE 151 - Av. Rio Branco, 25 - Centro, Rio de Janeiro RJ, 20.090-030 (LEI 5.817/2010)'.(!$resposta[0]['DSOBSCUPFIS'] ? '' : ' '.$resposta[0]['DSOBSCUPFIS']).' Trib aprox R$ '.number_format($saleArray['total']['ICMSTot']['vTotTrib'], 2, '.', '');
    		ksort($saleArray['infAdic']);
		}else if ($saleArray['emit']['enderEmit']['UF'] == 'RJ' && $saleArray['ide']['tpEmis'] != 9){
    		$saleArray['infAdic']['infCpl'] = 'PROCOM RJ - FONE 151 - Av. Rio Branco, 25 - Centro, Rio de Janeiro - RJ, 20.090-030 (LEI 5.817/2010)'.(!$resposta[0]['DSOBSCUPFIS'] ? '' : ' '.$resposta[0]['DSOBSCUPFIS']).' Trib aprox R$ '.number_format($saleArray['total']['ICMSTot']['vTotTrib'], 2, '.', '');
			ksort($saleArray['infAdic']);
		}else if ($saleArray['emit']['enderEmit']['UF'] != 'RJ' && $saleArray['ide']['tpEmis'] == 9){
			$saleArray['ide']['dhCont'] = $saleArray['ide']['dhEmi'];
            $saleArray['ide']['xJust'] = 'Motivo Emissao em Contingencia: Caixa OffLine';
    		$saleArray['infAdic']['infCpl'] ='NFCe EMITIDA EM CONTIGENCIA';
    		ksort($saleArray['infAdic']);
		}else if(($saleArray['emit']['enderEmit']['UF'] != 'RJ') && ($saleArray['ide']['tpEmis'] != 9) && ($resposta[0]['DSOBSCUPFIS'])){
    		$saleArray['infAdic']['infCpl'] = $resposta[0]['DSOBSCUPFIS'].' Trib aprox R$ '.number_format($saleArray['total']['ICMSTot']['vTotTrib'], 2, '.', '');
			ksort($saleArray['infAdic']);
		}
		
		
//Retirar campos filial, caixa e loja antes de validar o xml, esses campos são parametros utilizados para rodar a query GET_DSOBSCUPFIS	
		unset($saleArray['infAdic']['filial'],$saleArray['infAdic']['caixa'],$saleArray['infAdic']['loja']);

        $this->arrayElementsToXML($saleArray,$infNFe,$dom);
     	return $dom;
	}
	
	private function arrayElementsToXML($array,&$currentTag,&$dom){
		foreach($array as $key=>$value){
			$newElement =  null;
			$keyAsArray = explode('+',$key);
			
			$pkCount = (is_array($keyAsArray) ? count($keyAsArray) : 0);
			if($pkCount > 1){
				$key = $keyAsArray[0];
				$newElement = $dom->createElement($key);
				if($key == 'det' || $key == 'obsCont'){
					$attrName = $keyAsArray[1];
					$attrValue = $keyAsArray[2];
					$attribute = $dom->createAttribute($attrName);
					$attribute->value = $attrValue;
					$newElement->appendChild($attribute);
				}
				$currentTag->appendChild($newElement);
				if(is_array($value)){
					$this->arrayElementsToXML($value,$newElement,$dom);
				}else{
					$newElement->nodeValue = $value;
				}
			}else{
				if(is_array($value)){
					$newElement = $dom->createElement($key);
					$currentTag->appendChild($newElement);
					$this->arrayElementsToXML($value,$newElement,$dom);
				}else{
					$newElement = $dom->createElement($key,$value);
					$currentTag->appendChild($newElement);
				}
			}
        }
	}
	
	private static function clearXmlString($string, $removeEncodingTag = false){
        $aFind = array(
            'xmlns:default="http://www.w3.org/2000/09/xmldsig#"',
            ' standalone="no"',
            'default:',
            ':default',
            "\n",
            "\r",
            "\t"
        );
        $retXml = str_replace($aFind, "", $string);
        $retXml = preg_replace('/(\>)\s*(\<)/m', '$1$2', $retXml);
        if ($removeEncodingTag) {
            $retXml = $this->deleteAllBetween($retXml, '<?xml', '?>');
        }
        return $retXml;
    }
    
    private static function deleteAllBetween($string, $beginning, $end){
        $beginningPos = strpos($string, $beginning);
        $endPos = strpos($string, $end);
        if ($beginningPos === false || $endPos === false) {
            return $string;
        }
        $textToDelete = substr($string, $beginningPos, ($endPos + strlen($end)) - $beginningPos);
        return str_replace($textToDelete, '', $string);
    }
    
    /*
    Funcao para assinar NFCe XML
    */
    private function callSignNFCe($unsignedXML){
        try {
    		
    		$XMLASSINADO = $this->nfeTools->signNFe($unsignedXML);
		    if(!empty($XMLASSINADO)){
				return $XMLASSINADO;
		    }else{
		    	throw new \Exception("Não houve retorno ao tentar assinar XML da nota!");
		    }
			
		} catch (\Exception $e) {
			TransmissaoNfceNewController::escreverLog("Erro ao tentar assinar XML da nota! ". $this->exceptionMessage($e->getMessage()));
			throw new \Exception("Erro ao tentar assinar XML da nota!");
		}
    }
    
    private function montaArray_card($IDDESABTEF, $CDADMINCART, $CDBANCARTSEFAZ, $CDNSUHOST){
   		if ($IDDESABTEF == 'N'){
	   		return array(
				'tpIntegra' => 1,
				'CNPJ' => !empty($CDADMINCART) ? preg_replace('/[^0-9]/', '', $CDADMINCART) : '00000000000000',
				'tBand' => !empty($CDBANCARTSEFAZ) ? $CDBANCARTSEFAZ : '99',
				'cAut' => !empty($CDNSUHOST) ? $CDNSUHOST :'000000000'
			);
   		} else {
   			return array(
   				'tpIntegra' => 2
   			);
   		}
   	}
   	
   	// remove this
    private function databaseIsOracle() {
        return self::ORACLE;
    }
    
    private function roundABNT($valor, $casasdecimais) {
        $pot10 = pow(10, $casasdecimais);
    
        $trunc = floor($valor * $pot10) / $pot10;
        $valorDescartado = abs($valor) - abs($trunc);
    
        $limiteDescarte = 0.5 / $pot10;
        $incrementoUltimaCasa = 1 / $pot10;
        if ($this->floatEquals($valorDescartado, $limiteDescarte)) {
            return $this->isEven($trunc * $pot10) ? $trunc : $trunc + $incrementoUltimaCasa;            
        } else if (abs($valorDescartado) < $limiteDescarte) {
            return $trunc;
        } else if (abs($valorDescartado) > $limiteDescarte) {
            return $trunc + $incrementoUltimaCasa;
        }
    
        return $result;
    }
    
    private function floatEquals($a, $b) {
        $precision = 13;
        return round($a, $precision) == round($b, $precision);
    }
    
    public function isEven($a) {
        $intA = round($a, 0); 
        if ($this->floatEquals($intA, $a)) {
            return $intA % 2 == 0;
        } else {
			throw new Error("Argument passed to function 'isEven' is not an integer");
        }        
    }
    
    private function formataCaracteresXML($strXML){
		$formattedXML = strtr(
	            $strXML,
     			array (
			      'À' => 'A', 'Á' => 'A', 'Â' => 'A', 'Ã' => 'A', 'Ä' => 'A', 'Å' => 'A',
      			  'Æ' => 'A', 'Ç' => 'C', 'È' => 'E', 'É' => 'E', 'Ê' => 'E', 'Ë' => 'E',
      			  'Ì' => 'I', 'Í' => 'I', 'Î' => 'I', 'Ï' => 'I', 'Ð' => 'D', 'Ñ' => 'N',
				  'Ò' => 'O', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ö' => 'O', 'Ø' => 'O',
				  'Ù' => 'U', 'Ú' => 'U', 'Û' => 'U', 'Ü' => 'U', 'Ý' => 'Y', 'Ŕ' => 'R',
				  'Þ' => 's', 'ß' => 'B', 'à' => 'a', 'á' => 'a', 'â' => 'a', 'ã' => 'a',
				  'ä' => 'a', 'å' => 'a', 'æ' => 'a', 'ç' => 'c', 'è' => 'e', 'é' => 'e',
				  'ê' => 'e', 'ë' => 'e', 'ì' => 'i', 'í' => 'i', 'î' => 'i', 'ï' => 'i',
				  'ð' => 'o', 'ñ' => 'n', 'ò' => 'o', 'ó' => 'o', 'ô' => 'o', 'õ' => 'o',
				  'ö' => 'o', 'ø' => 'o', 'ù' => 'u', 'ú' => 'u', 'û' => 'u', 'ý' => 'y',
				  'þ' => 'b', 'ÿ' => 'y', 'ŕ' => 'r', ',' => ' ', '-' => ' ', 'º' => ' ', '&' => ' ', '°' => ' ','ª' => ' ','?' => ' ','—' => ' ','_' => ' '
    			)
			  );
		return $formattedXML;
	}
	
	private function exceptionMessage($msg){
		$msg = str_replace('An error occurred while trying to communication via soap', 'Erro ao tentar comunicação com a SEFAZ, ', $msg);
		$msg = str_replace('Operation timed out after', 'operação finalizada após ', $msg);
		$msg = str_replace('with 0 bytes received', '!', $msg);
		$msg = str_replace('Certificate invalid from', 'Certificado inválido. Data do certificado', $msg);
		$msg = str_replace('Unable to read certificate', 'Erro ao ler certificado', $msg);
		$msg = str_replace('Unable to open certificate', 'Erro ao abrir certificado', $msg);
		$msg = str_replace('An unexpected error has occurred when sign a content', 'Ocorreu um erro inesperado ao assinar um conteúdo', $msg);
		$msg = str_replace('An error has occurred when get private key', 'Ocorreu um erro ao obter chave privada', $msg);
		$msg = str_replace('An error has occurred when verify signature', 'Ocorreu um erro ao verificar a assinatura', $msg);
		$msg = str_replace('get follow error', 'seguinte(s) erro(s) encontrado(s)', $msg);
		return $msg;
	}
	
	public function getDuplicidades($row) {
        $parametros = array(
            ':CDFILIAL' => implode('_', $row['CDFILIAL']),
            ':DATAINI'    => $row['DATAINI'],
            ':DATAFIN'    => $row['DATAFIN']
        );
        $result = $this->executarQuery(TransmissaoNFCE::ADQGETDUPLICIDADES, $parametros);
        if($result->error()) {
            throw new \Exception("Erro ao buscar duplicidades: ".$result->getMessage(), 1);
        } 
        
        return $result;
	}
	
	public function atualizarXMLBanco($xml, $nota, $qrCode = null) {
        
        $xml = str_replace('<?xml version="1.0" encoding="UTF-8"?>', '', $xml);
        $xml = str_replace('<?xml version="1.0"?>', '', $xml);
        
        $parametros = array(
            ':DSARQXMLNFCE' => $xml,
            ':NRORG'        => $nota['NRORG'],
            ':NRSEQVENDA'   => $nota['NRSEQVENDA'],
            ':CDCAIXA'      => $nota['CDCAIXA'],
            ':CDFILIAL'     => $nota['CDFILIAL'],
            ':DSQRCODENFCE' => $qrCode
        );
        $this->updateVendaNFCE($parametros, $xml);
    }
    
    public function atualizaNFCE($duplicidades, $rows) {
        $time_start = microtime(true);
        $regerouChave = false;
        
        foreach ($rows as $nota) {
            
            $parametros['CDFILIAL']     = $nota['CDFILIAL'];
            $parametros['CDCAIXA']      = $nota['CDCAIXA'];
            $parametros['NRSEQVENDA']   = $nota['NRSEQVENDA'];
            
            //Verifica se nota encontra-se no array das notas validas
            $item = array_filter(
                $duplicidades,
                function ($e) use ($nota) {
                    return $e['CDFILIAL'] == $nota['CDFILIAL'] && $e['CDCAIXA'] == $nota['CDCAIXA'] && $e['NRSEQVENDA'] == $nota['NRSEQVENDA'];
                }
            );
            
            if (!empty($item) && sizeof($item) == 1) { 
            	
                $item = array_shift($item);
                
                //seta informacoes iniciais no log
				$this->getLogProcessNota($nota);
				
				//verifica status NFC-e na base de dados
				$this->getStatusNota($nota);
			
				//le certificado
				$certificado = $this->getCertificado($nota);
				
				//config certificado
				$this->nfeTools = $this->getNfeTools($nota, $certificado);
                
                try{
                	$xml = simplexml_load_string($item['DSARQXMLNFCE']);
                	
                }catch(\Exception $e){
                	TransmissaoNfceNewController::escreverLog("XML gravado na base de dados não é válido");
                	return;
                }
                
                if(!empty($xml)){
                	if($this->corrigeDuplicidade($item, $regerouChave, $parametros, $nota)){
                		//altera os dados gravados no xml com informacoes de protocolo no xml
						///quando nota aprovada o XML tem que receber as informacoes de protocolo para salvar certo
						$this->createProtocoloXML($parametros);	
                	}
                }
            }
        }
        $time_end = microtime(true);
        $time = $time_end - $time_start;
        TransmissaoNfceNewController::escreverLog("Tempo gasto: ".number_format($time, 4, '.', '')."s");
        TransmissaoNfceNewController::escreverLog("");
    }
    
    private function corrigeDuplicidade($item, $regerouChave, $parametros, $nota, $xml = null){
    	
    	$retorno = true;
    	$getNumeroDuplicidade = $this->getNumeroDuplicidade($item['DSOBSSTATUSNFCE']);
    	
    	//valida o retorno do codigo
    	if($item['CSTAT'] == '539'){
    		$chaveAcesso = $getNumeroDuplicidade['chave'];
    		//as vezes a chave de acesso nao vem no retorno da sefaz, por isso deve-se recalcular chave
            if(empty($chaveAcesso) || $chaveAcesso == null || $chaveAcesso == ''){
            	TransmissaoNfceNewController::escreverLog("Chave de acesso não retornada pela SEFAZ, recalculando!");
            	$chaveAcesso = $this->recalculaChave($nota['NRACESSONFCE']);
            }
    		
            $parametros['NRACESSONFCE'] = $chaveAcesso;
            $regerouChave = true;
    	}
    	if($item['CSTAT'] == '204' || $item['CSTAT'] == '52' || $item['CSTAT'] == '51' || $item['CSTAT'] == '613'){
            $parametros['NRACESSONFCE'] = $item['NRACESSONFCE'];
            $chaveAcesso = $item['NRACESSONFCE'];
            
    	}
    	$parametros['NRRECIBONFCE'] = (isset($getNumeroDuplicidade['recibo'])) ? $getNumeroDuplicidade['recibo'] : $item['NRRECIBONFCE'];
    	
    	//devera consultar chave de acesso, pode nao achar por consulta ser antiga, ou estado nao ter opcao, e atualiza status, e caso necessario XML
        $retornoXML = $this->callConsultaNFCe($parametros['NRACESSONFCE'], $nota['IDTPAMBNFCE']);
		if($retornoXML['cStat'] == 100 || $retornoXML['cStat'] == 101 || $retornoXML['cStat'] == 110 || $retornoXML['cStat'] == 150){
			$validaConsulta = $this->validaRetornoConsulta($nota, $retornoXML, $chaveAcesso,$regerouChave);			
			if($validaConsulta['error']){
             	TransmissaoNfceNewController::escreverLog("Não foi possível validar retorno da consulta SEFAZ, atualização não realizada!");
             	//foi aprovada mas por algum motivo nao retornou certo ao validar retorno de consulta
             	//$arraySucessos = $this->getSucessoConsulta();
	            //aprovada, entao colocada como status de aprovada
	            //$parametros['DSOBSSTATUSNFCE'] = $retornoXML['cStat'].' - '.$arraySucessos[$retornoXML['cStat']];
	            //$this->runQuery(true, TransmissaoNFCE::ACMDUPLICIDADES, $parametros);
            }
            TransmissaoNfceNewController::escreverLog("Atualizando status da Nota!");
            if($nota['DSARQXMLNFCE'] == null || empty($nota['DSARQXMLNFCE']) || $nota['DSARQXMLNFCE'] == ''){
            	TransmissaoNfceNewController::escreverLog("Não há XML gravado no banco, gravando XML gerado");
            	//regera XML pq difere do que esta no banco
	            $nota['NRACESSONFCE'] = $parametros['NRACESSONFCE'];
	            $nota['DSARQXMLNFCE'] = $xml;
	            $xmlRegerado =  $this->regerarXMLValido($parametros['NRACESSONFCE'], null, $nota, null);
	            if (!empty($xmlRegerado)) {
	            	$xmlAssinado = $this->signNFCe($xmlRegerado);
	            	if($xmlAssinado){
	            		$getQrCode = $this->getQrCode($xmlAssinado);
	            	}
	            	TransmissaoNfceNewController::escreverLog("Atualizando dados da venda com nova chave de acesso.");
	            	$this->atualizarQRCODEBanco($xmlAssinado, $nota, $getQrCode);
	            }else{
			        TransmissaoNfceNewController::escreverLog("XML gravado no banco inválido!");
			    }
            }
            return $retorno;
		}else{
			TransmissaoNfceNewController::escreverLog("Ao consultar chave não consta como aprovada na SEFAZ!");
			if(isset($retornoXML['xMotivo']) && isset($retornoXML['cStat'])){
				TransmissaoNfceNewController::escreverLog($retornoXML['cStat'].' - '.$retornoXML['xMotivo']);
			}
			$retorno = false;
			return $retorno;
		}
    }
	
	private function getNumeroDuplicidade($texto){
        $result = array(
            "chave" => '',
            "recibo" => ''
        );
        $cortado = array();
        preg_match('/\[(.*?)\]/', $texto, $cortado);
        
        if(!empty($cortado)){
            $posChave = strpos($cortado[1], 'chNFe:');
            $posRecibo = strpos($cortado[1], 'nRec:');
            if (!($posChave === false) && ($posRecibo === false)) {
                $tamanho = strlen('chNFe:');
                $result['chave'] = substr($cortado[1], $tamanho, 44);
            }else if(strlen($cortado[1]) == 44){
                $result['chave'] = $cortado[1];
            }
            if (!($posRecibo === false)) {
                $tamanho = strlen('nRec:');
                $result['recibo'] = substr($cortado[1], $tamanho, 15);
            }else if(strlen($cortado[1]) == 15){
                $result['recibo'] = $cortado[1];
            }
        }
        return $result;
    }
    
    /*
    Funcao para consulta NFCe XML pelo Recibo
    */
    private function callConsultaReciboNFCe($recibo, $ambiente){
        try {
		    $consulta = $this->nfeTools->sefazConsultaRecibo($recibo, $ambiente);
		    if(!empty($consulta)){
				if(!empty($consulta)){
					$stNfeTools = new Standardize($consulta);
					$resultXMLArray = $stNfeTools->toArray();
					return $resultXMLArray;
				}else{
					TransmissaoNfceNewController::escreverLog("Não foi possível recuperar informações ao tentar consultar o recibo da nota!");
					//throw new \Exception("Não foi possível recuperar informações ao tentar consultar status da nota!");
				}
		    }else{
		    	TransmissaoNfceNewController::escreverLog("Não houve retorno ao tentar realizar a consulta da nota!");
		    	//throw new \Exception("Não houve retorno ao tentar realizar a consulta da nota!");
		    }
			
		} catch (\Exception $e) {
			//throw new \Exception("Erro ao tentar realizar a consulta da nota!");
			TransmissaoNfceNewController::escreverLog("Erro ao tentar realizar a consulta da nota! Message: ".$this->exceptionMessage($e->getMessage()));
		}
		return;
    }
    
	public function trataRetornoMensagemSefaz($params, $xml){
		
		//tratando mensagens em relacao aos itens retornados
        $stNfeToolsXMLEnvio = new Standardize($xml);
		$arrayXml = $stNfeToolsXMLEnvio->toArray();
		
		$posInicial = strripos($params['DSOBSSTATUSNFCE'], '[nItem:');
		$posFinal = strripos($params['DSOBSSTATUSNFCE'], ']');
		$obsProduto = '';
		if ($posInicial === false) {
		    return $params;
		} else {
		    //pega o item que deve ser substituido no informacao de observacao
		    $det = $arrayXml['infNFe']['det'];
		    $nItem = substr($params['DSOBSSTATUSNFCE'], ($posInicial + 7), (($posInicial + 7) - $posFinal));
		    if(!empty($det)){
		    	if (!array_key_exists("attributes", $det)) {
				    foreach($det as $key => $value){
			    		if(isset($value['attributes'])){
				    		if($value['attributes']['nItem'] == $nItem ){
					    		$obsProduto .= '('.$value['prod']['cProd'].' - '.$value['prod']['xProd'].')';
					    	}
				    	}
				    }
		    	}else{
		    		if(isset($det['attributes'])){
			    		if($det['attributes']['nItem'] == $nItem ){
				    		$obsProduto .= '('.$det['prod']['cProd'].' - '.$det['prod']['xProd'].')';
				    	}
			    	}
		    	}
			    //$params['DSOBSSTATUSNFCE'] = str_replace("nItem:".$nItem, $obsProduto,$params['DSOBSSTATUSNFCE']);
			    $params['DSOBSSTATUSNFCE'] = $params['DSOBSSTATUSNFCE'].' '.$obsProduto;
		    }
		    return $params;
		}
	}    
	
	private function createProtocoloXML($params){
		
		TransmissaoNfceNewController::escreverLog("Buscando dados de protocolo para inserir no XML!");
		//busca dados de salvos no banco quando aprovado
		$dadosProtocolo = $this->executarQuery(TransmissaoNFCE::SELECT_VENDA_ARQXMLNFCESAT_PROT, $params);
		if($dadosProtocolo->error()) {
            throw new \Exception("Erro! ".$dadosProtocolo->getMessage(), 1);
        } else {
        	
        	if(!empty($dadosProtocolo->getResult())){
        		$dadosProtocolo = $dadosProtocolo->getResult()[0];
	        	if(!empty($dadosProtocolo['DSARQXMLNFCE']) && $dadosProtocolo['IDSTATUSNFCE'] == 'A') {
	        		
	        		$verificaProtcXML = strpos($dadosProtocolo['DSARQXMLNFCE'], 'nfeProc');
	        		if($verificaProtcXML === false){
	        			
	        			$xmlRetorno = simplexml_load_string($dadosProtocolo['DSARQXMLNFCE']);  
	        			if(!empty($xmlRetorno)){
							$arrayData = $this->xmlToArray($xmlRetorno);
							if(isset($arrayData['NFe'])){
								if(!empty($dadosProtocolo['DTHRPROTOCONFCE'])){
									$date = DateTime::createFromFormat('d/m/Y H:i:s', $dadosProtocolo['DTHRPROTOCONFCE']);
				                    $dataFormatada1 = $date->format('Y-m-d');
				                    $dataFormatada2 = $date->format('H:i:s');
				                    $stFusoHr = '-03:00';
				                    $dhRecbto = $dataFormatada1."T".$dataFormatada2.$stFusoHr;    
						        }else{
						            $dhRecbto = $arrayData['NFe']['infNFe']['ide']['dhEmi'];    
						        }
						            
						        $stProtNfe = '<protNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="'.$arrayData['NFe']['infNFe']['@versao'].'">';
						        $stProtNfe .= '<infProt Id="ID'.$dadosProtocolo['NRPROTOCOLONFCE'].'">';
						        $stProtNfe .= '<tpAmb>'.$dadosProtocolo['IDTPAMBNFCE'].'</tpAmb>';
						        $stProtNfe .= '<verAplic>'.$arrayData['NFe']['infNFe']['@versao'].'</verAplic>';
						        $stProtNfe .= '<chNFe>'.$dadosProtocolo['NRACESSONFCE'].'</chNFe>';
						        $stProtNfe .= '<dhRecbto>'.$dhRecbto.'</dhRecbto>';
						        $stProtNfe .= '<nProt>'.$dadosProtocolo['NRPROTOCOLONFCE'].'</nProt>';
						        
						        $stProtNfe .= '<digVal>'.$arrayData['NFe']['Signature']['SignedInfo']['Reference']['DigestValue'].'</digVal>';
						        
						       
						        $stProtNfe .= '<cStat>'.$dadosProtocolo['CODSTATUS'].'</cStat>';
						        $stProtNfe .= '<xMotivo>'.$dadosProtocolo['DSOBSSTATUSNFCE2'].'</xMotivo>';
						        $stProtNfe .= '</infProt></protNFe>';
						        
						        $stTextoXML = '';
						        $stTextoXML = '<?xml version="1.0" encoding="UTF-8"?><nfeProc versao="'.$arrayData['NFe']['infNFe']['@versao'].'" xmlns="http://www.portalfiscal.inf.br/nfe">';
						        $stTextoXML = $stTextoXML.$dadosProtocolo['DSARQXMLNFCE'].$stProtNfe;
						        $stTextoXML = $stTextoXML.'</nfeProc>';	
						        
						        if(!empty($stTextoXML)){
						        	TransmissaoNfceNewController::escreverLog("Salvando dados de protocolo no XML!");
						        	//salva xml no banco de dados
						        	$parametrosXML = array(
							            ':NRSEQVENDA'   => $params['NRSEQVENDA'],
							            ':CDCAIXA'      => $params['CDCAIXA'],
							            ':CDFILIAL'     => $params['CDFILIAL'],
							            ':DSARQXMLNFCE' => $stTextoXML
							        );
						        	$this->runQuery(true, TransmissaoNFCE::UPDATE_ARQXMLNFCESAT_XML, $parametrosXML);   
						        }	
							}
	        			}
	        		}
	        	}
        	}else{
        		TransmissaoNfceNewController::escreverLog("Sem retorno na consulta dos dados salvos para atualizar XML com protocolo!");
        	}
        }

        return;
	}
	
	private function createProtocoloXMLCANC($params){
		
		TransmissaoNfceNewController::escreverLog("Buscando dados de protocolo para inserir no XML de Cancelamento!");
		//busca dados de salvos no banco quando aprovado
		$dadosProtocolo = $this->executarQuery(TransmissaoNFCE::SELECT_VENDA_ARQXMLNFCESAT_PROT_CAN, $params);
		if($dadosProtocolo->error()) {
            throw new \Exception("Erro! ".$dadosProtocolo->getMessage(), 1);
        } else {
        	
        	if(!empty($dadosProtocolo->getResult())){
        		$dadosProtocolo = $dadosProtocolo->getResult()[0];
	        	if(!empty($dadosProtocolo['DSARQXMLCANCNFCE']) && $dadosProtocolo['IDSTATUSNFCE'] == 'C') {
	        		
	        		$xmlCancSalvo = $dadosProtocolo['DSARQXMLCANCNFCE'];
					$xmlCancelamentoSalvo = simplexml_load_string($xmlCancSalvo);
					
					if(!empty($xmlCancelamentoSalvo)){
					
						$xmlCanc = '<procEventoNFe></procEventoNFe>';
						$xmlCancelamento = simplexml_load_string($xmlCanc);
						
						if(!empty($xmlCancelamento)){
						
							$xmlCancelamento->addAttribute('versao', '1.00');
							$xmlCancelamento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
							
							if(!empty($dadosProtocolo['DTHRPROTOCOCANC'])){
								$dateCan = DateTime::createFromFormat('d/m/Y H:i:s', $dadosProtocolo['DTHRPROTOCOCANC']);
								$dataFormatada1Can = $dateCan->format('Y-m-d');
								$dataFormatada2Can = $dateCan->format('H:i:s');
								$stFusoHrCan = '-03:00';
								$dhRecbtoCan = $dataFormatada1Can."T".$dataFormatada2Can.$stFusoHrCan;    
							}else{
								$dataFormatada1Can = date('Y-m-d');
								$dataFormatada2Can = date('H:i:s');
								$stFusoHrCan = '-03:00';
								$dhRecbtoCan = $dataFormatada1Can."T".$dataFormatada2Can.$stFusoHrCan;    
							}
							
							if(isset($xmlCancelamentoSalvo->evento)){
								$evento = $xmlCancelamento->addChild('evento');
								$evento->addAttribute('versao', '1.00');
								$evento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
								if($xmlCancelamentoSalvo->evento->infEvento){
									$infEvento = $evento->addChild('infEvento');
									
									$arrayID = $xmlCancelamentoSalvo->evento->infEvento->attributes()->Id;
									foreach((array)$arrayID as $key => $val){
										$id = $val;
									}
									
									$infEvento->addAttribute('Id', $id);
									$infEvento->addChild('cOrgao',$xmlCancelamentoSalvo->evento->infEvento->cOrgao);
									$infEvento->addChild('tpAmb',$xmlCancelamentoSalvo->evento->infEvento->tpAmb);
									$infEvento->addChild('CNPJ',$xmlCancelamentoSalvo->evento->infEvento->CNPJ);
									$infEvento->addChild('chNFe',$xmlCancelamentoSalvo->evento->infEvento->chNFe);
									$infEvento->addChild('dhEvento',$xmlCancelamentoSalvo->evento->infEvento->dhEvento);
									$infEvento->addChild('tpEvento',$xmlCancelamentoSalvo->evento->infEvento->tpEvento);
									$infEvento->addChild('nSeqEvento',$xmlCancelamentoSalvo->evento->infEvento->nSeqEvento);
									$infEvento->addChild('verEvento',$xmlCancelamentoSalvo->evento->infEvento->verEvento);
									
									if($xmlCancelamentoSalvo->evento->infEvento->detEvento){
										$detEvento = $infEvento->addChild('detEvento');
										$arrayID = $xmlCancelamentoSalvo->evento->infEvento->detEvento->attributes()->versao;
										foreach((array)$arrayID as $key => $val2){
											$versao = $val2;
										}
										$detEvento->addAttribute('versao', $versao);
										$detEvento->addChild('descEvento',$xmlCancelamentoSalvo->evento->infEvento->detEvento->descEvento);
										$detEvento->addChild('nProt',$xmlCancelamentoSalvo->evento->infEvento->detEvento->nProt);
										$detEvento->addChild('xJust',$xmlCancelamentoSalvo->evento->infEvento->detEvento->xJust);    
									}else{
										$detEvento = $infEvento->addChild('detEvento');
										$detEvento->addAttribute('versao', '1.00');
										$detEvento->addChild('descEvento','Cancelamento');
										$detEvento->addChild('nProt',$dadosProtocolo['NRPROTOCOLONFCE']);
										$detEvento->addChild('xJust',$dadosProtocolo['DSRAZAOCANCNFCE']);    
									}
								}else{
									$infEvento = $evento->addChild('infEvento');
									$infEvento->addAttribute('Id', 'ID'.'110111'.$dadosProtocolo['NRACESSONFCE'].'01');
									$infEvento->addChild('cOrgao',$dadosProtocolo['CDESTADOIBGE']);
									$infEvento->addChild('tpAmb',$dadosProtocolo['IDTPAMBNFCE']);
									$infEvento->addChild('CNPJ',$dadosProtocolo['NRINSJURIEST']);
									$infEvento->addChild('chNFe',$dadosProtocolo['NRACESSONFCE']);
									
									$infEvento->addChild('dhEvento',$dhRecbtoCan);
									$infEvento->addChild('tpEvento','110111');
									$infEvento->addChild('nSeqEvento','1');
									$infEvento->addChild('verEvento','1.00');
									
									$detEvento = $infEvento->addChild('detEvento');
									$detEvento->addAttribute('versao', '1.00');
									$detEvento->addChild('descEvento','Cancelamento');
									$detEvento->addChild('nProt',$dadosProtocolo['NRPROTOCOLONFCE']);
									$detEvento->addChild('xJust',$dadosProtocolo['DSRAZAOCANCNFCE']);
								}
							}else{
								$evento = $xmlCancelamento->addChild('evento');
								$evento->addAttribute('versao', '1.00');
								$evento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
								
								$infEvento = $evento->addChild('infEvento');
								$infEvento->addAttribute('Id', 'ID'.'110111'.$dadosProtocolo['NRACESSONFCE'].'01');
								$infEvento->addChild('cOrgao',$dadosProtocolo['CDESTADOIBGE']);
								$infEvento->addChild('tpAmb',$dadosProtocolo['IDTPAMBNFCE']);
								$infEvento->addChild('CNPJ',$dadosProtocolo['NRINSJURIEST']);
								$infEvento->addChild('chNFe',$dadosProtocolo['NRACESSONFCE']);
								
								$infEvento->addChild('dhEvento',$dhRecbtoCan);
								$infEvento->addChild('tpEvento','110111');
								$infEvento->addChild('nSeqEvento','1');
								$infEvento->addChild('verEvento','1.00');
								
								$detEvento = $infEvento->addChild('detEvento');
								$detEvento->addAttribute('versao', '1.00');
								$detEvento->addChild('descEvento','Cancelamento');
								$detEvento->addChild('nProt',$dadosProtocolo['NRPROTOCOLONFCE']);
								$detEvento->addChild('xJust',$dadosProtocolo['DSRAZAOCANCNFCE']);
							}
							
							if(isset($xmlCancelamentoSalvo->retEnvEvento)){
								if(isset($xmlCancelamentoSalvo->retEnvEvento->retEvento)){
									$retEvento = $xmlCancelamento->addChild('retEvento');
									$retEvento->addAttribute('versao', '1.00');
									$retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
									if(isset($xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento)){
										$infEventoRet = $retEvento->addChild('infEvento');
										$infEventoRet->addChild('tpAmb',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->tpAmb);
										$infEventoRet->addChild('verAplic',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->verAplic);
										$infEventoRet->addChild('cOrgao',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->cOrgao);
										$infEventoRet->addChild('cStat',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->cStat);
										$infEventoRet->addChild('xMotivo',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->xMotivo);
										$infEventoRet->addChild('chNFe',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->chNFe); 
										$infEventoRet->addChild('tpEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->tpEvento);
										$infEventoRet->addChild('xEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->xEvento);
										$infEventoRet->addChild('nSeqEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->nSeqEvento);
										$infEventoRet->addChild('dhRegEvento',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->dhRegEvento);
										$infEventoRet->addChild('nProt',$xmlCancelamentoSalvo->retEnvEvento->retEvento->infEvento->nProt);
									}else{
										$infEventoRet = $retEvento->addChild('infEvento');
										$infEventoRet->addChild('tpAmb',$value['IDTPAMBNFCE']);
										$infEventoRet->addChild('verAplic','SEFAZ');
										$infEventoRet->addChild('cOrgao',$dadosProtocolo['CDESTADOIBGE']);
										$infEventoRet->addChild('cStat','135');
										$infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
										$infEventoRet->addChild('chNFe',$dadosProtocolo['NRACESSONFCE']);
										$infEventoRet->addChild('tpEvento','110111');
										$infEventoRet->addChild('xEvento','Cancelamento');
										$infEventoRet->addChild('nSeqEvento','1');
										$infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
										$infEventoRet->addChild('nProt',$dadosProtocolo['NRPROTOCOLOCANC']);
									}
								}else{
									$retEvento = $xmlCancelamento->addChild('retEvento');
									$retEvento->addAttribute('versao', '1.00');
									$retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
									$infEventoRet = $retEvento->addChild('infEvento');
									$infEventoRet->addChild('tpAmb',$dadosProtocolo['IDTPAMBNFCE']);
									$infEventoRet->addChild('verAplic','SEFAZ');
									$infEventoRet->addChild('cOrgao',$dadosProtocolo['CDESTADOIBGE']);
									$infEventoRet->addChild('cStat','135');
									$infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
									$infEventoRet->addChild('chNFe',$dadosProtocolo['NRACESSONFCE']);
									$infEventoRet->addChild('tpEvento','110111');
									$infEventoRet->addChild('xEvento','Cancelamento');
									$infEventoRet->addChild('nSeqEvento','1');
									$infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
									$infEventoRet->addChild('nProt',$dadosProtocolo['NRPROTOCOLOCANC']);
								}
							}else{
								$retEvento = $xmlCancelamento->addChild('retEvento');
								$retEvento->addAttribute('versao', '1.00');
								$retEvento->addAttribute('xmlns', 'http://www.portalfiscal.inf.br/nfe');
								$infEventoRet = $retEvento->addChild('infEvento');
								$infEventoRet->addChild('tpAmb',$dadosProtocolo['IDTPAMBNFCE']);
								$infEventoRet->addChild('verAplic','SEFAZ');
								$infEventoRet->addChild('cOrgao',$dadosProtocolo['CDESTADOIBGE']);
								$infEventoRet->addChild('cStat','135');
								$infEventoRet->addChild('xMotivo','Evento registrado e vinculado a NF-e');
								$infEventoRet->addChild('chNFe',$dadosProtocolo['NRACESSONFCE']);
								$infEventoRet->addChild('tpEvento','110111');
								$infEventoRet->addChild('xEvento','Cancelamento');
								$infEventoRet->addChild('nSeqEvento','1');
								$infEventoRet->addChild('dhRegEvento',$dhRecbtoCan);
								$infEventoRet->addChild('nProt',$dadosProtocolo['NRPROTOCOLOCANC']);
							}
							
							$XCanc = $xmlCancelamento->asXML();
							
							if(!empty($XCanc)){
					        	TransmissaoNfceNewController::escreverLog("Salvando dados de protocolo no XML de Cancelamento!");
					        	//salva xml no banco de dados
					        	$parametrosXML = array(
						            ':NRSEQVENDA'   => $params['NRSEQVENDA'],
						            ':CDCAIXA'      => $params['CDCAIXA'],
						            ':CDFILIAL'     => $params['CDFILIAL'],
						            ':DSARQXMLCANCNFCE' => $XCanc
						        );
					        	$this->runQuery(true, TransmissaoNFCE::UPDATE_ARQXMLNFCESAT_STATUS_NFCE_CAN_SB, $parametrosXML);   
					        }
						}
					}
	        	}
        	}else{
        		TransmissaoNfceNewController::escreverLog("Sem retorno na consulta dos dados salvos para atualizar XML com protocolo!");
        	}
        }
        return;
	}
	
	public function regerarChavesNFCe($nota){
		
		//seta informacoes iniciais no log
		$this->getLogProcessNota($nota);
		
		try {
    		
    		//verificar o ambiente se e online ou offline
    		$DTEMISSAOVENDA = new \DateTime($nota['DTEMISSAONFCE']);
    		
    		if(!empty($nota['NRACESSONFCE'])){
    			$tipoEmissao = substr($nota['NRACESSONFCE'], 34, 1);
    		}else{
    			$tipoEmissao = self::AMB_EMISSAO_ONLINE;
    			$nota['NRACESSONFCE'] = 'S/CHAVE';
    		}
    		
    		$tipoEmissao = substr($nota['NRACESSONFCE'], 34, 1);
    		$nvChaveAcesso = $this->gerarChaveAcessoNFCeNew($nota, $nota['P_CDSERIENFCE'],  $nota['P_NRNOTAFISCALCE'], $tipoEmissao, $DTEMISSAOVENDA, $nota['NRLANCTONFCE']);
    		TransmissaoNfceNewController::escreverLog("Alterando chave de acesso de ".$nota['NRACESSONFCE']." para ".$nvChaveAcesso);
    		return $this->updateDadosChaveAcessoNFCe($nota, $nvChaveAcesso);
    		
    		
		} catch (\Exception $e) {
			throw new \Exception("Erro ao tentar gerar chave de acesso de venda!".$e->getMessage());
		}
	}
	
	public function updateDadosChaveAcessoNFCe($nota, $nvChaveAcesso){
		
		TransmissaoNfceNewController::escreverLog("Atualizando dados da venda com novas informações!");
    	$result = array(
    		'error' => true,
    		'message' => ''
    	);
    	
    	try {
    		
             $parametros['CDFILIAL'] = $nota['CDFILIAL'];
             $parametros['CDCAIXA'] = $nota['CDCAIXA'];
             $parametros['NRSEQVENDA'] = $nota['NRSEQVENDA'];
             $parametros['NRACESSONFCE'] = $nvChaveAcesso;
             $parametros['NRNOTAFISCALCE'] = $nota['P_NRNOTAFISCALCE'];
             $parametros['CDSERIENFCE'] = $nota['P_CDSERIENFCE'];
             $parametros['NRINSJURIEST'] = $nota['NRINSJURFILI'];
             $parametros['IDTPAMBNFCE'] = $nota['IDAMBTRABNFCE'];
             $parametros['IDSTATUSNFCE'] = 'P';
            
             $this->runQuery(true, TransmissaoNFCE::UPDATECHAVEACESSO, $parametros);
        	 $result['error'] = false;
        	
    	} catch (\Exception $e) {
    		TransmissaoNfceNewController::escreverLog("Não foi possível atualizar informações da venda!");
    		$result['message'] = $e->getMessage();
    	}
    	return $result;
    }
    
    public function atualizaNovoCodigo($row){
		try {
			
			$row['P_CDSERIENFCE'] = str_pad($row['P_NRNOTAFISCALCE'], 3, 0, STR_PAD_LEFT);
    		$parametros[':P_CDCONTADOR'] = 'NFCE_'.$row['CDFILIAL'].$row['P_CDSERIENFCE'].'_AMB'.$row['IDAMBTRABNFCE'];
    		$result = $this->executarQuery(TransmissaoNFCE::SELECTNOVOCODIGO, $parametros);
            if($result->error()) {
                $this->getLogOperacao(2, $result->getMessage(), 'E');
            } else {
                $notaRegerarChave = $result->getResult();
                $parametros[':P_NRSEQUENCIAL'] = str_pad($row['P_NRNOTAFISCALCE'], 20, 0, STR_PAD_LEFT);
                if(!empty($notaRegerarChave)){
                	//se nao for vazio update
                	$this->runQuery(true, TransmissaoNFCE::UPDATENOVOCODIGO, $parametros);
                }else if(empty($notaRegerarChave)){
                	//se for vazio insert
                	$this->runQuery(true, TransmissaoNFCE::INSERTNOVOCODIGO, $parametros);
                }
                
            }
		} catch (\Exception $e) {
			throw new \Exception("Erro ao tentar atualizar NOVOCODIGO!".$e->getMessage());
		}
	}
	
	
}